package jastex;

import java.util.Vector;
import java.awt.*;

import jastex.*;

import jastex.old.obj.JasObject;


public class GastexGlobalProperties {
    //Vector listOfObject=new Vector();
    // List of all object which belongs to this class
    public String propName;

    /** Properties common to node and edge */
    public double plinegray;  // decimal number between 0 and 1
                      //   gray level used to draw lines. 0=black, 1=white.
    
    public String plinecolor;// color used to draw lines
                      //   The color name should be defined in dvipsnam.def
                      //   and one should include usepackage[usenames]{color}.
                      //   This is to avoid the trouble of defining our own 
                      //   colors.
                      //   The drawback is that it is not possible to define and
                      //   use other colors.
                      //   It should not be very restrictive since plenty of
                      //   colours are defined in dvipsnam.def.
                      //   It should not be difficult to add the possibility
                      //   of using new colors if needed. 
    
    
    public double plinewidth;   // width in unitlegth of lines
    public float[] pdash_length;// Set the dash pattern used for drawing paths.
    public float   pdash_empty;
    //   Set the dash pattern used for drawing postscript paths.
    //   The numbers in the list indicate alternatively lengths
    //   in unitlength of dashes and lengths in unitlength of spaces. 
    //   The list of lenghts is used circularly.
    //   offset allows to start the pattern at some distance from its beginning.
    //   Here are some examples:
    //      dash={}{0} % continuous path
    //      dash={1.5}0 % dashs of length 1.5 and empty spaces of length 1.5
    //      dash={0.2 0.5}0 % looks like a sequence of dots
    //      dash={4 1 1 1}0 % alternation of long and short dashs
    //      dash={1.5}{1.5} % we start with the empty space and not the dash
    //      dash={4}{2} % we start in the middle of the first dash

    /** Node Properties */
    public double pNw;          // width in unitlength of oval nodes
    public double pNh;          // height in unitlength of oval nodes
    public double pNmr;         // maximal radius in unitlength of oval nodes
                      //   The actual radius will be min(Nw/2,Nh/2,Nmr)
                      //   Nmr=0 yields a rectangle node

    public int pNadjust;     // any combination of the number 0,1,2 by sum
                      //   1 = width, 2 = height, 0 = no adjust
    public double pNadjustdist; // distance in unitlength between the label box 
                      //   and the node frame when using Nadjust
    public boolean pNframe;  // boolean flag defining whether nodes are framed or not
    public boolean pNfill;   // boolean flag defining whether nodes are filled or not
                      //   Automatically set to true by the parameters
                      //   fillgray and fillcolor

   // boolean flag defining whether node labels are external or not
    public boolean pExtNL;
    public double pNLangle;     // direction in degree of node labels
     // The meaning depends on the ExtNL flag.
    //   If the flag is set to "n" then NLdist defines the distance
    //   in unitlength between the node center and the label center.
    //   If the flag is set to "y" then NLdist defines the distance
    //   in unitlength between the node frame and the label box.
    public double pNLdist;
    public int pNmarks;      // any combination of the number 0,1,2,4 by sum
                      //    1 : initial mark on nodes
                      //    2 : final mark on nodes
                      //    4 : repeated mark on nodes
                      //    0 : no mark on nodes
    public double pilength;     // length in unitlength of arrows marking initial nodes
    public double piangle;      // direction in degree of arrows marking initial nodes
    public double pflength;     // length in unitlength of arrows marking final nodes
    public double pfangle;      // direction in degree of arrows marking final nodes
       // distance in unitlength between the lines of repeated nodes
    // a positive value draw the second line inside the normal one
    // a negative value draw the second line outside the normal one
    // but the edges are still connected to the normal line
    public double prdist;

    public double pfillgray;  // decimal number between 0 and 1
                      //   gray level used to fill nodes. 0=black, 1=white.
    public String pfillcolor;// color used to fill nodes.
                      // See remarks on plineoclor in JasObject.java.

    /** Edge Properties */
    /** For Loop */
    public double ploopdiam;    // diameters in unitlength of loops
    public double ploopangle;   // direction in degree of loops
    public boolean ploopCW;  // boolean flag defining whether 
                      //   loops are in clockwise direction or not

    /** Curved Edge */
    // depth in unitlength of curved edges between two nodes
    //   The absolute value of curvedepth defines 
    //   the distance between the middle of the curved edge 
    //   and the center of the line joining the two nodes 
    //   With a positive/negative value the curved edge
    // is on the left/right of the line joining the two nodes.
        public double pcurvedepth;
    /** ArrowHead */
    public int pAHnb;        // number of arrowhead(s) at the end of edges
    public double pAHdist;      // distance in unitlength between two arrowheads
     // angle in degree between the edge and the arrowhead side
    public double pAHangle;
    public double pAHLength;    // Length in unitlength of the arrowhead side
     // length in unitlength defining the shape of the arrowhead
    public double pAHlength;

    /** Starting point x offset */
    public double psxo;
    /** Starting point y offset */
    public double psyo;

    /** Ending point x offset */
    public double pexo;
    /** Ending point y offset */
    public double peyo;

    
    /** Label */
    public double pELside;      // label on the left(0) or right(1) side of the edge
    public double pELpos;       // position of the label along the edge
                      //   0 : starting node
                      //   50 : middle of the edge
                      //   100 : ending node
   public double pELdist;   // distance in unitlength between the label and the edge
    /** The distance is between the center of the label and the edge */
    public boolean pELdistC;

    public double unitlength;
    public String unitName;
    public double ratio;

    public int dashIndex=-1;

    //     static double scale = 3.8;
    
    public GastexGlobalProperties() {
// 	unitlength = 1;
// 	unitName = "mm";
	unitlength = Configuration.unitLength;
	unitName = Configuration.unitName;
	ratio = getRatio();
	initialise();
    }

    public static GastexGlobalProperties texDefaults = new GastexGlobalProperties();
    
    public GastexGlobalProperties(double ul, String un) {
	unitlength = ul;
	unitName = un;
	ratio = getRatio();
	initialise();
    }

    public GastexGlobalProperties(String type) {
	this();
	if(type.equals("slide")) {
	    //gasset{Nw=12,Nh=12,Nmr=6,ilength=8,flength=8,rdist=1,loopdiam=12}
	    pNw = unitLength2Internal(12);
	    pNh = unitLength2Internal(12);
	    pNmr = unitLength2Internal(6);
	    pilength = unitLength2Internal(8);
	    pflength = unitLength2Internal(8);
	    prdist = unitLength2Internal(1);
	    ploopdiam = unitLength2Internal(12);
	}
    }
    
    public GastexGlobalProperties(GastexGlobalProperties ggp) {
	pNw = ggp.pNw;
	pNh = ggp.pNh;
	pNmr = ggp.pNmr;

	pNframe = ggp.pNframe;
	
	pfillgray = ggp.pfillgray;
	pNfill = ggp.pNfill;

	pExtNL = ggp.pExtNL;
	pNLangle = ggp.pNLangle;
	pNLdist = ggp.pNLdist; //0?
	
	piangle = ggp.piangle;
	pilength = ggp.pilength;
	pfangle =ggp.pfangle;
	pflength = ggp.pflength;
	
	prdist = ggp.prdist;
	
	pNmarks = ggp.pNmarks;	
	pNadjustdist = ggp.pNadjustdist;
	pNadjust = ggp.pNadjust;
	

	ploopdiam = ggp.ploopdiam;
	ploopangle = ggp.ploopangle;
	ploopCW = ggp.ploopCW;

	pcurvedepth = ggp.pcurvedepth;
	
	pELside = ggp.pELside;
	pELpos = ggp.pELpos;
	pELdist = ggp.pELdist;
	
	plinegray = ggp.plinegray;
	
	plinewidth = ggp.plinewidth;
	pdash_length = ggp.pdash_length;
	pdash_empty = ggp.pdash_empty;

	pfillcolor = ggp.pfillcolor;
	plinecolor = ggp.plinecolor;
	unitlength = ggp.unitlength;
	unitName = ggp.unitName;
	ratio = getRatio();

	// Added by Teo
	pAHnb = ggp.pAHnb;
	pAHdist =  ggp.pAHdist;
	pAHangle = ggp.pAHangle;
	pAHLength = ggp.pAHLength;
	pAHlength = ggp.pAHlength;

 	psxo = ggp.psxo;
	psyo = ggp.psyo;
	pexo = ggp.pexo;
	peyo = ggp.peyo;
	pELdistC = ggp.pELdistC;
  }
    void initialise() {
		//gasset{Nw=8,Nh=8,Nmr=4} % circle
	pNw  = unitLength2Internal(8);
	pNh  = unitLength2Internal(8);
	pNmr = unitLength2Internal(4);
	//gasset{Nframe=y}
	pNframe = true;
	//gasset{fillgray=0,Nfill=n} % Not filled but black if filled
	pfillgray=0; pNfill = false;
	//gasset{ExtNL=n,NLangle=90,NLdist=0}
	pExtNL = false; pNLangle = Math.toRadians(90);
	pNLdist = unitLength2Internal(0);
	//gasset{iangle=180,ilength=5}
	piangle = Math.toRadians(180);
	pilength = unitLength2Internal(5);
	//gasset{fangle=0,flength=5}
	pfangle = Math.toRadians(0);
	pflength = unitLength2Internal(5);
	//gasset{rdist=0.7}
	prdist = unitLength2Internal(0.7);
	//gasset{Nmarks=n} % no mark
	pNmarks = 0;
	//gasset{Nadjustdist=1,Nadjust=n} % no adjust
	pNadjustdist = unitLength2Internal(1); pNadjust = 0;
	//gasset{loopdiam=8,loopangle=90,loopCW=y}
	ploopdiam = unitLength2Internal(8);
	ploopangle = Math.toRadians(90);
	ploopCW = true;
	//gasset{curvedepth=0}
	pcurvedepth = unitLength2Internal(0);
	//% One triangular small arrowhead
	//gasset{ELside=l,ELpos=50,ELdist=1}
	pELside = 0; pELpos = 50;
	pELdist = unitLength2Internal(1);
	//gasset{linegray=0} % black lines
	plinegray=0;
	//gasset{linewidth=0.14,dash={}0} % continuous path
	plinewidth = unitLength2Internal(0.14);
	pdash_length = null;	pdash_empty = (float)unitLength2Internal(0);
	//gasset{AHnb=1,AHdist=1.41,AHangle=20,AHLength=1.5,AHlength=1.41} 
	pAHnb = 1;
	pAHdist = unitLength2Internal(1.41);
	pAHangle = Math.toRadians(20); 
	pAHLength = unitLength2Internal(1.5);
	pAHlength = unitLength2Internal(1.41);
	
	psxo =  unitLength2Internal(0);
	psyo =  unitLength2Internal(0);
	pexo =  unitLength2Internal(0);
	peyo =  unitLength2Internal(0);
	pELdistC = false;
	pfillcolor = null;
	plinecolor = null;
     }

    /*
    void detach(JasObject jo){
	listOfObject.remove(jo);
    }

    void detach(Vector vjo){
	int sz=vjo.size();
	for(int i=0;i<sz;i++)
	    detach((JasObject) vjo.elementAt(i));
    }

    void attach(JasObject jo){
	listOfObject.add(jo);
	jo.globalProp=this;
    }

    void attach(Vector vjo){
	int sz=vjo.size();
	for(int i=0;i<sz;i++)
	    attach((JasObject) vjo.elementAt(i));
    }
    
    boolean belongsToProp(JasObject jo){
	return listOfObject.contains(jo);
    }

    boolean moveToProp(GastexGlobalProperties gp,JasObject jo){
	if(belongsToProp(jo)){
	    detach(jo);
	    gp.attach(jo);
	    return true;
	}
	return false; // error !
    }

    boolean moveToProp(GastexGlobalProperties gp,Vector vjo){
	detach(vjo);
	gp.attach(vjo);
	return true;
    }

    Vector getListObject(){ return listOfObject; }
    boolean[] whichBelongsTo(Vector arrayObject){
	int sz=arrayObject.size();
	boolean temp[]=new boolean[sz];
	for(int i=0;i<sz;i++){
	    temp[i]=belongsToProp((JasObject) arrayObject.elementAt(i));
	}
	return temp;
    }
    */

    public void setName(String pname){ propName=pname; }
    
    public boolean belongsToProp(JasObject jo){
	if (jo.getGgp()==this) return true;
	return false;
    }

    public Vector whichBelongsTo(Vector arrayObject){
	int sz=arrayObject.size();
	Vector temp=new Vector();
	JasObject jo;
	for(int i=0;i<sz;i++){
	    jo=(JasObject) arrayObject.elementAt(i);
	    if(belongsToProp(jo))
		temp.add(jo);
	}
	return temp;
    }

    public double ratioIN() {
	return (double)(Toolkit.getDefaultToolkit().getScreenResolution());
    }
     
    public double getRatio(double _unitlength, String _unitname) {
	double ratio = ratioIN();
	if(_unitname.equals("mm")) {
	    ratio /= 25.4;
	}
// 	} else if (_unitname.equals("in")) {
// 	    ratio = ratioIN();
// 	}
	return (ratio * _unitlength);
    }
    public double getRatio() {
	return getRatio(unitlength, unitName);
    }

    public void setUnitLength(double _unitlength) {
	unitlength = _unitlength;
	ratio = getRatio();
    }
    public void setUnitName(String _unitname) {
	unitName = _unitname;
	ratio = getRatio();
    }
    public void setUnit(double _unitlength, String _unitname) {
	unitlength = _unitlength;
	unitName = _unitname;
	ratio = getRatio();
    }
    public double getUnitLength() {
    	return unitlength;
    }
    public String getUnitName() {
	return unitName;
    }

    public double unitLength2Internal(double ul) {
	return (ul*ratio);
    }

    public double internal2UnitLength(double internal) {
	return (internal/ratio);
    }

    /** Function used to round a Double using Configuration.roundDecal
     * @param value the number to round up
     * @see Configuration#roundDecal
    */
    public double roundDouble(double value) {
	return ((Math.round(value*Configuration.roundDecal))/
		Configuration.roundDecal);
    }
    public String getLatexCodeProperties(GastexGlobalProperties globalProp) {
	String lProp = "";
	//JasObject
	if (plinecolor == null) {
	    if (plinegray != globalProp.plinegray) {
		lProp += ",linegray="+roundDouble(plinegray);
	    }
	} else if (!plinecolor.equals(globalProp.plinecolor)) {
	    lProp += ",linecolor="+plinecolor;
	}
	
	if (plinewidth != globalProp.plinewidth) {
	    lProp += ",linewidth="+roundDouble(internal2UnitLength(plinewidth));
	}
	//here, get dash property
	//Node
	if (pNw != globalProp.pNw) {
		lProp += ",Nw="+roundDouble(internal2UnitLength(pNw));
	}
	if (pNh != globalProp.pNh) {
		lProp += ",Nh="+roundDouble(internal2UnitLength(pNh));
	}
	if (pNmr != globalProp.pNmr) {
		lProp += ",Nmr="+roundDouble(internal2UnitLength(pNmr));
	}
	//ANode
	if (pNadjust != globalProp.pNadjust) {
	    lProp += ",Nadjust=";
	    if (pNadjust == 0) {
		lProp += "n";
	    } else {
		if ((pNadjust & 1) != 0) {
		    lProp += "w";
		}
		if ((pNadjust & 2) != 0) {
		    lProp += "h";
		}
	    }
	}
	if (pNadjustdist != globalProp.pNadjustdist) {
		lProp += ",Nadjustdist="+roundDouble(internal2UnitLength(pNadjustdist));
	}


	if (pNframe != globalProp.pNframe) {
		lProp += ",Nframe="+(pNframe?"y":"n");
	}
	if (pNfill != globalProp.pNfill) {
		lProp += ",Nfill="+(pNfill?"y":"n");
	}
	
	if (pfillcolor == null) {
	    if (pNfill && (pfillgray != globalProp.pfillgray)) {
		lProp += ",fillgray="+roundDouble(pfillgray);
	    }
	} else {
	    if (pNfill && !pfillcolor.equals(globalProp.pfillcolor)) {
		lProp += ",fillcolor="+pfillcolor;
	    }
	}
	if (pNmarks != globalProp.pNmarks) {
	    lProp += ",Nmarks=";
	    if (pNmarks == 0) {
		lProp += "n";
	    } else {
		if ((pNmarks & 1) != 0) {
		    lProp += "i";
		}
		if ((pNmarks & 2) != 0) {
		    lProp += "f";
		}
		if ((pNmarks & 4) != 0) {
		    lProp += "r";
		}
	    }
	}
	//NodeLabel
	if (pExtNL != globalProp.pExtNL) {
	    lProp += ",ExtNL="+(pExtNL?"y":"n");
	}
	if (pNLangle != globalProp.pNLangle) {
	    lProp += ",NLangle="+roundDouble(Math.toDegrees(pNLangle));
	}
	if (pNLdist != globalProp.pNLdist) {
	    lProp += ",NLdist="+roundDouble(internal2UnitLength(pNLdist));
	}

	//IFMark
	if (pilength != globalProp.pilength) {
	    lProp += ",ilength="+roundDouble(internal2UnitLength(pilength));
	}
	if (piangle != globalProp.piangle) {
	    lProp += ",iangle="+roundDouble(Math.toDegrees(piangle));
	}
	if (pflength != globalProp.pflength) {
	    lProp += ",flength="+roundDouble(internal2UnitLength(pflength));
	}
	if (pfangle != globalProp.pfangle) {
	    lProp += ",fangle="+roundDouble(Math.toDegrees(pfangle));
	}
	//RMark
	if (prdist != globalProp.prdist) {
	    lProp += ",rdist="+roundDouble(internal2UnitLength(prdist));
	}
	//LOOP
	if (ploopdiam != globalProp.ploopdiam) {
	    lProp += ",loopdiam="+roundDouble(internal2UnitLength(ploopdiam));
	}
	if (ploopangle != globalProp.ploopangle) {
	    lProp += ",loopangle="+roundDouble(Math.toDegrees(ploopangle));
	}
	if (ploopCW != globalProp.ploopCW) {
	    lProp += ",loopCW="+(ploopCW?"y":"n");
	}
	//AEdge
	if (pAHnb != globalProp.pAHnb) {
	    lProp += ",AHnb="+(int)pAHnb;
	}
	if (pAHdist != globalProp.pAHdist) {
	    lProp += ",AHdist="+roundDouble(internal2UnitLength(pAHdist));
	}
	if (pAHangle != globalProp.pAHangle) {
	    lProp += ",AHangle="+roundDouble(Math.toDegrees(pAHangle));
	}
	
	if (pAHLength != globalProp.pAHLength) {
	    lProp += ",AHLength="+roundDouble(internal2UnitLength(pAHLength));
	}
	if (pAHlength != globalProp.pAHlength) {
	    lProp += ",AHlength="+roundDouble(internal2UnitLength(pAHlength));
	}
	//Edge
	if (pcurvedepth != globalProp.pcurvedepth) {
	    lProp += ",curvedepth="+roundDouble(internal2UnitLength(pcurvedepth));
	}

	//EdgeLabel
	if (pELside != globalProp.pELside) {
	    lProp += ",ELside="+(pELside==0?"l":"r");
	}
	if ( pELpos!= globalProp.pELpos) {
	    lProp += ",ELpos="+Math.round(pELpos);
	}
	if ( pELdist!= globalProp.pELdist) {
	    lProp += ",ELdist="+roundDouble(internal2UnitLength(pELdist));
	}
	if (pELdistC != globalProp.pELdistC) {
		lProp += ",ELdistC="+(pELdistC?"y":"n");
	}
	if (!isDashEquals(pdash_length,
			  pdash_empty,
			  globalProp.pdash_length,
			  globalProp.pdash_empty)) {
	    lProp += ",dash={";
	    if (pdash_length != null) {
		int len = pdash_length.length;
		for(int i = 0; i < len; i++) {
		    lProp += 
			roundDouble(internal2UnitLength(pdash_length[i]));;
		    if (i < len-1) {
			lProp += " ";
		    }
		}
	    }		
	    lProp +="}{"+ roundDouble(internal2UnitLength(pdash_empty)) +"}";
	}
	if (psxo != globalProp.psxo) {
	    lProp += ",sxo="+roundDouble(internal2UnitLength(psxo));
	}
	if (psyo != globalProp.psyo) {
	    lProp += ",syo="+roundDouble(internal2UnitLength(psyo));
	}
	if (pexo != globalProp.pexo) {
	    lProp += ",exo="+roundDouble(internal2UnitLength(pexo));
	}
	if (peyo != globalProp.peyo) {
	    lProp += ",eyo="+roundDouble(internal2UnitLength(peyo));
	}

	if (lProp.equals("")) {
	    return "";
	} else {
	    //cut the first comma
	    return ("{" + lProp.substring(1) + "}");
	}
    }

   public  static boolean isDashEquals(float[] dash_length, float dash_empty,
			       float[] dash_l, float dash_e) {
	if (dash_l != null) {
	    if (dash_length == null)
		return false;
	    int len = dash_l.length;
	    if (len != dash_length.length)
		return false;
	    if (dash_e != dash_empty)
		return false;
	    for(int i = 0; i < len; i++) {
		if (dash_l[i] != dash_length[i])
		    return false;
	    }
	    return true;
	}
	if (dash_length == null) {
	    return true;
	} else {
	    return false;
	}
    }
	

    public String getLatexUnitProperty(GastexGlobalProperties ggp) {
	String unitProp = "";
	if (ggp.getUnitLength() != unitlength &&
	    !ggp.getUnitName().equals(unitName)) {
	    unitProp = "\\unitlength="+unitlength+unitName+"\n";
	}
	return unitProp;
    }

	  
    public String getLatexCode(GastexGlobalProperties ggp) {
	String unitl = getLatexUnitProperty(ggp);
	String gasset = "";
	String lProp = getLatexCodeProperties(ggp);

	if (!lProp.equals("")) {
	    gasset = "\\gasset"+lProp+"\n";
	}
	return (unitl+gasset);
    }

  public float[] getpdash_length() {
	float[] pdash_length2;
	if (pdash_length != null) {
	    int len = pdash_length.length;
	    pdash_length2 = new float[len];
	    for (int i = 0; i < len; i++) {
		pdash_length2[i] = 
		    (float)(internal2UnitLength(pdash_length[i]));
	    }
	    return pdash_length2;
	} else {
	    return null;
	}
    }   
    public void setpdash_length(float[] pdash_length) {
	float[] pdash_length2;
	if (pdash_length != null) {
	    int len = pdash_length.length;
	    pdash_length2 = new float[len];
	    for (int i = 0; i < len; i++) {
		pdash_length2[i] = 
		    (float)(unitLength2Internal(pdash_length[i]));
	    }
	    this.pdash_length = pdash_length2;
	} else {
	    this.pdash_length = null;
	}
    }

    public float getpdash_empty() {
	return (float)internal2UnitLength(pdash_empty);
    }   
    public void setpdash_empty(float pdash_empty) { 
	this.pdash_empty = (float)unitLength2Internal(pdash_empty);
    }
    
    public void setDashIndex(int i)  {	dashIndex = i;    }
    public int getDashIndex() {return dashIndex;} 
    
  public double getploopdiam() {
	return internal2UnitLength(ploopdiam);
    }
    public void   setploopdiam(double ploopdiam) {  
	this.ploopdiam = unitLength2Internal(ploopdiam);
    }

    public double getploopangle() {
	return Math.toDegrees(ploopangle);
    }
    public void setploopangle(double angle) {
	ploopangle = Math.toRadians(angle);
    }



  public double getpsxo() {
	return internal2UnitLength(psxo);
    }
    public double getpsyo() {
	return internal2UnitLength(psyo);
    }
    public double getpexo() {
	return internal2UnitLength(pexo);
    }
    public double getpeyo() {
	return internal2UnitLength(peyo);
    }
    public void setpsxo(double _psxo) {
	psxo = unitLength2Internal(_psxo);
    }
    public void setpsyo(double _psyo) {
	psyo = unitLength2Internal(_psyo);
    }
    public void setpexo(double _pexo) {
	pexo = unitLength2Internal(_pexo);
    }
    public void setpeyo(double _peyo) {
	peyo = unitLength2Internal(_peyo);
    }

    public void setSlide(){ 
	pNw = unitLength2Internal(12);
	pNh = unitLength2Internal(12);
	pNmr = unitLength2Internal(6);
	pilength = unitLength2Internal(8);
	pflength = unitLength2Internal(8);
	prdist = unitLength2Internal(1);
	ploopdiam = unitLength2Internal(12);
    }
    
    public void setToGastexDefaults() {
        // unitlength=1mm        
        unitlength = 1;
        unitName = "mm";
        // gasset{Nw=8,Nh=8,Nmr=4} % circle
        pNw = 8;
        pNh = 8;
        pNmr = 4;
        // gasset{Nframe=y}
        pNframe = true;
        // gasset{fillgray=0,Nfill=n} % Not filled but black if filled
        pfillgray = 0;
        pNfill = false;
        // gasset{ExtNL=n,NLangle=90,NLdist=0}
        pExtNL = false;
        pNLangle = 90;
        pNLdist = 0;
        // gasset{iangle=180,ilength=5}
        piangle = 180;
        pilength = 5;
        // gasset{fangle=0,flength=5}
        pfangle = 0;
        pflength = 5;
        // gasset{rdist=0.7}
        prdist = 0.7;
        // gasset{Nmarks=n} % no mark
        pNmarks = 0;
        // gasset{Nadjustdist=1,Nadjust=n} % no adjust
        pNadjustdist = 1;
        pNadjust = 0;
        // gasset{sxo=0,syo=0,exo=0,eyo=0}
        psxo = 0; psyo = 0;
        pexo = 0; peyo = 0;
        // gasset{curvedepth=0}
        pcurvedepth = 0;
        // gasset{loopdiam=8,loopangle=90,loopCW=y}
        ploopdiam = 8;
        ploopangle = 90;
        ploopCW = true;
        // One triangular small arrowhead
        // gasset{AHnb=1,AHdist=1.41,AHangle=20,AHLength=1.5,AHlength=1.41} 
        pAHnb = 1;
        pAHdist = 1.41;
        pAHangle = 20;
        pAHLength = 1.5;
        pAHlength = 1.41;
        // gasset{ATnb=0,ATdist=1.41,ATangle=20,ATLength=1.5,ATlength=1.41} 
        // TODO
        // gasset{ELside=l,ELpos=50,ELdist=1}
        pELside = 0;
        pELpos = 50;
        pELdist = 1;
        // gasset{linegray=0} % black lines
        plinegray = 0;
        // gasset{linewidth=0.14,dash={}0} % continuous path
        plinewidth = 0.14;
        pdash_length = null;
        pdash_empty = 0;
        // gasset{arcradius=0,polyangle=0} % sharp angles    }
        // TODO
    }
}

