package jastex;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import jastex.old.gui.tabs.JasArrowTab;
import jastex.old.gui.JasToolBox;
import jastex.old.gui.tabs.JasLoopTab;
import jastex.old.gui.tabs.OJasLineTab;
import jastex.old.gui.tabs.JasLineOffsetTab;
import jastex.old.gui.tabs.OJasNodeTab;
import jastex.old.gui.tabs.JasIFMarkTab;
import jastex.old.gui.tabs.JasGGPLabelTab;


/** This class create the configuration dialog
 * in order to modify JasTeX general config
 */
public class JasDefaultGGP extends JasToolBox implements ActionListener{
    
    public GastexGlobalProperties oldGgp;

    public GastexGlobalProperties myGgp;
    public OJasLineTab lt;
    public OJasNodeTab nt;
    public JasGGPLabelTab lbt;
    public JasIFMarkTab ift;
    public JasArrowTab eat;
    public JasLoopTab loopTab;
    public JasLineOffsetTab offsetTab;
    
    public ObjetManager objMan;
    
    static private int lineTabIndex = 0;
    static private int nodeTabIndex = 1;
    static private int labelTabIndex = 2;
    static private int ifmarkTabIndex = 3;
    static private int arrowTabIndex = 4;
    static private int loopTabIndex = 5;
    static private int offsetTabIndex = 6;
    
    
    public JasDefaultGGP(Frame parentFrame , String toolBoxTitle, ObjetManager objMan)
    {
	super(parentFrame,toolBoxTitle);

	this.objMan = objMan;
	// save the old one
	oldGgp = objMan.getActualProp();
	
	// myGgp = new GastexGlobalProperties(objMan.getActualProp());
	objMan.createNewProp();
	myGgp = objMan.getActualProp();

	/* OK Buton */
	JButton okButton = new JButton("OK");
	okButton.setActionCommand("ok_ggp");
	okButton.addActionListener(this);

	/* Apply Buton */
	JButton applyButton = new JButton(Languages.tb_ApplyBtn);
	applyButton.setActionCommand("apply_ggp");
	applyButton.addActionListener(this);

	/* Cancel Buton */
	JButton cancelButton = new JButton("Annuler");
	cancelButton.setActionCommand("cancel_ggp");
	cancelButton.addActionListener(this);

	
	JPanel buttonPane = new JPanel(); 
	buttonPane.setLayout(new BoxLayout(buttonPane, BoxLayout.X_AXIS));
	buttonPane.add(Box.createHorizontalGlue());
	buttonPane.add(okButton);
	buttonPane.add(applyButton);
	buttonPane.add(cancelButton);

	
	GridBagLayout layout = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	c.fill = GridBagConstraints.BOTH;	
	getContentPane().setLayout(layout);
	
	c.gridx=0;
	c.gridy=0;
	layout.setConstraints(tabbedPane,c);
	getContentPane().add(tabbedPane);
	
	c.gridx=0;
	c.gridy=2;
	c.insets = new Insets(5,0,0,0);
	layout.setConstraints(buttonPane,c);
	getContentPane().add(buttonPane);



	createLineTab();
	createNodeTab();
	createGGPLabelTab();
	createIFMarkTab();
	createArrowTab();
	createLoopTab();
	createOffsetTab();
	pack();
	setVisible(true);
	
	
    }



    /** Create the first tab Line tab 
     */
    private void createLineTab()
    {
	lt = new OJasLineTab(Languages.tb_tabtitle[0],myGgp);
	addTab(lt);
	tabbedPane.insertTab(lt.getTabTitle(),null,lt.getPanel(),Languages.tb_tabtooltip[0], lineTabIndex);
    }



    /** Create the Second Tab  Node properties 
     */
    private void createNodeTab()
    {
	nt = new OJasNodeTab(Languages.tb_tabtitle[1],myGgp);
	addTab(nt);
	tabbedPane.insertTab(nt.getTabTitle(),null,nt.getPanel(),Languages.tb_tabtooltip[1], nodeTabIndex);
    }



    /** create the third tab  the Label tab 
     */
    private void createGGPLabelTab()
    {
	lbt = new JasGGPLabelTab(Languages.tb_tabtitle[2],myGgp);
	addTab(lbt);
	tabbedPane.insertTab(lbt.getTabTitle(),null,lbt.getPanel(),Languages.tb_tabtooltip[2], labelTabIndex);
    }


    /** Create the fourth tab the IFRMark tab 
     */
    private void createIFMarkTab()
    {
	ift = new JasIFMarkTab("IFRMarks",myGgp);
	addTab(ift);
	tabbedPane.insertTab(ift.getTabTitle(),null,ift.getPanel(),Languages.tb_tabtooltip[3], ifmarkTabIndex);
    }


    /** Create the fifth tab the  Arrow tab 
     */
    private void createArrowTab()
    {
	eat = new JasArrowTab(Languages.tb_tabtitle[5],myGgp);
	addTab(eat);
	tabbedPane.insertTab(eat.getTabTitle(),null,eat.getPanel(),Languages.tb_tabtooltip[5], arrowTabIndex);
    }


   /** Create the sixth tab the  Arrow tab 
     * @param objects the vector of objects being edited
     */
    private void createLoopTab()
    {
	loopTab = new JasLoopTab(Languages.tb_tabtitle[7],myGgp);
	addTab(loopTab);
	tabbedPane.insertTab(loopTab.getTabTitle(),null,loopTab.getPanel(),null, loopTabIndex);
    }


   /** Create the seventh tab the  Arrow tab 
     * @param objects the vector of objects being edited
     */
    private void createOffsetTab()
    {
	offsetTab = new JasLineOffsetTab(Languages.tb_tabtitle[8],myGgp);
	addTab(offsetTab);
	tabbedPane.insertTab(offsetTab.getTabTitle(),null,offsetTab.getPanel(),null, offsetTabIndex);
    }


  /** Save the toolbox content in the ggp
     * of all of the vector objects 
     */
    public void setProp()
    {
	lt.saveGgp(myGgp);
	nt.saveGgp(myGgp);
	lbt.saveGgp(myGgp);
	ift.saveGgp(myGgp);
	eat.saveGgp(myGgp);
	loopTab.saveGgp(myGgp);
	offsetTab.saveGgp(myGgp);

	//objMan.selectProp(myGgp);
	if(myGgp.getLatexCodeProperties(oldGgp).equals("")){
	    // no difference
	    // removing
	    objMan.removeProp(myGgp);
	    objMan.selectProp(oldGgp);
	}
	
    }
    


  public void actionPerformed(ActionEvent e) {
	String cmd = e.getActionCommand();
	if (cmd.equals("apply_ggp"))
	    {
		setProp();
		
	    }
	else if	 (cmd.equals("ok_ggp"))
	    {
		setProp();
		dispose();
		
	    }
	else if (cmd.equals("cancel_ggp"))
	    {
		dispose();
		
	    }

  }

}
