/*
 * JasDrawingArea.java
 *
 * Created on 2 avril 2005, 09:35
 */

package jastex.gui;

import jastex.GastexGlobalProperties;
import java.awt.Graphics;
import java.awt.Graphics2D;
import jastex.obj.*;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Rectangle;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.geom.Rectangle2D;
import javax.swing.JComponent;
import javax.swing.event.MouseInputListener;
import java.util.Enumeration;
import java.util.Vector;
import javax.swing.Timer;


/**
 *
 * @author steph
 */
public class JasDrawingArea extends JComponent implements MouseInputListener {
    
    public GastexGlobalProperties ggp;
    public JasVector jasVector;
    public float zoomFactor = 1;
    private int height;
    
    public double gridStep = 4;
    public boolean gridShow = true;
    
    public Vector<IEditable> editableObjects;
    public IEditable selEditable = null;
    public MouseablePoint selMP = null;
    public JasObjectToolBox tb = null;
    
    /** Creates a new instance of JasDrawingArea */
    public JasDrawingArea() {
        ggp = new GastexGlobalProperties();
        ggp.setToGastexDefaults();
        jasVector = new JasVector(ggp);
        editableObjects = new Vector<IEditable>();
        addMouseListener(this);
        addMouseMotionListener(this);
        
        // sometimes, Java doesn't repaint automatically
        new Timer(1000, new ActionListener() {
            public void actionPerformed(ActionEvent evt) {
                repaint();
            }
        }).start();
    }
    
    protected void paintComponent(Graphics roGraphics) {
        if (roGraphics != null) {
            Graphics2D g = (Graphics2D)roGraphics.create();
            Rectangle rect = g.getClipBounds();
            g.setColor(Color.WHITE);
            g.fill(rect);
            height = rect.height;
            g.translate(0, height);
            g.scale(1, -1);
            
            // draw magnetic grid
            if (gridShow) {
                Rectangle2D.Double r = new Rectangle2D.Double(0, 0, 0, 0);
                double step = gridStep*zoomFactor;
                g.setColor(Color.BLACK);
                for (r.y=0; r.y<rect.height; r.y+=step) {
                    for (r.x=0; r.x<rect.width; r.x+=step) {
                        g.draw(r);
                    }
                }
            }
            
            // draw objects
            g.scale(zoomFactor, zoomFactor);
            jasVector.draw(g);

            // draw selection
            if (selEditable != null) selEditable.drawSketch(g);
        }
    }
    
    public void mouseMoved(MouseEvent ev) {
        double x = ev.getX()/zoomFactor;
        double y = (height-ev.getY())/zoomFactor;
        MouseablePoint[] mp;
        
        if (selEditable != null && (mp = selEditable.getMouseablePoints()) != null) {
            for (int i=mp.length-1; i>=0; i--) {
                if (mp[i].contains(x, y)) {
                    setCursor(mp[i].cursor);
                    return;
                }
            }
        }
        
        for (Enumeration<IEditable> e = editableObjects.elements(); e.hasMoreElements() ;) {
            IEditable object = e.nextElement();
            if (object.contains(x, y)) {
                setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
                return;
            }
        }

        setCursor(Cursor.getDefaultCursor());
    }
    
    public void mouseClicked(MouseEvent e) {}    
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    
    public void mousePressed(MouseEvent ev) {
        double x = ev.getX()/zoomFactor;
        double y = (height-ev.getY())/zoomFactor;
        MouseablePoint[] mp;
        
        for (Enumeration<IEditable> e = editableObjects.elements(); e.hasMoreElements() ;) {
            IEditable object = e.nextElement();
            if (object.contains(x, y)) {
                selEditable = object;
                selEditable.updateMouseablePoints();
                repaint();
                break;
            }
        }

        if (selEditable != null && (mp = selEditable.getMouseablePoints()) != null) {
            for (int i=mp.length-1; i>=0; i--) {
                if (mp[i].startDrag(x, y)) {
                    setCursor(mp[i].cursor);
                    selMP = mp[i];
                    selMP.granularity = gridStep;
                    if (tb != null) tb.setVisible(false);
                    return;
                }
            }
        }
        setCursor(Cursor.getDefaultCursor());
    }
    
    public void mouseReleased(MouseEvent ev) {
        double x = ev.getX()/zoomFactor;
        double y = (height-ev.getY())/zoomFactor;
        
        if (selMP == null && selEditable != null && !selEditable.contains(x, y)) {
            selEditable = null;
        }
        selMP = null;
        repaint();
        if (tb != null && selEditable != null) {
            tb.sendObject(selEditable);
            tb.setVisible(true);
        }
    }
    
    public void mouseDragged(MouseEvent e) {
        double x = e.getX()/zoomFactor;
        double y = (height-e.getY())/zoomFactor;
        
        if (selMP != null) {
            selMP.onDrag(x, y);
            repaint();
        }
    }
}
