/*
 * AbstractClosedLine.java
 *
 * Created on 11 mars 2005, 13:32
 */

package jastex.obj;
import jastex.*;
import java.awt.Color;


/**
 * Abstract class which implements IClosedLine provides standard behavior
 * of closed lines.
 * @author sglondu
 */
public abstract class AbstractClosedLine extends AbstractLine implements IClosedLine {
    
    protected boolean pNframe;
    protected boolean pNfill;
    protected float pfillgray;
    protected String pfillcolor;
    
    /** Constructor initializing the field with 
     * GastexGlobalProperties default value
     * @param _ggp The GastexGlobalProperties used to init
     * @see GastexGlobalProperties
     */
    protected AbstractClosedLine(GastexGlobalProperties _ggp) {
        super(_ggp);
    }

    public void setFromGgp(GastexGlobalProperties _ggp){
	super.setFromGgp(_ggp);
        pNframe = _ggp.pNframe;
	pNfill = _ggp.pNfill;
	pfillgray = (float)_ggp.pfillgray;	
	pfillcolor = _ggp.pfillcolor;
    }
    
    protected String getLatexCodeProperties() {
	String lProp = super.getLatexCodeProperties();

	if (pNframe != globalProp.pNframe) {
		lProp += ",Nframe="+(pNframe?"y":"n");
	}
	
        if (pNfill != globalProp.pNfill) {
            lProp += ",Nfill="+(pNfill?"y":"n");
	}
	
	if (pNfill) {
            if (pfillcolor == null) {
                if (pfillgray != globalProp.pfillgray) {
                    lProp += ",fillgray="+pfillgray;
                }
            } else {
                if (!pfillcolor.equals(globalProp.pfillcolor)) {
                    lProp += ",fillcolor="+pfillcolor;
                }
            }
        }
        
        return lProp;
    }

    /** get the fill color
    * @return a color, depending on pNfill and (pfillcolor or pfillgray)
    */
    public Color getFillColor() {
	if (pNfill) {
	    if (pfillcolor != null) {
		JasColorName x = new JasColorName(pfillcolor);
		return (x.getColor());
	    } else {
		//gray level
		return (new Color(pfillgray, pfillgray, pfillgray));
	    }
	} else {
	    //Not filled
	    return null;
	}
    }
    
    /* There is nothing interesting below... */

    public boolean getPNframe() {
        return pNframe;
    }

    public void setPNframe(boolean _PNframe) {
        pNframe = _PNframe;
    }

    public boolean getPNfill() {
        return pNfill;
    }
    
    public float getPfillgray() {
        return pfillgray;
    }

    public String getPfillcolor() {
        return pfillcolor;
    }

    public void setPNfill(boolean _PNfill) {
        pNfill = _PNfill;
    }

    public void setPfillcolor(String _pfillcolor) {
        pfillcolor = _pfillcolor;
        pNfill = pfillcolor != null;
    }

    public void setPfillgray(float _pfillgray) {
        pfillgray = _pfillgray;
        pNfill = true;
    }
}
