/*
 * AbstractLine.java
 *
 * Created on 8 mars 2005, 10:18
 */

package jastex.obj;
import jastex.*;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Stroke;


/**
 * Abstract class which implements ILine provides standard behavior of
 * lines.
 * @author sglondu
 */
public abstract class AbstractLine extends AbstractJasObject implements ILine {
    
    protected float plinegray;
    protected String plinecolor;
    protected float plinewidth;
    protected float[] pdash_length;
    protected float   pdash_empty;
 
    /** Constructor initializing the field with 
     * GastexGlobalProperties default value
     * @param _ggp The GastexGlobalProperties used to init
     * @see GastexGlobalProperties
     */
    protected AbstractLine(GastexGlobalProperties _ggp){
	super(_ggp);
    }
        
    /** Set the internal value to the one of the GastexGlobalProperties
     * @param _ggp the GastexGlobaProperties used to initialize values
     * @see GastexGlobalProperties
     */ 
    public void setFromGgp(GastexGlobalProperties _ggp){
        plinegray = (float)_ggp.plinegray;
	plinecolor = _ggp.plinecolor;
	plinewidth = (float)_ggp.plinewidth;
	pdash_length = _ggp.pdash_length;
	pdash_empty = _ggp.pdash_empty;
    }

    /** Return the Gastex Code associated with the current fields values
     * @return a String containing the properties modified compare to 
     * the current GastexGlobalProperties, beginning with a comma or 
     * the empty string ("")
     */
   protected String getLatexCodeProperties() {
	String lProp = "";
        
	if (plinecolor == null) {
	    if (plinegray != globalProp.plinegray) {
		lProp += ",linegray="+plinegray;
	    }
	} else if (!plinecolor.equals(globalProp.plinecolor)) {
	    lProp += ",linecolor="+plinecolor;
	}
	
	if (plinewidth != globalProp.plinewidth) {
	    lProp += ",linewidth="+plinewidth;
	}

	if (!GastexGlobalProperties.isDashEquals(pdash_length,
                                                 pdash_empty,
                                                 globalProp.pdash_length,
					         globalProp.pdash_empty)) {
	    lProp += ",dash={";
	    if (pdash_length != null) {
		int len = pdash_length.length;
		for(int i = 0; i < len; i++) {
		    lProp += pdash_length[i];
		    if (i < len-1) {
			lProp += " ";
		    }
		}
	    }		
	    lProp +="}{"+ pdash_empty +"}";
	}
        
        return lProp;
    }

    /** get the color used to draw a line
     * @return a color, depending on plinecolor or plinegray
     */
    public Color getLineColor() {
	if (plinecolor != null) {
	    JasColorName x = new JasColorName(plinecolor);
	    return (x.getColor());
	} else {
	    //graylevel line
	    return (new Color(plinegray, plinegray, plinegray));
	}
    }

    public Stroke getStroke() {
        if (pdash_length == null) {
            return new BasicStroke(plinewidth);
        } else {
            return new BasicStroke(
                plinewidth,
                BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL, 0,
                pdash_length, pdash_empty);
        }
    }
    
    /* There is nothing interesting below... */
   
    public float getPlinegray() {
        return plinegray;
    }
    
    public String getPlinecolor() {
        return plinecolor;
    }
    
    public float getPlinewidth() {
        return plinewidth;
    }
    
    public float[] getPdash_length() {
        return pdash_length;
    }
    
    public float getPdash_empty() {
        return pdash_empty;
    }
 
    public void setPdash_empty(float _pdash_empty) {
        pdash_empty = _pdash_empty;
    }

    public void setPdash_length(float[] _pdash_length) {
        pdash_length = _pdash_length;
    }

    public void setPlinecolor(String _plinecolor) {
        plinecolor = _plinecolor;
    }

    public void setPlinegray(float _plinegray) {
        plinegray = _plinegray;
    }

    public void setPlinewidth(float _plinewidth) {
        plinewidth = _plinewidth;
    }
}
