/*
 * ILine.java
 *
 * Created on 15 mars 2005, 10:00
 */

package jastex.obj;

import java.awt.Stroke;

/**
 * Properties common to all lines
 * @author steph
 */
public interface ILine extends IJasObject {

    /**
     * Gray level used to draw lines. 0=black, 1=white.
     * @return plinegray
     */
    public float getPlinegray();
    
    /**
     * Color used to draw line, if null, use plinegray.
     * The color name should be defined in dvipsnam.def
     * and one should include usepackage[usenames]{color}.
     * This is to avoid the trouble of defining our own
     * colors.
     * The drawback is that it is not possible to define and
     * use other colors.
     * It should not be very restrictive since plenty of
     * colours are defined in dvipsnam.def.
     * It should not be difficult to add the possibility
     * of using new colors if needed.
     * @return plinecolor
     */
    public String getPlinecolor();
    
    /**
     * Width in unitlength of lines.
     * @return plinewidth
     */
    public float getPlinewidth();
    
    /**
     * Dash pattern used for drawing postscript paths.
     * The numbers in the list indicate alternatively lengths
     * in unitlength of dashes and lengths in unitlength of spaces. 
     * The list of lenghts is used circularly.
     * Offset allows to start the pattern at some distance from its beginning.
     * Here are some examples:
     *     dash={}{0}    % continuous path
     *     dash={1.5}{0} % dashs of length 1.5 and empty spaces of length 1.5
     *     dash={0.2 0.5}{0} % looks like a sequence of dots
     *     dash={4 1 1 1}{0} % alternation of long and short dashs
     *     dash={1.5}{1.5} % we start with the empty space and not the dash
     *     dash={4}{2}     % we start in the middle of the first dash
     * @return pdash_length
     */
    public float[] getPdash_length();
    
    /**
     * Offset allows to start the pattern at some distance 
     * from its beginning.
     * @return pdash_empty
     */
    public float getPdash_empty();
 
    /**
     * Setter for property pdash_empty.
     * @param pdash_empty New value of property pdash_empty.
     */
    public void setPdash_empty(float pdash_empty);

    /**
     * Setter for property pdash_length.
     * @param pdash_length New value of property pdash_length.
     */
    public void setPdash_length(float[] pdash_length);

    /**
     * Setter for property plinecolor.
     * @param plinecolor New value of property plinecolor.
     */
    public void setPlinecolor(String plinecolor);

    /**
     * Setter for property plinegray.
     * @param plinegray New value of property plinegray.
     */
    public void setPlinegray(float plinegray);

    /**
     * Setter for property plinewidth.
     * @param plinewidth New value of property plinewidth.
     */
    public void setPlinewidth(float plinewidth);
    
    public Stroke getStroke();
}
