/*
 * AbstractBezierEdge.java
 *
 * Created on 1 avril 2005, 23:38
 */

package jastex.obj.graph;

import jastex.GastexGlobalProperties;
import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.geom.CubicCurve2D;
import java.awt.geom.Line2D;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;


/**
 *
 * @author steph
 */
public abstract class AbstractBezierEdge extends AbstractEdge {
    
    // TODO : documentation
    
    protected double p0x, p0y, p1x, p1y, p2x, p2y, p3x, p3y;
    protected CubicCurve2D.Double edgeGeom = new CubicCurve2D.Double();
    
    protected double[] eqnX = new double[4];
    protected double[] eqnY = new double[4];
    
    /** Creates a new instance of AbstractBezierEdge */
    protected AbstractBezierEdge(GastexGlobalProperties _ggp, INode s, INode e) {
        super(_ggp, s, e);
    }
    
    protected void putParamAtPoint(double t, Point2D.Double p) {
        p.x = ((eqnX[3]*t + eqnX[2])*t + eqnX[1])*t + eqnX[0];
        p.y = ((eqnY[3]*t + eqnY[2])*t + eqnY[1])*t + eqnY[0];
    }
    
    public static double getCubicValue(double[] eqn, double t) {
        return ((eqn[3]*t + eqn[2])*t + eqn[1])*t + eqn[0];
    }
    
    public static double getDiffCubicValue(double[] eqn, double t) {
        return (3*eqn[3]*t + 2*eqn[2])*t + eqn[1];
    }
    
    protected abstract void updateControlPoints();
    
    protected void updateEquations() {
        eqnX[0] = p0x;
        eqnX[1] = 3*(p1x-p0x);
        eqnX[2] = 3*(p0x-2*p1x+p2x);
        eqnX[3] = 3*(p1x-p2x)+p3x-p0x;
        eqnY[0] = p0y;
        eqnY[1] = 3*(p1y-p0y);
        eqnY[2] = 3*(p0y-2*p1y+p2y);
        eqnY[3] = 3*(p1y-p2y)+p3y-p0y;
    }
    
    /** adjust the curve depending on the angle and the length
     * perform a dichotomy and trnaslate it to the intersection point
     */
    protected void updateCurve () {
        // reinitiate the curve
        updateControlPoints();
        
        // compute the coeffs of the parametric representation
        updateEquations();
        
        // get the two intersections
        // TODO : gérer les marqueurs
	double t1 = getIntersection(startingpoint.getExternalShape(), true);
	double t2 = getIntersection(endingpoint.getExternalShape(), false);

        // adjust the control points
        p0x = getCubicValue(eqnX, t1);
        p0y = getCubicValue(eqnY, t1);
        p3x = getCubicValue(eqnX, t2);
        p3y = getCubicValue(eqnY, t2);
        p1x = p0x + (t2-t1)*getDiffCubicValue(eqnX, t1)/3;
        p1y = p0y + (t2-t1)*getDiffCubicValue(eqnY, t1)/3;
        p2x = p3x + (t1-t2)*getDiffCubicValue(eqnX, t2)/3;
        p2y = p3y + (t1-t2)*getDiffCubicValue(eqnY, t2)/3;
        
        // adjust the curve
        edgeGeom.setCurve(p0x, p0y, p1x, p1y, p2x, p2y, p3x, p3y);
    }

    /**
     * Draw the object
     * @param g the Graphics2D instance on which to draw
     */
    public void draw(Graphics2D g) {
        // draw the curve
        g.setColor(getLineColor());
        g.setStroke(getStroke());
        updateCurve();
        g.draw(edgeGeom);
        
        // draw the arrow heads
        float strokeWidth = 1/(float)g.getTransform().getScaleX();
        g.setStroke(new BasicStroke(strokeWidth));
        updateArrowHeads(edgeGeom);
	for(int i = 0; i < pAHnb; i++) {
	    if (arrowHeads[i] != null) {
		g.fill(arrowHeads[i]);
		g.draw(arrowHeads[i]);
	    }
	}
        
        // TODO: labels
    }
    
    /** Useful to select the object
     *  method provided by edgeGeom is not correct
     */
    public boolean contains(double x, double y) {
	Point2D p = new Point2D.Double(x, y);
	double temp;

	// 0.5 is the precision
	PathIterator fit = edgeGeom.getPathIterator(null, 0.5);
	double[] coords = {0, 0, 0, 0, 0, 0};
	double[] oldcoords = {0, 0, 0, 0, 0, 0};

	if (!fit.isDone()) {
	    fit.currentSegment(oldcoords);
	}
	Line2D.Double actualline;

	while(!fit.isDone()) {
	    fit.currentSegment(coords);
	    actualline = new Line2D.Double((int)oldcoords[0], (int)oldcoords[1], 
					   (int)coords[0], (int)coords[1]);
	    temp = actualline.ptSegDist(p)-4;
	    if (temp < 0.5) return true;
	    oldcoords[0] = coords[0];
	    oldcoords[1] = coords[1];
	    fit.next();
	}
	return false;
    }
}
