/*
 * AbstractEdge.java
 *
 * Created on 25 mars 2005, 15:15
 */

package jastex.obj.graph;

import jastex.GastexGlobalProperties;
import jastex.obj.AbstractLine;
import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.GeneralPath;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;

/**
 *
 * @author sglondu
 */
public abstract class AbstractEdge extends AbstractLine implements IEdge {
    
    // TODO : portée des champs
    // TODO : documentation
    // TODO : flèches, labels
    /** Properties */

    /** ArrowHead */
    protected GeneralPath[] arrowHeads;

    /** number of arrowhead(s) at the end of edges */
    protected int pAHnb;

    /** distance between two arrowheads */
    protected double pAHdist;
    
    /** angle between the edge and the arrowhead side */
    protected double pAHangle;
    
    /**  Length of the arrowhead side */
    protected double pAHLength;
    
    /** length defining the shape of the arrowhead */
    protected double pAHlength;
    
    /** Starting point x offset */
    protected double psxo;
    
    /** Starting point y offset */
    protected double psyo;

    /** Ending point x offset */
    protected double pexo;
    
    /** Ending point y offset */
    protected double peyo;

    /** the starting node of the edge */
    protected INode startingpoint;
    
    /** the ending node of the edge
     * null for an imark and equals to starting point for a loop*/
    protected INode endingpoint;
    
    /** the label of the edge */
    // TODO : public EdgeLabel label;

    /** the position of the label */
    // TODO : les labels
    /*public Point2D curvePosLabel;
    public Point2D curvePosLabel2;*/

    /** set if a label has been recently added */
    /*public boolean newlabeladded=false;
    public boolean marked;*/

    /** Creates a new instance of AbstractEdge */
    protected AbstractEdge(GastexGlobalProperties _ggp, INode s, INode e) {
        super(_ggp);
        startingpoint = s;
        endingpoint = e;
    }
    
    public void setFromGgp(GastexGlobalProperties _ggp){
	super.setFromGgp(_ggp);
	pAHnb = _ggp.pAHnb;
	pAHdist = _ggp.pAHdist;
	pAHangle = _ggp.pAHangle;
	pAHLength = _ggp.pAHLength;
	pAHlength = _ggp.pAHlength;
 	psxo = _ggp.psxo;
	psyo = _ggp.psyo;
	pexo = _ggp.pexo;
	peyo = _ggp.peyo;
    }
    
    /** Return the Gastex Code associated with the current fields values
     * @return a String containing the properties modified compare to 
     * the current GastexGlobalProperties, beginning with a comma or 
     * the empty string ("")
     */
    public String getLatexCodeProperties() {
	String lProp = super.getLatexCodeProperties();
        
	/* TODO if (label != null) {
	    lProp += label.getLatexCodeProperties(false);
	} */
	if (pAHnb     != globalProp.pAHnb)      lProp += ",AHnb="+pAHnb;
	if (pAHdist   != globalProp.pAHdist)    lProp += ",AHdist="+pAHdist;
	if (pAHangle  != globalProp.pAHangle)   lProp += ",AHangle="+pAHangle;	
	if (pAHLength != globalProp.pAHLength)  lProp += ",AHLength="+pAHLength;
        if (pAHlength != globalProp.pAHlength)  lProp += ",AHlength="+pAHlength;
	if (psxo != globalProp.psxo)            lProp += ",sxo="+psxo;
	if (psyo != globalProp.psyo)            lProp += ",syo="+psyo;
	if (pexo != globalProp.pexo)            lProp += ",exo="+pexo;
	if (peyo != globalProp.peyo)            lProp += ",eyo="+peyo;
	
	return lProp;
    }
       
    protected abstract void putParamAtPoint(double t, Point2D.Double p);
    
    /** dichotomy
     * assumes eqnX and eqnY are up to date
     */
    protected double getIntersection(Shape node, boolean isStarting) {
        Point2D.Double cur = new Point2D.Double();
        double begin=0, end=1, middle;
        int i;
        
        for(i=0; i<10; i++) {
            middle = (begin+end)/2;
            putParamAtPoint(middle, cur);
            if (node.contains(cur) != isStarting) {
                end = middle;
            } else {
                begin = middle;
            }
        }
        
        return isStarting ? end : begin;
    }

    /** get the angle made by two points and a horizontal line
     * @param pt1 the first point used to calculate the angle
     * @param pt2 the second point used to calculate the angle
     * @return an angle in radian
     */
    public static double getAngle(Point2D pt1, Point2D pt2) {
	double x, y, slope;

	x = pt2.getX() - pt1.getX();
	y = pt2.getY() - pt1.getY();

	if (x == 0) {
            // FIXME
            slope = (y >= 0) ? 1000 : -1000;
	} else
            slope = y/x;
        
	if (x < 0)
            return Math.atan(slope) + Math.PI;
	else
            return Math.atan(slope);
    }

    /** create an array of polygon containing the arrow heads
     * and store the result in arrowHeads */
    protected void updateArrowHeads(Shape curve) {
	arrowHeads = null;

        double angle;
	double delta, sx, sy;
        
        AffineTransform afr;
	Point2D[] ptSrc, ptDst;
        
	// FIXME: 0.5 is the precision ?
        PathIterator fit = curve.getPathIterator(null, 0.5);

	int itNb = 0;
	double[] coords = {0, 0, 0, 0, 0, 0};
	Point2D[] tab = new Point2D[1024];
        
	// get the coordinate of the potential spikes of the arrows
	while(!fit.isDone() && itNb < 1024) {
	    fit.currentSegment(coords);
	    tab[itNb++] = new Point2D.Double(coords[0], coords[1]);
	    fit.next();
	}
        
	arrowHeads = new GeneralPath[pAHnb];
	ptSrc = new Point2D[4];
	ptDst = new Point2D[4];
	// FIXME: the minus is due to the fact that Java turns in ClockWise
	delta = pAHLength * Math.sin(pAHangle*Math.PI/180);
	int index = 0;
 	Point2D pt = tab[itNb-1];

	for (int i=0; i < pAHnb; i++) {
	    double distRest, dist;
	    if (i != 0) {
		distRest = pAHdist;
		if (index >= (itNb-1)) {
		    return;
		}
		dist = pt.distance(tab[itNb-1-(index+1)]);
		while (dist < distRest) {
		    distRest -= dist;
		    index++;
		    if (index >= (itNb-1)) {
			return;
		    }
		    pt = tab[itNb-1-index];
		    dist = pt.distance(tab[itNb-1-(index+1)]);
		}
		if (distRest != 0) {
		    double alpha = getAngle(tab[itNb-1-index],
					    tab[itNb-1-(index+1)]);
		    pt = new Point2D.Double(
                            pt.getX() + distRest * Math.cos(alpha),
                            pt.getY() + distRest * Math.sin(alpha));
		}
	    }
	    ptSrc[0] = pt;
	    sx = ptSrc[0].getX();
	    sy = ptSrc[0].getY();
	    
	    ptSrc[1] = new Point2D.Double(sx + pAHlength, sy);
	    ptSrc[2] = new Point2D.Double(sx + (pAHLength * Math.cos(pAHangle*Math.PI/180)),
                                          sy + delta);
	    ptSrc[3] = new Point2D.Double(ptSrc[2].getX(), sy - delta);
	    
	    afr = new AffineTransform();

	    /** get the best angle to put the head and the tail on the curve */
	    int index2 = 1;
	    while (itNb-1-(index+index2) > 0 &&
		   tab[itNb-1-index].distance(tab[itNb-1-(index+index2)]) 
		   < pAHlength) {
		index2++; 
	    }
	    angle = getAngle(tab[itNb-1-index], tab[itNb-1-(index+index2)]);
	    afr.setToRotation(angle, sx, sy);

	    afr.transform(ptSrc, 0, ptDst, 0, 4);

	    arrowHeads[i] = new GeneralPath(GeneralPath.WIND_EVEN_ODD, 4);
	    arrowHeads[i].moveTo((float)ptDst[0].getX(), (float)ptDst[0].getY());
	    arrowHeads[i].lineTo((float)ptDst[2].getX(), (float)ptDst[2].getY());
	    arrowHeads[i].lineTo((float)ptDst[1].getX(), (float)ptDst[1].getY());
	    arrowHeads[i].lineTo((float)ptDst[3].getX(), (float)ptDst[3].getY());
            arrowHeads[i].closePath();
	}
    }
}
