/*
 * AbstractNode.java
 *
 * Created on 18 mars 2005, 14:34
 */

package jastex.obj.graph;
import jastex.*;
import jastex.obj.*;
import java.util.Vector;

/**
 *
 * @author sglondu
 */
public abstract class AbstractNode extends AbstractClosedLine implements INode, Cloneable {    
    
    // TODO : portée des champs
    // TODO : documentation
    
    /** Field used to generated an unique  name for each node */
    static public int nodeNum = 0;

    double x, y;
    
    /** Gastex Properties */
    /** Adjust the node on the label either by height or width or both */
    public int pNadjust;     // any combination of the number 0,1,2 by sum
                      //   1 = width, 2 = height, 0 = no adjust
    /** Distance between the label box and the node frame when using Nadjust */
    public double pNadjustdist; // distance in unitlength between the label box 
                      //   and the node frame when using Nadjust
    /**  boolean flag defining whether nodes are filled or not */
    /** Marks on the node (initial and/or final and/or repeated) */
    public int pNmarks;      // any combination of the number 0,1,2,4 by sum
                      //    1 : initial mark on nodes
                      //    2 : final mark on nodes
                      //    4 : repeated mark on nodes
                      //    0 : no mark on nodes

    
    /** A vector containing all the final/initial marks */
    public Vector<FMark> fiMarks;
    /** A vector containing all the repeated marks */
    public Vector<RMark> rMarks;

    /** the Gastex internal name of the node */
    public String nodeName;

    /** A vector containing all the labels of the node */
    public Vector nodeLabels;

    /** A vector containing all the edges related to the Node */
    // FIXME public Vector vEdges;

    /** Creates a new instance of AbstractNode */
    protected AbstractNode(GastexGlobalProperties _ggp, double _x, double _y) {
        super(_ggp);
        // TODO : arranger ça avec vraie gestion des marques, labels
        // FIXME viMarks = new Vector();
	// FIXME vfMarks = new Vector();
	rMarks = new Vector<RMark>();
        fiMarks = new Vector<FMark>();
	// FIXME vEdges = new Vector();
	nodeLabels = new Vector();
        x = _x;
        y = _y;
	nodeName = "n" + nodeNum;
	nodeNum++;
    }

    public void setFromGgp(GastexGlobalProperties _ggp){
	super.setFromGgp(_ggp);
	pNmarks = _ggp.pNmarks;	
	pNadjust = _ggp.pNadjust;
	pNadjustdist = _ggp.pNadjustdist;
    }

    /** Generated the Gastex Properties Code 
     * associated with the current fields values, and which can include
     * also the code associated with one initial and/or final mark,
     * and the code associated with the first label
     * @param bestIMark the index in the initial mark vector which can be
     * include in the properties
     * @param bestFMark the index in the final mark vector which can be
     * include in the properties
     * @param bestRMark the index in the repeated mark vector which can be
     * include in the properties     
     * @return a String containing the properties modified compare to 
     * the current GastexGlobalProperties, surrounded by '[' and ']'
     * if isStandAlone is true, else a beginning by a common 
     * or the empty string ("")
     */
    protected String getLatexCodeProperties(/*int bestIMark, int bestFMark, int bestRMark*/) {
	String lProp = super.getLatexCodeProperties();
	String firstLabelProp = "";
	String bestIMarkProp = "";
	String bestFMarkProp = "";
	String bestRMarkProp = "";

	int pNmarks = 0;

	// TODO : gérer les marqueurs
        /*
        if (nodeLabels.size() > nodeLabelIndex && nodeLabelIndex >= 0) {
	    NodeLabel nl = (NodeLabel)nodeLabels.elementAt(nodeLabelIndex);
	    if (nl.getGgp() == globalProp) {
		firstLabelProp = nl.getLatexCodeProperties(false);
	    }
	}
	if (bestIMark != -1) {
	    IFMark imark = (IFMark)viMarks.elementAt(bestIMark);
	    bestIMarkProp = imark.getLatexCodeProperties(false);
	    pNmarks |= 1;
	}
	if (bestFMark != -1) {
	    IFMark fmark = (IFMark)vfMarks.elementAt(bestFMark);
	    bestFMarkProp = fmark.getLatexCodeProperties(false);
	    pNmarks |= 2;
	}
	if (bestRMark != -1) {
	    RMark rmark = (RMark)vrMarks.elementAt(bestRMark);
	    bestRMarkProp = rmark.getLatexCodeProperties(false);
	    pNmarks |= 4;
	}
	lProp += firstLabelProp + bestIMarkProp + bestFMarkProp;*/

	if (pNadjust != globalProp.pNadjust) {
	    lProp += ",Nadjust=";
	    if (pNadjust == 0) {
		lProp += "n";
	    } else {
		if ((pNadjust & 1) != 0) {
		    lProp += "w";
		}
		if ((pNadjust & 2) != 0) {
		    lProp += "h";
		}
	    }
	}
	if (pNadjustdist != globalProp.pNadjustdist) {
            lProp += ",Nadjustdist="+pNadjustdist;
	}

        if (pNmarks != globalProp.pNmarks) {
	    lProp += ",Nmarks=";
	    if (pNmarks == 0) {
		lProp += "n";
	    } else {
		if ((pNmarks & 1) != 0) {
		    lProp += "i";
		}
		if ((pNmarks & 2) != 0) {
		    lProp += "f";
		}
		if ((pNmarks & 4) != 0) {
		    lProp += "r";
		}
	    }
	}

        return lProp;
    }
    
    /* There is nothing interesting below... */
    
    public double getX() { return x; }
    public double getY() { return y; }
    public String getNodeName() { return nodeName; }
    
    public void setPosition(double _x, double _y) {
        x = _x;
        y = _y;
    }
        
    public INode clone() {
        try {
            return (INode)super.clone();
        } catch (CloneNotSupportedException e) {
            return null;
        }
    }
}
