/*
 * IFMark.java
 *
 * Created on 22 avril 2005, 15:21
 */

package jastex.obj.graph;

import jastex.GastexGlobalProperties;
import java.awt.BasicStroke;
import java.awt.Graphics2D;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;

/**
 *
 * @author sglondu
 */
public class FMark extends AbstractEdge {
    
    protected INode node;
    protected double angle;
    protected double length;
    
    protected Line2D.Double edgeGeom = new Line2D.Double();
    protected double[] eqnX = new double[2];
    protected double[] eqnY = new double[2];
    
    /** Creates a new instance of IFMark */
    public FMark(GastexGlobalProperties _ggp, INode n, boolean initial) {
        super(_ggp, initial ? n : null, initial ? null : n);
        node = n;
    }
    
    public void setFromGgp(GastexGlobalProperties _ggp){
        super.setFromGgp(_ggp);
        angle = (startingpoint == null) ? _ggp.pfangle : _ggp.piangle;
        length = (startingpoint == null) ? _ggp.pflength : _ggp.pilength;
    }
    
    protected void updateEquations() {
        double srcX, srcY, dstX, dstY;
        double angle_rad = angle*Math.PI/180;
        
        // FIXME
        double length = 1000;
        
        if (startingpoint == null) {
            dstX = endingpoint.getX();
            dstY = endingpoint.getY();
            srcX = dstX + length * Math.cos(angle_rad);
            srcY = dstY + length * Math.sin(angle_rad);
        } else {
            srcX = startingpoint.getX();
            srcY = startingpoint.getY();
            dstX = srcX + length * Math.cos(angle_rad);
            dstY = srcY + length * Math.sin(angle_rad);
        }
        
        eqnX[0] = srcX;
        eqnX[1] = dstX - srcX;
        eqnY[0] = srcY;
        eqnY[1] = dstY - srcY;
    }
    
    protected void putParamAtPoint(double t, Point2D.Double p) {
        p.x = eqnX[1]*t + eqnX[0];
        p.y = eqnY[1]*t + eqnY[0];
    }
    
    /** adjust the curve depending on the angle and the length
     * perform a dichotomy and trnaslate it to the intersection point
     */
    protected void updateCurve () {
        // compute the coeffs of the parametric representation
        updateEquations();
        
        // adjust the curve
        double t, angle_rad = angle*Math.PI/180;
        if (startingpoint == null) {
            t = getIntersection(endingpoint.getExternalShape(), false);
            edgeGeom.x2 = eqnX[1]*t + eqnX[0];
            edgeGeom.y2 = eqnY[1]*t + eqnY[0];
            edgeGeom.x1 = edgeGeom.x2 + length * Math.cos(angle_rad);
            edgeGeom.y1 = edgeGeom.y2 + length * Math.sin(angle_rad);
        } else {
            t = getIntersection(startingpoint.getExternalShape(), true);
            edgeGeom.x1 = eqnX[1]*t + eqnX[0];
            edgeGeom.y1 = eqnY[1]*t + eqnY[0];
            edgeGeom.x2 = edgeGeom.x1 + length * Math.cos(angle_rad);
            edgeGeom.y2 = edgeGeom.y1 + length * Math.sin(angle_rad);
        }
    }

    /**
     * Draw the object
     * @param g the Graphics2D instance on which to draw
     */
    public void draw(Graphics2D g) {
        // draw the curve
        g.setColor(getLineColor());
        g.setStroke(getStroke());
        updateCurve();
        g.draw(edgeGeom);
        
        // draw the arrow heads
        float strokeWidth = 1/(float)g.getTransform().getScaleX();
        g.setStroke(new BasicStroke(strokeWidth));
        updateArrowHeads(edgeGeom);
	for(int i = 0; i < pAHnb; i++) {
	    if (arrowHeads[i] != null) {
		g.fill(arrowHeads[i]);
		g.draw(arrowHeads[i]);
	    }
	}
        
        // TODO: labels
    }
    
    public String getLatexCode(double oriX, double oriY) {
        // TODO
        return "% TODO [FIMark]\n";
    }
}
