/*
 * StdEdge.java
 *
 * Created on 25 mars 2005, 20:03
 */

package jastex.obj.graph;

import jastex.GastexGlobalProperties;
import jastex.gui.IEditable;
import jastex.gui.JasObjectToolBox;
import jastex.gui.MouseablePoint;
import jastex.obj.tabs.JasLineTab;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Graphics2D;

import java.awt.geom.Line2D;


/**
 *
 * @author steph
 */
public class StdEdge extends AbstractBezierEdge implements IEditable {
    
    // TODO : portée des champs
    // TODO : documentation
    
    /** Curved Edge
     * depth in unitlength of curvededges between two nodes
     * The absolute value of curvedepth defines
     * the distance between the middle of the curved edge
     * and the center of the line joining the two nodes
     * With a positive/negative value the curved edge
     * is on the left/right of the line joining the two nodes.
     */
    protected double pcurvedepth = 0;
    protected double ctrlx, ctrly;
    protected MouseablePoint[] mouseablePoints;

    /** Creates a new instance of StdEdge */
    public StdEdge(GastexGlobalProperties _ggp, INode s, INode e) {
        super(_ggp, s, e);
        
        final StdEdge me = this;
        mouseablePoints = new MouseablePoint[1];
        
        mouseablePoints[0] = new MouseablePoint() {
            public void onDrag(double x, double y) {
                // compute new curvedepth
                double srcX = me.startingpoint.getX();
                double srcY = me.startingpoint.getY();
                double vectX = me.endingpoint.getX()-srcX;
                double vectY = me.endingpoint.getY()-srcY;
                double res = vectX*(y-srcY) - vectY*(x-srcX);
                res = res / Math.sqrt(vectX*vectX + vectY*vectY);
                res = res / 2;
                if (granularity > 0) {
                    res = ((int)(res/granularity + 0.5)) * granularity;
                }
                me.pcurvedepth = res;
            }
        };
    }
    
    public StdEdge(GastexGlobalProperties _ggp, INode s, INode e, double depth) {
        this(_ggp, s, e);
        pcurvedepth = depth;
    }

    /** Set the internal value to the one of the GastexGlobalProperties
     * @param _ggp the GastexGlobaProperties used to initialize values
     * @see GastexGlobalProperties
     */ 
    public void setFromGgp(GastexGlobalProperties _ggp){
	super.setFromGgp(_ggp);
	pcurvedepth = _ggp.pcurvedepth;
    }
    
    protected void updateControlPoints() {
        // end coordinates
        p0x = startingpoint.getX();
        p0y = startingpoint.getY();
        p3x = endingpoint.getX();
        p3y = endingpoint.getY();
        
        // normalized orthogonal vector
        double vx = p0y-p3y, vy = p3x-p0x;
        double norm = Math.sqrt(vx*vx + vy*vy);
        vx /= norm; vy /= norm;
        
        // the single control point coordinates
        ctrlx = (p0x+p3x)/2 + vx*2*pcurvedepth;
        ctrly = (p0y+p3y)/2 + vy*2*pcurvedepth;
        
        // the adapted cubic control points
        p1x = p0x/3 + ctrlx*2/3;
        p1y = p0y/3 + ctrly*2/3;
        p2x = p3x/3 + ctrlx*2/3;
        p2y = p3y/3 + ctrly*2/3;
    }

    public String getLatexCodeProperties() {
	String lProp = super.getLatexCodeProperties();
	if (pcurvedepth != globalProp.pcurvedepth) {
            lProp += ",curvedepth="+pcurvedepth;
        }
        return lProp;
    }
    
    public String getLatexCode(double oriX, double oriY) {
        String lCode = "\\drawedge";
        lCode += formatLatexCodeProperties();
        lCode += "(" + startingpoint.getNodeName();
        lCode += "," + endingpoint.getNodeName() + "){";
        // TODO : labels, flèches
        lCode += "}";
	return lCode;
    }

    public void updateMouseablePoints() {
        updateControlPoints();
        mouseablePoints[0].curX = ctrlx;
        mouseablePoints[0].curY = ctrly;
    }
    
    public void drawSketch(Graphics2D g) {
        // set up the stroke
        float strokeWidth = 1/(float)g.getTransform().getScaleX();
        g.setStroke(new BasicStroke(strokeWidth));
        
        // update control points and mouseable points
        updateMouseablePoints();
        
        // set up the stroke
        float dash_length = 4*strokeWidth;
	g.setStroke(new BasicStroke(
            strokeWidth,
            BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL,
	    0, new float[] {dash_length, dash_length}, 0));
        
        // draw the whole curve
        g.setColor(Color.RED);
        edgeGeom.setCurve(p0x, p0y, p1x, p1y, p2x, p2y, p3x, p3y);
        g.draw(edgeGeom);
        
        // draw sketch lines
        Line2D.Double line = new Line2D.Double();
        g.setColor(Color.BLUE);
        line.setLine(p0x, p0y, ctrlx, ctrly); g.draw(line);
        line.setLine(ctrlx, ctrly, p3x, p3y); g.draw(line);
        line.setLine(p0x, p0y, p3x, p3y); g.draw(line);
        line.setLine(ctrlx, ctrly, (p0x+p3x)/2, (p0y+p3y)/2); g.draw(line);
        
        // draw mouseable point
        g.setStroke(new BasicStroke(strokeWidth));
        for (int i=0; i<1; i++) {
            MouseablePoint mp = mouseablePoints[i];
            mp.updateGeom(strokeWidth);
            g.setColor(Color.WHITE);
            g.fill(mp);
            g.setColor(Color.BLUE);
            g.draw(mp);
        }
    }

    public void updateToolBox(JasObjectToolBox tb) {
        JasLineTab tab = new JasLineTab();
        tab.updateContent(this);
        tb.addTab(tab);
    }
    
    public MouseablePoint[] getMouseablePoints() {
        // TODO        
        return mouseablePoints;
    }
}
