/*
 * StandardNode.java
 *
 * Created on 18 mars 2005, 14:27
 */

package jastex.obj.graph;
import jastex.*;
import jastex.gui.IEditable;
import jastex.gui.JasObjectToolBox;
import jastex.gui.MouseablePoint;
import jastex.obj.tabs.JasLineTab;
import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.RoundRectangle2D;
import java.util.Enumeration;

/**
 *
 * @author sglondu
 */
public class StdNode extends AbstractNode implements IEditable {

    /** width of node */
    double pNw;
    
    /** height of node */
    double pNh;
    
    /** maximal radius of oval nodes 
     * The actual radius will be min(Nw/2,Nh/2,Nmr)
     * Nmr=0 yields a rectangle node
     */
    double pNmr;
    
    /** the internal representation of the node */
    private RoundRectangle2D nodeGeom; // FIXME = new RoundRectangle2D.Double();

    protected MouseablePoint[] mouseablePoints;/* = {
        new MouseablePoint(), new MouseablePoint(),
        new MouseablePoint(), new MouseablePoint()                
    };*/
    
    /** Creates a new instance of StandardNode */
    public StdNode(GastexGlobalProperties _ggp, double _x, double _y) {        
        super(_ggp, _x, _y);
        final StdNode me = this;
        
        // TODO : déterminer min_dim de manière plus intelligente
        final double min_dim = 5;
        mouseablePoints = new MouseablePoint[5];
        
        mouseablePoints[0] = new MouseablePoint() {
            public void onDrag(double x, double y) {
                x += dx;
                y += dy;
                if (granularity > 0) {
                    x = ((int)(x/granularity + 0.5)) * granularity;
                    y = ((int)(y/granularity + 0.5)) * granularity;
                }
                me.x = x;
                me.y = y;
            }
        };
        mouseablePoints[1] = new MouseablePoint(Cursor.SW_RESIZE_CURSOR) {
            public void onDrag(double x, double y) {
                if (granularity > 0) {
                    x = ((int)(x/granularity + 0.5)) * granularity;
                    y = ((int)(y/granularity + 0.5)) * granularity;
                }
                pNw = Math.max( me.x+pNw/2-x, min_dim);
                pNh = Math.max( me.y+pNh/2-y, min_dim);
            }
        };
        mouseablePoints[2] = new MouseablePoint(Cursor.NW_RESIZE_CURSOR) {
            public void onDrag(double x, double y) {
                if (granularity > 0) {
                    x = ((int)(x/granularity + 0.5)) * granularity;
                    y = ((int)(y/granularity + 0.5)) * granularity;
                }
                pNw = Math.max( me.x+pNw/2-x, min_dim);
                pNh = Math.max(-me.y+pNh/2+y, min_dim);
            }
        };
        mouseablePoints[3] = new MouseablePoint(Cursor.NE_RESIZE_CURSOR) {
            public void onDrag(double x, double y) {
                if (granularity > 0) {
                    x = ((int)(x/granularity + 0.5)) * granularity;
                    y = ((int)(y/granularity + 0.5)) * granularity;
                }
                pNw = Math.max(-me.x+pNw/2+x, min_dim);
                pNh = Math.max(-me.y+pNh/2+y, min_dim);
            }
        };
        mouseablePoints[4] = new MouseablePoint(Cursor.SE_RESIZE_CURSOR) {
            public void onDrag(double x, double y) {
                if (granularity > 0) {
                    x = ((int)(x/granularity + 0.5)) * granularity;
                    y = ((int)(y/granularity + 0.5)) * granularity;
                }
                pNw = Math.max(-me.x+pNw/2+x, min_dim);
                pNh = Math.max( me.y+pNh/2-y, min_dim);
            }
        };
    }
    
    public void setFromGgp(GastexGlobalProperties _ggp){
	super.setFromGgp(_ggp);
	pNw = _ggp.pNw;
	pNh = _ggp.pNh;
	pNmr = _ggp.pNmr;
        updateGeom();
    }

    /** calculate the node internal geometry */
    private void updateGeom() {
        if (nodeGeom == null) nodeGeom = new RoundRectangle2D.Double();
	nodeGeom.setRoundRect(x-pNw/2, y-pNh/2, pNw, pNh, pNmr*2, pNmr*2);
    }
    
    public void draw(Graphics2D g) {
        // draw repeated marks
        for (Enumeration<RMark> e = rMarks.elements() ; e.hasMoreElements() ;) {
            e.nextElement().draw(g);
        }
        
        // update the nodeGeom for the node
        updateGeom();

        // fill the node
        Color fillColor = getFillColor();
        if (fillColor != null) {
	    g.setColor(fillColor);
            g.fill(nodeGeom);
	}

        // set up the stroke
        g.setStroke(getStroke());

        // set up the frame
        if (pNframe) {
            g.setColor(getLineColor());
        } else {
            g.setColor(Color.lightGray);
        }

        // draw the node
        g.draw(nodeGeom);
        
        // draw initial/final marks
        for (Enumeration<FMark> e = fiMarks.elements() ; e.hasMoreElements() ;) {
            e.nextElement().draw(g);
        }
        // TODO : marqueurs labels
    }

    public void drawRMark(Graphics2D g, RMark mark) {
        
        // update the nodeGeom for the marker
        float dist = mark.prdist;
        nodeGeom.setRoundRect(
                x-pNw/2-dist, y-pNh/2-dist,
                pNw+2*dist, pNh+2*dist,
                (pNmr+dist)*2, (pNmr+dist)*2);

        // set up the stroke
        g.setStroke(mark.getStroke());

        // set up the frame
        if (pNframe) {
            g.setColor(mark.getLineColor());
        } else {
            g.setColor(Color.lightGray);
        }
        
        // draw the node
        g.draw(nodeGeom);        
    }

    public void updateMouseablePoints() {
        mouseablePoints[0].setPosition(x, y);
        mouseablePoints[0].setFrame(x-pNw/2, y-pNh/2, pNw, pNh);
        mouseablePoints[1].setPosition(x-pNw/2, y-pNh/2);
        mouseablePoints[2].setPosition(x-pNw/2, y+pNh/2);
        mouseablePoints[3].setPosition(x+pNw/2, y+pNh/2);
        mouseablePoints[4].setPosition(x+pNw/2, y-pNh/2);
    }
    
    public void drawSketch(Graphics2D g) {
        updateMouseablePoints();
        
        // set up the stroke
        float strokeWidth = 1/(float)g.getTransform().getScaleX();
        g.setColor(Color.BLUE);
        
        // draw the bounding box
        float dash_length = 4*strokeWidth;
	g.setStroke(new BasicStroke(
            strokeWidth,
            BasicStroke.CAP_BUTT, BasicStroke.JOIN_BEVEL,
	    0, new float[] {dash_length, dash_length}, 0));
        g.draw(mouseablePoints[0]);
        
        // draw mouseable points
        g.setStroke(new BasicStroke(strokeWidth));
        for (int i=1; i<5; i++) {
            MouseablePoint mp = mouseablePoints[i];
            mp.updateGeom(strokeWidth);
            g.setColor(Color.WHITE);
            g.fill(mp);
            g.setColor(Color.BLUE);
            g.draw(mp);
        }        
    }
    
    public void updateToolBox(JasObjectToolBox tb) {
        JasLineTab tab = new JasLineTab();
        tab.updateContent(this);
        tb.addTab(tab);
    }

    /** Generated the Gastex Properties Code 
     * associated with the current fields values, and which can include
     * also the code associated with one initial and/or final mark,
     * and the code associated with the first label
     * @param bestIMark the index in the initial mark vector which can be
     * include in the properties
     * @param bestFMark the index in the final mark vector which can be
     * include in the properties
     * @param bestRMark the index in the repeated mark vector which can be
     * include in the properties     
     * @return a String containing the properties modified compare to 
     * the current GastexGlobalProperties, surrounded by '[' and ']'
     * or the empty string ("")
     */
    public String getLatexCodeProperties() {
	String lProp = super.getLatexCodeProperties();
        // TODO: marqueurs
        if (pNw != globalProp.pNw) lProp += ",Nw="+pNw;
	if (pNh != globalProp.pNh) lProp += ",Nh="+pNh;
	if (pNmr != globalProp.pNmr) lProp += ",Nmr="+pNmr;
        return lProp;
    }

    public String getLatexCode(double oriX, double oriY) {
	String lCode = "\\node";
        lCode += formatLatexCodeProperties();
        lCode += "(" + nodeName + ")(" + (x-oriX) + "," + (y-oriY) + "){";
        // TODO : labels, marqueurs
        lCode += "}";
        return lCode;
    }
    
    public Shape getExternalShape() {
        float max = 0, tmp;
        
        for (Enumeration<RMark> e = rMarks.elements() ; e.hasMoreElements() ;) {
            RMark m = e.nextElement();
            if ((tmp = m.getExternalRadius()) > max) {
                max = tmp;
                float dist = m.prdist;
                nodeGeom.setRoundRect(
                    x-pNw/2-dist, y-pNh/2-dist,
                    pNw+2*dist, pNh+2*dist,
                    (pNmr+dist)*2, (pNmr+dist)*2);
            }
        }
        
        return nodeGeom;
    }
    
    /* There is nothing interesting below... */
    
    public void setSize(double w, double h) {
        pNw = w;
        pNh = h;
    }
    
    public MouseablePoint[] getMouseablePoints() {
        return mouseablePoints;
    }
    
    public boolean contains(double x, double y) {
        return nodeGeom.contains(x, y);
    }
}
