package jastex.obj.tabs;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.event.*;

import jastex.*;
import jastex.obj.ILine;
import jastex.old.gui.JasColorComboBox;
import jastex.gui.JasScrollBox;
import jastex.gui.JasComboBox;
import jastex.gui.JasTab;
import jastex.obj.IClosedLine;
import jastex.obj.IJasObject;



public class JasLineTab extends JasTab {
    
    protected IJasObject object;
    
    /** the gastex 'linewidth' propertie */
    public JasScrollBox lineWidth;
    /** the gastex 'linegray' propertie */
    public JLabel linegray ;
    /** selected if the line is in grayscale */
    public JRadioButton wbline ;
    public JSlider wbLevel ;
    /** selected if the line is in color */
    public JRadioButton colorline;
    /** the gastex 'colorline' propertie */
    public JasColorComboBox colorlist;
    /** the gastex 'dash' propertie */
    public JasComboBox linelist;
    /** the gastex 'Nframe' propertie */
    public JCheckBox nframeCheckBox;

    public JasScrollBox curve;

    public JCheckBox polarCheckBox;
    public float[] dash0 = null;
    public float[] dash1 = {2,2,2,3};
    public float[] dash2 = {5,3};
    public float[] dash3 = {9,3};
    public float[] dash4 = {13,3,5,3};
    

    public String[] lineStrings = { "dash_0","dash_1","dash_2","dash_3","dash_4" };
    public JasDash[] dash = {
        new JasDash(dash0,0), new JasDash(dash1,0),
        new JasDash(dash2,0), new JasDash(dash3,0),
        new JasDash(dash4,0)
    };


    /** Constructor : Instanciate a new Tab for the Line
     * @param title : the tab caption
     * @param object : the JasObject filling the Tab field
     */
    public JasLineTab()
    {
        super(Languages.tb_tabtitle[0]);
        
	nframeCheckBox = new JCheckBox(Languages.tb_lineTab[0]);
        polarCheckBox = new JCheckBox("Coordonnées polaires");

	curve = new JasScrollBox(0, 0.01);
	lineWidth = new JasScrollBox(0.1, 0.01);
	linegray = new JLabel();

	wbLevel = new JSlider(JSlider.HORIZONTAL, 0, 10, 10);
	wbLevel.setMajorTickSpacing(5);
	wbLevel.setMinorTickSpacing(1);
	wbLevel.setPaintTicks(true) ;
        wbLevel.addChangeListener(new ChangeListener(){
            public void stateChanged(ChangeEvent e)
            {
                linegray.setText(java.lang.Double.toString((wbLevel.getValue())/10.0));
            }
        });

	colorline = new JRadioButton(Languages.tb_lineTab[2]);
	colorline.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e)
            {
                wbline.setSelected(false);
                wbLevel.setEnabled(false);
                colorlist.setEnabled(true);
            }
        });
        
	colorlist = new JasColorComboBox(colorStrings, colorCaption);
	linelist = new JasComboBox(lineStrings, dash);

	wbline = new JRadioButton(Languages.tb_lineTab[1]);
	wbline.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e)
            {
                colorline.setSelected(false);
                colorlist.setEnabled(false);
                wbLevel.setEnabled(true);
                if(linegray.getText().length() == 0) {
		    linegray.setText("1.0");
		}
            }
        });
        
	// FIXME nframeCheckBox et polarCheckBox
        insertProperty(polarCheckBox);
        insertProperty(nframeCheckBox);
	insertProperty(Languages.tb_lineTab[3], lineWidth);
	insertProperty("Curve depth", curve);
	insertProperty(wbline);
	insertProperty(wbLevel, linegray);
	insertProperty(colorline);
	insertProperty(colorlist);
	insertPropertie(Languages.tb_lineTab[4]);
	insertProperty(linelist);

    }
    

    /** Save the fields in the ggp given in parameter
    * @param ggp the target GastexGlobalProperties 
    */
    public GastexGlobalProperties saveToGgp(GastexGlobalProperties ggp)
    {
        if (ggp == null) ggp = new GastexGlobalProperties();
	ggp.pcurvedepth = curve.getValue();
        ggp.pNframe = nframeCheckBox.isSelected();
	
        // FIXME
        if(wbline.isSelected()) {
	    ggp.plinegray = Double.parseDouble(linegray.getText());
	} else if(colorline.isSelected()) {
	    ggp.plinecolor = colorlist.getSelected();
	}
	
	ggp.plinewidth = lineWidth.getValue();
	JasDash tmp = linelist.getSelectedDash();
	ggp.setDashIndex(linelist.getSelectedIndex());
	ggp.setpdash_length(tmp.getDashLength());
	ggp.setpdash_empty(tmp.getDashEmpty());
        return ggp;
    }

    public void apply()
    {
        object.setFromGgp(saveToGgp(new GastexGlobalProperties(object.getGgp())));
    }
    
    /** Update the tab field with the Node object given in parameter
     * @param object the source Node
     */
    public void updateContent(ILine object)
    {
        boolean frame = true;
        
        this.object = (IJasObject)object;
	lineWidth.setValue(object.getPlinewidth());
        
        if (object instanceof IClosedLine) {
            nframeCheckBox.setVisible(true);
            frame = ((IClosedLine)object).getPNframe();
        } else {
            nframeCheckBox.setVisible(false);
        }
        
        if (false /* FIXME object instanceof ???*/) {
            polarCheckBox.setVisible(true);
            // FIXME polarCheckBox.setSelected( ??? );
        } else {
            polarCheckBox.setVisible(false);
        }
        
        if (frame) {
            nframeCheckBox.setSelected(true);            
            if (object.getPlinecolor() != null) {
		colorline.setSelected(true);
		colorlist.setEnabled(true);
		wbline.setSelected(false);
		wbLevel.setEnabled(false);
		colorlist.setSelectedItem(object.getPlinecolor());
	    } else {
		wbline.setSelected(true);
		wbLevel.setEnabled(true);
		colorline.setSelected(false);
		colorlist.setEnabled(false);
		linegray.setText(Double.toString(object.getPlinegray()));
		wbLevel.setValue((int) (object.getPlinegray() * 10));
	    }
        } else {
            nframeCheckBox.setSelected(false);
            wbline.setSelected(false);
            colorline.setSelected(false);
            colorlist.setEnabled(false);
            wbLevel.setEnabled(false);
        }

        linelist.setEnabled(false);
        /* FIXME mise à jour de la dash list
        int n = linelist.comboList.getItemCount();
        if(object.getDashIndex() != -1)
	    linelist.setselectedIndex(object.getDashIndex());
	else 
	    linelist.setselectedIndex(0); */
    }
}
    
