package jastex.old.gui;

import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;

import jastex.*;
import jastex.old.gui.tabs.OJasNodeTab;
import jastex.old.gui.tabs.OJasLineTab;
import jastex.old.gui.tabs.JasIFMarkTab;
import jastex.old.gui.tabs.JasLineOffsetTab;
import jastex.old.gui.tabs.JasLoopTab;
import jastex.old.gui.tabs.JasGGPLabelTab;
import jastex.old.gui.tabs.JasArrowTab;
import jastex.old.obj.JasObject;


/** This class create the toolbox in order to modify
 * GasTexGlobalProperties of a group of object
 */
class JasGGPToolBox extends JasToolBox implements ActionListener {


    /** GGP being modify through the ToolBox */
    public GastexGlobalProperties ggp;
    /** SurfaceDessin sending the object */
    public SurfaceDessin sd; 
    
    public Vector objects;
    public JLabel unitNameLabel;

    public OJasLineTab lt;
    public OJasNodeTab nt;
    public JasGGPLabelTab lbt;
    public JasIFMarkTab ift;
    public JasArrowTab eat;
    public JasLoopTab loopTab;
    public JasLineOffsetTab offsetTab;


    static private int lineTabIndex = 0;
    static private int nodeTabIndex = 1;
    static private int labelTabIndex = 2;
    static private int ifmarkTabIndex = 3;
    static private int arrowTabIndex = 4;
    static private int loopTabIndex = 5;
    static private int offsetTabIndex = 6;

    /** Constructor which initializes a toolBox
     * unvisible and empty
     * @param parentFrame the Parent frame of the ToolBox dialog
     * @param toolBoxTitle the title of the toolbox dialogbox
     */
    public JasGGPToolBox(Frame parentFrame , String toolBoxTitle)
    {
	/* Creation of the JDialog of the death */
	super(parentFrame,toolBoxTitle,true);
	
	
	/* OK Buton */
	JButton okButton = new JButton("OK");
	okButton.setActionCommand("ok_ggp");
	okButton.addActionListener(this);

	/* Apply Buton */
	JButton applyButton = new JButton(Languages.tb_ApplyBtn);
	applyButton.setActionCommand("apply_ggp");
	applyButton.addActionListener(this);

	/* Cancel Buton */
	JButton cancelButton = new JButton("Annuler");
	cancelButton.setActionCommand("cancel_ggp");
	cancelButton.addActionListener(this);

	unitNameLabel = new JLabel();
	
	JPanel buttonPane = new JPanel(); 
	buttonPane.setLayout(new BoxLayout(buttonPane, BoxLayout.X_AXIS));
	//buttonPane.add(unitNameLabel);
	buttonPane.add(Box.createHorizontalGlue());
	buttonPane.add(okButton);
	buttonPane.add(applyButton);
	buttonPane.add(cancelButton);

	
	GridBagLayout layout = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	c.fill = GridBagConstraints.BOTH;	
	getContentPane().setLayout(layout);
	
	c.gridx=0;
	c.gridy=0;
	layout.setConstraints(tabbedPane,c);
	getContentPane().add(tabbedPane);
	
	c.gridx=0;
	c.gridy=2;
	c.insets = new Insets(5,0,0,0);
	layout.setConstraints(buttonPane,c);
	getContentPane().add(buttonPane);
    }

    
    /** Create a new ggptoolbox  for the object 
     * sent by SurfaceDessin 
     * @param obj the objects edited in the toolbox
     * @param sd a reference of SurfaceDessin
     */
    public void send2Toolbox(Vector objects,SurfaceDessin sd)
    {
	this.sd=sd;
	this.objects = objects;
	buildGgpBox(objects);
	updateSize();
	setVisible(true);
	updateSize();
    }

    /** Building the toolbox for 
     * @param obj a Vector of object 
     */
    private void buildGgpBox(Vector objects)
    {
	destroyTab();
	this.ggp = ggp;
	createLineTab(objects);
	createNodeTab(objects);
	createGGPLabelTab(objects);
	createIFMarkTab(objects);
	createArrowTab(objects);
	createLoopTab(objects);
	createOffsetTab(objects);
	updateSize();
    }


    
    /** Create the first tab Line tab 
     * @param objects the vector of objects being edited
     */
    private void createLineTab(Vector objects)
    {
	lt = new OJasLineTab(Languages.tb_tabtitle[0],objects);
	addTab(lt);
	tabbedPane.insertTab(lt.getTabTitle(),null,lt.getPanel(),Languages.tb_tabtooltip[0], lineTabIndex);
    }



    /** Create the Second Tab  Node properties 
     * @param objects the vector of objects being edited
     */
    private void createNodeTab(Vector objects)
    {
	nt = new OJasNodeTab(Languages.tb_tabtitle[1],objects);
	addTab(nt);
	tabbedPane.insertTab(nt.getTabTitle(),null,nt.getPanel(),Languages.tb_tabtooltip[1], nodeTabIndex);
    }



    /** create the third tab  the Label tab 
     * @param objects the vector of objects being edited
     */
    private void createGGPLabelTab(Vector objects)
    {
	lbt = new JasGGPLabelTab(Languages.tb_tabtitle[2],objects);
	addTab(lbt);
	tabbedPane.insertTab(lbt.getTabTitle(),null,lbt.getPanel(),Languages.tb_tabtooltip[2], labelTabIndex);
    }


    /** Create the fourth tab the IFRMark tab 
     * @param objects the vector of objects being edited
     */
    private void createIFMarkTab(Vector objects)
    {
	ift = new JasIFMarkTab("IFRMarks",objects);
	addTab(ift);
	tabbedPane.insertTab(ift.getTabTitle(),null,ift.getPanel(),Languages.tb_tabtooltip[3], ifmarkTabIndex);
    }


    /** Create the fifth tab the  Arrow tab 
     * @param objects the vector of objects being edited
     */
    private void createArrowTab(Vector objects)
    {
	eat = new JasArrowTab(Languages.tb_tabtitle[5],objects);
	addTab(eat);
	tabbedPane.insertTab(eat.getTabTitle(),null,eat.getPanel(),Languages.tb_tabtooltip[5], arrowTabIndex);
    }


    /** Create the sixth tab the  Arrow tab 
     * @param objects the vector of objects being edited
     */
    private void createLoopTab(Vector objects)
    {
	loopTab = new JasLoopTab(Languages.tb_tabtitle[7],objects);
	addTab(loopTab);
	tabbedPane.insertTab(loopTab.getTabTitle(),null,loopTab.getPanel(),null, loopTabIndex);
    }

    /** Create the seventh tab the  Arrow tab 
     * @param objects the vector of objects being edited
     */
    private void createOffsetTab(Vector objects)
    {
	offsetTab = new JasLineOffsetTab(Languages.tb_tabtitle[8],objects);
	addTab(offsetTab);
	tabbedPane.insertTab(offsetTab.getTabTitle(),null,offsetTab.getPanel(),null, offsetTabIndex);
    }


    /** Save the toolbox content in the ggp
     * of all of the vector objects 
     */
    public void setProp()
    {
	GastexGlobalProperties myGgpb = sd.getManager().getActualProp();
	GastexGlobalProperties myGgp = new GastexGlobalProperties(myGgpb);
	sd.getManager().addProp(myGgp);
	sd.getManager().selectProp(myGgpb);
	lt.saveGgp(myGgp);
	nt.saveGgp(myGgp);
	lbt.saveGgp(myGgp);
	ift.saveGgp(myGgp);
	eat.saveGgp(myGgp);
	loopTab.saveGgp(myGgp);
	offsetTab.saveGgp(myGgp);
	
	for(int i = 0 ; i< objects.size() ; i++) {
	    ((JasObject)objects.elementAt(i)).setGgpValues(myGgp);
	    ((JasObject)objects.elementAt(i)).setGgp(myGgp);
	    ((JasObject)objects.elementAt(i)).setDashIndex(-1);
	}
	sd.repaint();
    }
    




  public void actionPerformed(ActionEvent e) {
	String cmd = e.getActionCommand();
	if (cmd.equals("apply_ggp"))
	    {
		setProp();
		
	    }
	else if	 (cmd.equals("ok_ggp"))
	    {
		setProp();
		dispose();
		
	    }
	else if (cmd.equals("cancel_ggp"))
	    {
		dispose();
		
	    }


    }



}
