package jastex.old.gui;

import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;

import jastex.*;
import jastex.old.gui.tabs.OJasRMarkTab;
import jastex.old.gui.tabs.OJasNodeTab;
import jastex.old.gui.tabs.JasNodeLabelTab;
import jastex.old.gui.tabs.OJasLineTab;
import jastex.old.gui.tabs.JasIFMarkTab;
import jastex.old.gui.tabs.JasEdgeLabelTab;
import jastex.old.gui.tabs.JasLineOffsetTab;
import jastex.old.gui.tabs.JasLoopTab;
import jastex.old.gui.tabs.JasArrowTab;
import jastex.old.obj.AEdge;
import jastex.old.obj.JasRectangle;
import jastex.old.obj.Node;
import jastex.old.obj.IFMark;
import jastex.old.obj.BezierCubicCurve;
import jastex.old.obj.JasObject;
import jastex.old.obj.Circle;
import jastex.old.obj.Loop;
import jastex.old.obj.Edge;
import jastex.old.obj.NodeLabel;
import jastex.old.obj.SimpleObjectE;
import jastex.old.obj.EdgeLabel;
import jastex.old.obj.Oval;


/** This class create the toolbox in order to modify
 * Node or Edge properties
 */
public class JasObjectToolBox extends JasToolBox {

    /** an Event Listener */
    public JasToolBoxListener listen;
    /** JasObject being modify through the ToolBox */
    public JasObject object;
    /** SurfaceDessin sending the object */
    public SurfaceDessin sd; 
    /** true if this is a Node ToolBox  */
    public boolean isNodeToolBox=false;
    /** true if this is an Edge ToolBox */
    public boolean isEdgeToolBox=false;
    /** true if this is an Edge ToolBox */
    public boolean isSimpleObjectToolBox=false;
    /** the value and the name of the unitlength */
    public JLabel unitNameLabel;


    /** index of the different tabs in the tabbedPane */
    static private int lineTabIndex = 0;
    static private int nodeTabIndex = 1;
    static private int labelTabIndex = 2;
    static private int ifmarkTabIndex = 3;
    static private int rmarkTabIndex = 4;

    static private int arrowTabIndex = 1;
    static private int edgeLabelTabIndex = 2;
    static private int offsetTabIndex = 3;
    static private int loopTabIndex = 4;

    /** the Line Tab */
    public OJasLineTab lt;
    /** the Node Tab */
    public OJasNodeTab nt;
    /** the Node Label Tab */
    public JasNodeLabelTab lbt;
    /** the ifmark tab */
    public JasIFMarkTab ift;
    /** the Rmark tab */
    public OJasRMarkTab rmt;
    /** the arrow tab */
    public JasArrowTab eat;
    //JasTransitionTab ett;
    /** the Edge Label Tab */
    public JasEdgeLabelTab elt;

    /** the loop Tab */
    public JasLoopTab loopTab;
    
    public JasLineOffsetTab offsetTab;

    /** tab icons */
    public static ImageIcon tabIcons[];
    public static String[] icon_filename = {"images/ico_line_mini.gif","images/ico_node_round_mini.gif",
				     "images/ico_label_mini.gif","images/ico_ifmark_mini.gif",
				     "images/ico_rmark_mini.gif", "images/ico_arrow_mini.gif",
				     "images/ico_trans_mini.gif" };
        

    public boolean debug=false;

    /** Constructor which initializes a toolBox
     * unvisible and empty
     * @param parentFrame the Parent frame of the ToolBox dialog
     * @param toolBoxTitle the title of the toolbox dialogbox
     * */
    public JasObjectToolBox(Frame parentFrame , String toolBoxTitle)
    {
	/* Creation of the ToolBox of the death */
	super(parentFrame,toolBoxTitle);
	
	listen = new JasToolBoxListener(this);
	
	/** allocation of icons */
	boolean isAppli = MainWindow.isAppli;
	tabIcons = new ImageIcon[10];
	if(isAppli)
	    for(int i=0; i< java.lang.reflect.Array.getLength(icon_filename) ; i++)
		tabIcons[i] = new ImageIcon(icon_filename[i]);
	else {
	    for(int i=0; i< java.lang.reflect.Array.getLength(icon_filename) ; i++)
		{
		    URL location; 
		    location = JasToolBox.class.getResource(icon_filename[i]);
		    tabIcons[i] = new ImageIcon(location);
		}
	}

	
	/* Apply Buton */
	JButton applyButton = new JButton(Languages.tb_ApplyBtn);
	applyButton.setActionCommand("apply");
	applyButton.addActionListener(listen);

	/* unitLength / unitName label */
	unitNameLabel = new JLabel();
	
	JPanel buttonPane = new JPanel(); 
	buttonPane.setLayout(new BoxLayout(buttonPane, BoxLayout.X_AXIS));
	buttonPane.add(unitNameLabel);
	buttonPane.add(Box.createHorizontalGlue());
	buttonPane.add(applyButton);

	
	GridBagLayout layout = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();
	c.fill = GridBagConstraints.BOTH;	
	getContentPane().setLayout(layout);
	
	c.gridx=0;
	c.gridy=0;
	layout.setConstraints(tabbedPane,c);
	getContentPane().add(tabbedPane);
	
	c.gridx=0;
	c.gridy=1;
	c.insets = new Insets(5,0,0,0);
	layout.setConstraints(buttonPane,c);
	getContentPane().add(buttonPane);
    }
    

    
    /** Create a new toolbox  for the object 
     * sent by SurfaceDessin 
     * @param obj the object edited in the toolbox
     * @sd a reference of SurfaceDessin
     */
    public void send2Toolbox(JasObject obj,SurfaceDessin sd)
    {
	this.sd=sd;
	buildToolBox(obj);
	updateSize();
	setVisible(true);
	
	//parentFrame.requestFocus();

    }


    /** Destroy the content of the toolbox
     * destroy all the tabs in tabbed pane
     */
    public void destroyTab()
    {
	tabbedPane.removeAll();
    }
    
    
    /** Building the toolbox for 
     * the object obj
     * @param obj a JasObject ofr which the ToolBox is build 
     */
    public void buildToolBox(JasObject obj)
    {
	if(obj != this.object)
	    {
		setResizable(true);
		unitNameLabel.setText("Unité : "+obj.getGgp().getUnitLength()+" " +obj.getGgp().getUnitName());
		if(obj instanceof Node)
		    {
			
			if(!isNodeToolBox)
			    {
				debug("[NODE] : Création de la toolbox: Node");
				destroyTab();
				buildNodeToolBox((Node)obj);
				isNodeToolBox=true;
				isEdgeToolBox=false;
				isSimpleObjectToolBox = false;
				tabbedPane.setSelectedIndex(nodeTabIndex);
				
			    }
			else 
			    {
				if(object instanceof AEdge && !(object instanceof IFMark))
				    {
					debug("[NODE] : Destruction de la toolbox: Edge et Création de la toolbox: Node");
					destroyTab();
					buildNodeToolBox((Node)obj);
				    }
				debug("[NODE] : Update de la toolbox: Node");
				object = obj; 
				updateNodeToolBox();
				updateRMarkTab();
				updateImarkTab();
				lbt.updateLabelList(object);
				lbt.refresh(object);
				tabbedPane.setSelectedIndex(nodeTabIndex);
			    }
		    }
		
		else if(obj instanceof IFMark)
		    {
			if(object != (Node)((IFMark)obj).getNode())
			    {
				debug("[IFMARK] : Destruction de la toolbox");
				destroyTab();
				isNodeToolBox=true;
				isEdgeToolBox=false;
				isSimpleObjectToolBox = false;
				buildNodeToolBox((Node)((IFMark)obj).getNode());
				updateRMarkTab();
				updateImarkTab();
				tabbedPane.setSelectedIndex(ifmarkTabIndex);
			    }
			ift.setSelectedIFMark((IFMark)obj);
		    }
	    
	
		else if(obj instanceof AEdge)
		     {
			 boolean refresh=false;
			 boolean refresh2=false;

			 if(((obj instanceof BezierCubicCurve && !(obj instanceof Edge)) && this.object instanceof Edge) || ( obj instanceof Edge && (this.object instanceof BezierCubicCurve && !(this.object instanceof Edge))))
			     refresh = true;
			 else 
			     refresh=false;


			 if( obj instanceof Loop && this.object instanceof Loop)
			     refresh2 = false;
			 else 
			     refresh2=true;
			 
			 
			 if(isNodeToolBox || isSimpleObjectToolBox || refresh || refresh2)
			    {
				debug("[AEDGE] Destruction de la toolbox Node : Création de la toolbox: Edge");
				destroyTab();
				buildEdgeToolBox((AEdge)obj);
				isNodeToolBox = false ;
				isEdgeToolBox = true;
				isSimpleObjectToolBox = false;

			    }
			else 
			    {
				if(isEdgeToolBox)
				    {
					debug("[AEDGE] Mise à jour de la toolbox Edge");
					object = obj;
					updateEdgeToolBox();
				    }
				else
				    {
					debug("[AEDGE] Création de la toolbox Edge");
					buildEdgeToolBox((AEdge)obj);
					isEdgeToolBox = true;
					isNodeToolBox = false;
					isSimpleObjectToolBox = false;
				    }
			    }
			tabbedPane.setSelectedIndex(lineTabIndex);
		     }

		else if(obj instanceof Circle || obj instanceof JasRectangle || obj instanceof Oval)
		    {
			if(isSimpleObjectToolBox)
			    {
				/* update the toolbox */
				object = obj;
				updateSimpleObjectToolBox();
				isNodeToolBox=false;
				isEdgeToolBox=false;
				
			    }
			else 
			    {
				destroyTab();
				buildSimpleObjectToolBox(obj);
				isSimpleObjectToolBox = true;
				isNodeToolBox = false ;
				isEdgeToolBox = false;
			    }
		    }
		else if(obj instanceof NodeLabel)
		    {
			if(object != ((NodeLabel)obj).getNode())
			    {
			
				if(!isNodeToolBox)
				    {
					debug("[NODE] : Création de la toolbox: Node");
					destroyTab();
					buildNodeToolBox((Node)((NodeLabel)obj).getNode());
					isNodeToolBox=true;
					isEdgeToolBox=false;
					tabbedPane.setSelectedIndex(labelTabIndex);
					
				    }
				else 
				    {
					if(object instanceof AEdge && !(object instanceof IFMark))
					    {
						debug("[NODE] : Destruction de la toolbox: Edge et Création de la toolbox: Node");
						destroyTab();
						buildNodeToolBox((Node)((NodeLabel)obj).getNode());
					    }
					debug("[NODE] : Update de la toolbox: Node");
					object = ((NodeLabel)obj).getNode(); 
					updateNodeToolBox();
					updateRMarkTab();
					updateImarkTab();
					
					lbt.updateLabelList(object);
					lbt.refresh(object);
					tabbedPane.setSelectedIndex(nodeTabIndex);
				    }
			    }
		    }
		else if(obj instanceof EdgeLabel)
		    {
			if(object != ((EdgeLabel)obj).getEdge())
			    {
				if(isNodeToolBox || isSimpleObjectToolBox )
				    {
					debug("[AEDGE] Destruction de la toolbox Node : Création de la toolbox: Edge");
					destroyTab();
					buildEdgeToolBox((AEdge)((EdgeLabel)obj).getEdge());
					isNodeToolBox = false ;
					isEdgeToolBox = true;

				    }
				else 
				    {
					if(isEdgeToolBox)
					    {
						debug("[AEDGE] Mise à jour de la toolbox Edge");
						object = ((EdgeLabel)obj).getEdge();
						updateEdgeToolBox();
					    }
					else
					    {
						debug("[AEDGE] Création de la toolbox Edge");
						buildEdgeToolBox((AEdge)((EdgeLabel)obj).getEdge());
						isEdgeToolBox = true;
						isNodeToolBox = false;
					    }
				    }
				tabbedPane.setSelectedIndex(edgeLabelTabIndex);
			    }
		    }

		updateSize();
	    }
    
    }
    
    
    /** Create the first tab 
     * Line tab */
    private void createLineTab()
    {
	lt  = new OJasLineTab(Languages.tb_tabtitle[0],object);
	addTab(lt);
	tabbedPane.insertTab(lt.getTabTitle(),tabIcons[0],lt.getPanel(),Languages.tb_tabtooltip[0], lineTabIndex);
    }


    /** Create the Second Tab
     * Node properties 
     */
    private void createNodeTab()
    {
	nt = new OJasNodeTab(Languages.tb_tabtitle[1],(Node)object,sd);
	addTab(nt);
	tabbedPane.insertTab(nt.getTabTitle(),tabIcons[1],nt.getPanel(), Languages.tb_tabtooltip[1], nodeTabIndex);
    }



    /** create the third tab
     * the Node Label tab 
     */
    private void createLabelTab()
    {
	lbt  = new JasNodeLabelTab(Languages.tb_tabtitle[2],object,sd);
	addTab(lbt);
	tabbedPane.insertTab(lbt.getTabTitle(),tabIcons[2],lbt.getPanel(),Languages.tb_tabtooltip[2], labelTabIndex);
	
    }


    /** Create the fourth tab
     * the IFMark tab 
     */
    private void createIFMarkTab()
    {
	ift  = new JasIFMarkTab(Languages.tb_tabtitle[3],object,sd);
	addTab(ift);
	tabbedPane.insertTab(ift.getTabTitle(),tabIcons[3],ift.getPanel(),Languages.tb_tabtooltip[3], ifmarkTabIndex);
    }


    /** Create the fifth tab
     * the RMark tab 
     */
    private void createRMarkTab()
    {
	rmt = new OJasRMarkTab(Languages.tb_tabtitle[4],object,sd);
	addTab(rmt);
	tabbedPane.insertTab(rmt.getTabTitle(),tabIcons[4],rmt.getPanel(),Languages.tb_tabtooltip[4], rmarkTabIndex);
    }


    
    /** Create the second tab
     * the Edge Arrow tab 
     */
    private void createEdgeArrowTab()
    {
	eat  = new JasArrowTab(Languages.tb_tabtitle[5],object,sd);
	addTab(eat);
	tabbedPane.insertTab(eat.getTabTitle(),tabIcons[5],eat.getPanel(),Languages.tb_tabtooltip[5],arrowTabIndex);
    }


    /** Create the third tab
     * the Edge Label tab 
     */
    private void createEdgeLabelTab()
    {
	elt  = new JasEdgeLabelTab(Languages.tb_tabtitle[6],object,sd);
	addTab(elt);
	tabbedPane.insertTab(elt.getTabTitle(),tabIcons[2],elt.getPanel(),Languages.tb_tabtooltip[2],edgeLabelTabIndex);
    }





    private void createSimpleObjectNodeTab()
    {
	nt = new OJasNodeTab(Languages.tb_tabtitle[1],object,sd);
	addTab(nt);
	tabbedPane.insertTab(nt.getTabTitle(),tabIcons[1],nt.getPanel(), Languages.tb_tabtooltip[1], nodeTabIndex);
    }



    private void createLoopTab()
    {
	loopTab = new JasLoopTab(Languages.tb_tabtitle[7],object);
	addTab(loopTab);	
	tabbedPane.insertTab(loopTab.getTabTitle(),tabIcons[5],loopTab.getPanel(), null, loopTabIndex);
    }

    private void createOffsetTab()
    {
	offsetTab = new JasLineOffsetTab(Languages.tb_tabtitle[8],object);
	addTab(offsetTab);	
	tabbedPane.insertTab(offsetTab.getTabTitle(),null,offsetTab.getPanel(), null, offsetTabIndex);
    }



    /** Building the 5 tabs for a Node 
     * @param obj the Node which will be edited in the toolbox
     */       
    private void buildNodeToolBox(Node obj)
    {
	object = obj;
	createLineTab();
	createNodeTab();
	createLabelTab();
	createIFMarkTab();
	createRMarkTab();
    }
    

    /** Building the 3 tabs for an Edge 
     * @param obj  the AEdge which will be edited in the toolbox
     */       
    private void buildEdgeToolBox(AEdge obj)
    {
	object = obj;
	createLineTab();	
	createEdgeArrowTab();
	if(!(obj instanceof SimpleObjectE))
	    {
		createEdgeLabelTab();
		createOffsetTab();
	    }
	if(obj instanceof Loop)
	    createLoopTab();

    }




    private void buildSimpleObjectToolBox(JasObject obj)
    {
	object = obj;
	createLineTab();
	createSimpleObjectNodeTab();
    }


    /** Update then IFMark Tab 
     * deleting it and recreating it
     */
    public void updateImarkTab()
    {
	deleteTab(ift,ifmarkTabIndex);
	createIFMarkTab();
	tabbedPane.setSelectedIndex(ifmarkTabIndex);	
    }


    /** Update then RMark Tab 
     * deleting it and recreating it
     */
    public void updateRMarkTab()
    {
	if(object!=null){ 
	    deleteTab(rmt,rmarkTabIndex);
	    createRMarkTab();
	}
    }

//     public void updateRMarkTab2()
//     {
// 	deleteTab(rmt,rmarkTabIndex);
// 	createRMarkTab();
//     }


    /** Update the Rmarktab ComboBox */
    public void updateRMarkList()
    {
	updateRMarkTab();
	tabbedPane.setSelectedIndex(rmarkTabIndex);	
    }


    /** Update the Label ComboBox */
    public void updateNodeLabelList(JasObject object)
    {
	lbt.updateLabelList(object);
    }

    /** Update the Node Toolbox */
    private void updateNodeToolBox()
    {
       	lt.updateContent((Node)object);
	nt.updateContent(object);
	lbt.updateContent(object);
	rmt.updateContent(object);
    }


    /** Update the Edge Toolbox */
    private void updateEdgeToolBox()
    {
	lt.updateContent((AEdge)object);
	eat.updateContent(object);
	elt.updateContent(object);
	if(object instanceof Loop) loopTab.updateContent(object);
    }


    private void updateSimpleObjectToolBox()
    {
	lt.updateContent(object);
	deleteTab(nt,nodeTabIndex);
	createSimpleObjectNodeTab();
	
    }
    
    
    /** Update the JasObject properties */
    public void setProp()
    {
	if(object instanceof Node)
	    {
		/* Properties of the Line Tab */
		lt.writeNodeLineProperties(object);
		/* Properties of the Node Tab  */
		nt.writeNodeProperties(object);
		/* Properties of the Label Tab  */
		lbt.writeNodeLabelProperties(object);
		/** Properties of the IFmark Tab */
		ift.writeIFMarkProperties(object);
		/** Properties of the RMarks Tab */
		rmt.writeRMarksProperties(object);
	    }
	else if(object instanceof AEdge)
	    {
		/* Properties of the Line Tab */
		lt.writeEdgeLineProperties(object);
		/* Properties of the Arrow Tab */
		eat.writeArrowProperties(object);
		/* Properties of the Edge Label Tab */
		if(!(object instanceof SimpleObjectE))
		    {
			elt.writeEdgeLabelProperties(object);
			offsetTab.writeOffset(object);
		    }
		if(object instanceof Loop)
		    loopTab.writeLoopProperties(object);
	    }

	else if(object instanceof Circle || object instanceof JasRectangle || object instanceof Oval)
	    {
		/* Properties of the Line Tab */
		lt.writeNodeLineProperties(object);
		/* Properties of the Node Tab  */
		nt.writeSimpleObjectNodeProperties(object);
	
	    }
	sd.fromToolBoxRefresh(object); 
    }
    

    /** Update the X propertie 
     * for the current Node 
     */
    public void updateNodeX(double x) { if(nt != null) nt.updateNodeX(x);    }


    /** Update the Y propertie  
     * for the current Node   
     */
    public void updateNodeY(double y) { if(nt != null) nt.updateNodeY(y); }


    /** Update the nw (Node width) 
     * propertie for the current Node 
     */
    public void updateNodeW(double nw)    { if(nt != null) nt.updateNodeW(nw); }


    /** Update the nh (Node heigth) 
     * propertie for the current Node 
     */
    public void updateNodeH(double nh)    { if(nt != null) nt.updateNodeH(nh); }

    /** Update the nmr propertie for
     * the current Node 
     */
    public void updateNodeNmr(double nmr) { if(nt != null) nt.updateNodeNmr(nmr); }


    /** Update the IFMarkAngle propertie */
    public void updateIFMarkAngle(double angle) { if(ift != null) ift.updateIFMarkAngle(angle); }

    /** Update the IFMarkLength propertie */
    public void updateIFMarkLength(double length) { if(ift != null) ift.updateIFMarkLength(length); }

    public void updateCircleDiameter(double diam) { if(nt != null) nt.updateCircleDiameter(diam); }


    public void updateDistance(double dist)
    {
	elt.updateDistance(dist);
    }

    public void updatePosition(double pos)
    {
	elt.updatePosition(pos);
    }

    public void updateSide(double side) 
    {
	elt.updateSide(side);
    }

    public void updateLabel(String text) {
	elt.updateLabel(text);
    }
    

   public void updateLoopAngle(double a)
    {
	loopTab.updateLoopAngle(a);
    }

    public void updateLoopDiam(double diam)
    {
	loopTab.updateLoopDiam(diam);
    }






  protected void debug(String msg)
    {
	if(debug)
	    System.out.println(msg);
	
    }

   protected void debug(int i)
    {
	if(debug)
	    System.out.println(i);
    }

}
