package jastex.old.gui;
import java.awt.Component;
import java.awt.Dimension;
import javax.swing.plaf.ComponentUI;
import javax.swing.JTextPane;
import javax.swing.text.StyledDocument;
import javax.swing.text.*;
import java.util.*;
import java.awt.*;

public class JasTextPane extends JTextPane {
    private boolean wrap=false;
    private boolean color = true;

    private boolean isGasset = false;
    private boolean isComment = false;
    private boolean isMacro = false;
    private boolean isBraced = false;
    private boolean isParen = false;
    private boolean isBrack = false;

    public JasTextPane(){
        super();
    }

    public void setWordWrap(boolean wrap) {
        this.wrap=wrap;
    }

    public boolean getWordWrap() {
        return this.wrap;
    }

    public void setColor(boolean c) {
	color = c;
    }
    public boolean getColor() {
	return color;
    }

    public boolean getScrollableTracksViewportWidth() {
        if (!wrap) {
            Component parent=this.getParent();
            ComponentUI ui=this.getUI();
            int uiWidth=ui.getPreferredSize(this).width;
	    int parentWidth = 0;
	    if (parent != null) {
		parentWidth=parent.getSize().width;
	    }
            boolean bool= (parent !=null)?(uiWidth < parentWidth):true;
            return bool;
        }
        else return super.getScrollableTracksViewportWidth();
    }

    public void setBounds(int x, int y, int width, int height)
    {
        if (wrap)
            super.setBounds(x, y, width, height);
        else
        {
            Dimension size = this.getPreferredSize();
            super.setBounds(x,y,Math.max(size.width, width),Math.max(size.height, height));
        }
    }


    public void setText (String s) {
	if (color) {
	    super.setText("");
	    this.setCaretPosition(0);
	    insertColorized(s);
	} else {
	    super.setText(s);
	}
    }

    public void insertColorized(String s) {
	String sc;
	SimpleAttributeSet sas = new SimpleAttributeSet();
	StringTokenizer st = new StringTokenizer(s,"%\\(){}[]\n",true);
	if (isComment)
	    StyleConstants.setForeground(sas,Color.red);		    
	else if (isMacro)
	    StyleConstants.setForeground(sas,Color.blue);
	else if (isGasset || isBrack || isParen)
	    StyleConstants.setForeground(sas,Color.black);
	else if (isBraced && !isGasset)
	    StyleConstants.setForeground(sas,Color.gray);
	
	while (st.hasMoreTokens()) {
	    sc = st.nextToken();
	    //token color
	    switch (sc.charAt(0)) {
	    case '%':
		isComment = true;
		StyleConstants.setForeground(sas,Color.red);
		break;
	    case '\n':
		isComment = false;
		isGasset = false;
		isMacro = false;
		StyleConstants.setForeground(sas,Color.black);
		break;
	    case '}':
		if (!isComment) {
		    isBraced = false;
		    isGasset = false;
		    isMacro = false;
		    StyleConstants.setForeground(sas,Color.black);
		}
		break;
	    case ')':
		if (!isComment) {
		    isParen = false;
		    isGasset = false;
		    isMacro = false;
		    StyleConstants.setForeground(sas,Color.black);
		}
		break;
	    case ']':
		if (!isComment) {
		    isBrack = false;
		    isGasset = false;
		    isMacro = false;
		    StyleConstants.setForeground(sas,Color.black);
		}
		break;

	    case '[':
		if (!isComment) {
		    isBrack = true;
		    isMacro = false;
		    StyleConstants.setForeground(sas,Color.black);
		}
		break;

	    case '(':
		if (!isComment) {
		    isParen = true;
		    isMacro = false;
		    StyleConstants.setForeground(sas,Color.black);
		}
		break;

	    case '{':
		if (!isComment) {
		    isBraced = true;
		    isMacro = false;
		    StyleConstants.setForeground(sas,Color.black);
		}
		break;

	    case '\\':
		if (!isComment) {
		    isMacro = true;
		    StyleConstants.setForeground(sas,Color.blue);
		}
		break;
	    default:
		if (isComment)
		    StyleConstants.setForeground(sas,Color.red);		    
		else if (isMacro)
		    StyleConstants.setForeground(sas,Color.blue);
		else if (isGasset || isBrack || isParen)
		    StyleConstants.setForeground(sas,Color.black);
		else if (isBraced && !isGasset)
		    StyleConstants.setForeground(sas,Color.gray);
		isGasset = (sc.compareTo("gasset") == 0);
		    
		break;
	    }
	    try {
		int caretPos = this.getCaretPosition();
		this.getDocument().insertString(caretPos,sc,sas);
		this.setCaretPosition(caretPos + sc.length());
	    } catch (BadLocationException e) { System.out.println("warning"); }
	}
    }	

    public void replaceSelection(String s) {
	if (color) {
	    //erase
	    super.replaceSelection("");
	    //insert:
	    insertColorized(s);
	} else {
	    super.replaceSelection(s);
	}
    }
}

