package jastex.old.gui;

import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.*;
import java.awt.*;
import java.net.*;
import java.awt.image.*;

import jastex.*;



/** This class is a kind of combo box but it only works 
 * with icons and doesn't show the menu each the main 
 * button is clicked
 */
class MyButtonComboBox extends JPanel implements ActionListener {
    /** The main button showing the selected item */
    private JButton main_button;
    /** The small button with show the menu */
    private JButton dropdown_button;
    /** The different Icon to display in the menu
     * (their description field must be provided) */
    private ImageIcon[] image_list;
    
    private ImageIcon[] image_list_selected;
    /** The action listener called when the main button is clicked */
    private ActionListener l;
    /** The action command for the listener */
    private String actionCommand;

    /** The index of the currently selected item */
    private int currentItem;
    
    /** Constructor which initialize the button
     * using the ImageIcon list provided */
    public MyButtonComboBox(ImageIcon[] il) {
	boolean isAppli = MainWindow.isAppli;
	image_list = il;
	l = null;
	actionCommand = null;

	currentItem = 0;

	ImageIcon icon = il[0];
	ImageIcon smallArrow = null;
	if (isAppli) {
	    smallArrow = new ImageIcon("images/arrow-black.gif");
	} else {
	    URL location; 
	    location = MyButtonComboBox.class.getResource("images/arrow-black.gif");
	    smallArrow = new ImageIcon(location);
	}
	
	image_list_selected = new ImageIcon[il.length];
	for (int i = 0; i < il.length; i++) {
	    Image src = ((ImageIcon)il[i]).getImage();
	    ImageFilter colorfilter = new ImageSelectedFilter();
	    Image img = createImage(new FilteredImageSource(src.getSource(),
							colorfilter));
	    image_list_selected[i] = new ImageIcon(img);
	}
	main_button = new JButton(icon);
	main_button.setSelectedIcon(image_list_selected[0]);
	main_button.setMargin(new Insets(0,0,0,0));

	dropdown_button = new JButton(smallArrow);
	int d = (icon.getIconHeight()-smallArrow.getIconHeight()+1)/2;
 	dropdown_button.setMargin(new Insets(d,0,d,0));

	setLayout(new FlowLayout(FlowLayout.LEFT, 0, 0));
	add(main_button);
	add(dropdown_button);

	main_button.setActionCommand("main");
	dropdown_button.setActionCommand("dropdown");
	main_button.addActionListener(this);
	dropdown_button.addActionListener(this);

	main_button.setToolTipText(icon.getDescription());
	dropdown_button.setToolTipText(Languages.mbcb_ToolTip);
		
	setVisible(true);

    }

    /** Select or unselect the main button */
    public void setSelected(boolean select) {
	main_button.setSelected(select);
    }
    public void setSelected(boolean select, int num) {
	if (num == -1 || num >= image_list.length) return; 
	main_button.setIcon(image_list[num]);
	main_button.setSelectedIcon(image_list_selected[num]);
	// 	    main_button.setSelected(true);
	main_button.setToolTipText(image_list[num].getDescription());
	currentItem = num;
	setSelected(select);
    }

    public void setEnabled(boolean state) {
	main_button.setEnabled(state);
	dropdown_button.setEnabled(state);
    }
	

    /** Get the current Item */
    public int getCurrentItem() {
	return currentItem;
    }

    /** Set up the action listener */
    public void addListener (ActionListener al, String ac) {
	l = al;
	actionCommand = ac;
    }

    /** Used when the user click on a button or on a menu item */
    public void actionPerformed(ActionEvent e) {
	String cmd = e.getActionCommand();
	if (cmd.equals("main")) { //Main Button
	    //execute the ActionListener if possible
	    if (l == null) {
		return;
	    } else {
// 		main_button.setSelected(true);
		l.actionPerformed(new ActionEvent(e.getSource(),
						  e.getID(),
						  actionCommand+currentItem,
						  e.getModifiers()));
	    }
	} else if (cmd.equals("dropdown")) { //DropDown button
	    //Create the menu
	    JPopupMenu menu = new JPopupMenu();
	    JMenuItem jmi = null;
	    for (int i = 0; i < image_list.length; i++) {
		jmi = new JMenuItem(image_list[i].getDescription(),
					      image_list[i]);
		if (jmi != null) {
		    menu.add(jmi);
		    jmi.setActionCommand("MenuItem" + i);
		    jmi.addActionListener(this);
		}
	    }

	    menu.setInvoker(dropdown_button);
	    Dimension dim = dropdown_button.getSize();
	    menu.show(dropdown_button,
		      -image_list[currentItem].getIconWidth()-7,
		      dim.height);
	    
	} else if (cmd.indexOf("MenuItem",0) != -1) { //A Menu Item
	    // get index of the item,
	    // then replace the icon on the main button
	    // and finally, click on it
	    String sNum = cmd.substring(cmd.length() - 1);
	    int num = -1;
	    try {
		num = Integer.parseInt(sNum);
	    } catch (NumberFormatException exp) {}
	    if (num == -1 || num >= image_list.length) return; 
	    main_button.setIcon(image_list[num]);
	    main_button.setSelectedIcon(image_list_selected[num]);
// 	    main_button.setSelected(true);
	    main_button.setToolTipText(image_list[num].getDescription());
	    currentItem = num;
	    //main_button.doClick();
	    if (l != null) {
		l.actionPerformed(new ActionEvent(e.getSource(),
						  e.getID(),
						  actionCommand+currentItem,
						  e.getModifiers()));
	    }
	}
	else return;
    }

    public void initLanguage() {
	main_button.setToolTipText(image_list[currentItem].getDescription());
	dropdown_button.setToolTipText(Languages.mbcb_ToolTip);
    }
}

class ImageSelectedFilter extends RGBImageFilter {
    public ImageSelectedFilter() {
	// The filter's operation does not depend on the
	// pixel's location, so IndexColorModels can be
	// filtered directly.
	canFilterIndexColorModel = true;
    }
    
    
    public int filterRGB(int x, int y, int rgb) {
	int r = (rgb & 0x00FF0000) >> 16;
	int g = (rgb & 0x0000FF00) >> 8;
	int b = (rgb & 0x000000FF);
	float [] hsb;

	hsb = Color.RGBtoHSB(r,g,b,null);
	hsb[0] = -100;
	hsb[1] = (float)0.7;
	return (Color.HSBtoRGB(hsb[0], hsb[1], hsb[2]));
   }
}
