package jastex.old.obj;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.Vector;
import java.io.*;
import java.awt.geom.*;
import java.util.*;

import jastex.*;



/** This abstract class is intended to design all the node in gastex and to make
 * easiest the possibility to add new kind of node
 */
public abstract class ANode extends JasObject {
    /** Field used to generated an unique  name for each node */
    static public int nodeNum = 0;

    /** Gastex Properties */
    /** Adjust the node on the label either by height or width or both */
    public int pNadjust;     // any combination of the number 0,1,2 by sum
                      //   1 = width, 2 = height, 0 = no adjust
    /** Distance between the label box and the node frame when using Nadjust */
    public double pNadjustdist; // distance in unitlength between the label box 
                      //   and the node frame when using Nadjust
    /**  boolean flag defining whether nodes are filled or not */
    /** Marks on the node (initial and/or final and/or repeated) */
    public int pNmarks;      // any combination of the number 0,1,2,4 by sum
                      //    1 : initial mark on nodes
                      //    2 : final mark on nodes
                      //    4 : repeated mark on nodes
                      //    0 : no mark on nodes

    
    /** A vector containing all the initial marks */
    public Vector viMarks;
    /** A vector containing all the final marks */
    public Vector vfMarks;
    /** A vector containing all the repeated marks */
    public Vector vrMarks;

    /** used by updateVector method */
    public Vector newAddedXMark; 

    /** the Gastex internal name of the node */
    public String nodeName;

    /** A vector containing all the labels of the node */
    public Vector nodeLabels;
    /** an index to determine which label is inside */
    public int nodeLabelIndex = 0;

    /** A vector containing all the edges related to the Node */
    public Vector vEdges;

    /** the last ggp seen */
    public GastexGlobalProperties lastggp;
    /** Simple constructor, maybe not useful */
    public ANode() {
	viMarks = new Vector();
	vfMarks = new Vector();
	vrMarks  = new Vector();
	vEdges=new Vector();
	newAddedXMark = new Vector();
	nodeLabels = new Vector();
	nodeName = "n" + nodeNum;
	nodeNum++;
    }
	
    /** A constructor which initialize the node properties
     * using a given GastexGlobalProperties
     * @param _ggp the used GastexGlobalProperties
     * @see GastexGlobalProperties
     */
    public ANode(GastexGlobalProperties _ggp) {
// 	super(_ggp); Useless
	this();
	setGgpValues(_ggp);
	lastggp = _ggp;
    }
    
    /** Set the internal value to the one of the GastexGlobalProperties
     * @param _ggp the GastexGlobaProperties used to initialize values
     * @see GastexGlobalProperties
     */ 
    public void setGgpValues(GastexGlobalProperties _ggp){
	super.setGgpValues(_ggp);

	pNmarks = _ggp.pNmarks;	

	pNadjust = _ggp.pNadjust;
	pNadjustdist = _ggp.pNadjustdist;
	
    }

    /** Generated the Gastex Properties Code 
     * associated with the current fields values, and which can include
     * also the code associated with one initial and/or final mark,
     * and the code associated with the first label
     * @param bestIMark the index in the initial mark vector which can be
     * include in the properties
     * @param bestFMark the index in the final mark vector which can be
     * include in the properties
     * @param bestRMark the index in the repeated mark vector which can be
     * include in the properties     
     * @param isStandAlone a boolean to indicate how to format the return string
     * @return a String containing the properties modified compare to 
     * the current GastexGlobalProperties, surrounded by '[' and ']'
     * if isStandAlone is true, else a beginning by a common 
     * or the empty string ("")
     */
    String getLatexCodeProperties(int bestIMark, int bestFMark, int bestRMark,
				  boolean isStandAlone) {
	String lProp = super.getLatexCodeProperties(false);
	String firstLabelProp = "";
	String bestIMarkProp = "";
	String bestFMarkProp = "";
	String bestRMarkProp = "";

	pNmarks = 0;

	if (nodeLabels.size() > nodeLabelIndex && nodeLabelIndex >= 0) {
	    NodeLabel nl = (NodeLabel)nodeLabels.elementAt(nodeLabelIndex);
	    if (nl.getGgp() == globalProp) {
		firstLabelProp = nl.getLatexCodeProperties(false);
	    }
	}
	if (bestIMark != -1) {
	    IFMark imark = (IFMark)viMarks.elementAt(bestIMark);
	    bestIMarkProp = imark.getLatexCodeProperties(false);
	    pNmarks |= 1;
	}
	if (bestFMark != -1) {
	    IFMark fmark = (IFMark)vfMarks.elementAt(bestFMark);
	    bestFMarkProp = fmark.getLatexCodeProperties(false);
	    pNmarks |= 2;
	}
	if (bestRMark != -1) {
	    RMark rmark = (RMark)vrMarks.elementAt(bestRMark);
	    bestRMarkProp = rmark.getLatexCodeProperties(false);
	    pNmarks |= 4;
	}
	lProp += firstLabelProp + bestIMarkProp + bestFMarkProp;

	if (pNadjust != globalProp.pNadjust) {
	    lProp += ",Nadjust=";
	    if (pNadjust == 0) {
		lProp += "n";
	    } else {
		if ((pNadjust & 1) != 0) {
		    lProp += "w";
		}
		if ((pNadjust & 2) != 0) {
		    lProp += "h";
		}
	    }
	}
	if (pNadjustdist != globalProp.pNadjustdist) {
		lProp += ",Nadjustdist="+
		    roundDouble(getpNadjustdist());
	}


	if (pNmarks != globalProp.pNmarks) {
	    lProp += ",Nmarks=";
	    if (pNmarks == 0) {
		lProp += "n";
	    } else {
		if ((pNmarks & 1) != 0) {
		    lProp += "i";
		}
		if ((pNmarks & 2) != 0) {
		    lProp += "f";
		}
		if ((pNmarks & 4) != 0) {
		    lProp += "r";
		}
	    }
	}

	if (lProp.equals("")) {
	    return "";
	} else {
	    if (isStandAlone) {
		//cut the first comma
		return ("[" + lProp.substring(1) + "]");
	    } else {
		return (lProp);
	    }
	}
    }

    /** Search if an initial mark associated with the Node can
     * be directly include in the code of the node by checking their
     * common properties
     * @return the index of the node if one match, else -1
     */
    public int searchBestiMark() {
	int max;
	IFMark imark;
	max = viMarks.size();
	for (int i = 0; i < max; i++) {
	    imark = ((IFMark)viMarks.elementAt(i));
	    if (imark.getGgp() == globalProp
		&& imark.getplinegray() == plinegray
		&& imark.getplinecolor()  == plinecolor
		&& imark.getplinewidth() == getplinewidth()
		&& imark.getpdash_length() == pdash_length
		&& imark.getpdash_empty() == getpdash_empty()) {
		return i;
	    }
	}
	    return -1;
    }

     /** Search if a final mark associated with the Node can
     * be directly include in the code of the node by checking their
     * common properties
     * @param bestIMark the index in the initial mark vector which can be
     * include in the node code, so the parameters must be the same
       * @return the index of the node if one match, else -1
     */
   public int searchBestfMark(int bestIMark) {
	int max;
	IFMark fmark, imark;
	if (bestIMark != -1) {
	    imark = ((IFMark)viMarks.elementAt(bestIMark));
	} else {
	    imark = null;
	}
	max = vfMarks.size();
	for (int i = 0; i < max; i++) {
	    fmark = ((IFMark)vfMarks.elementAt(i));
	    if (fmark.getGgp() == globalProp
		&& fmark.getplinegray() == plinegray
		&& fmark.getplinecolor()  == plinecolor
		&& fmark.getplinewidth() == getplinewidth()
		&& fmark.getpdash_length() == pdash_length
		&& fmark.getpdash_empty() == getpdash_empty()) {
		// this one go nice with the node
		//try to see if it works with the imark
		if (imark != null) {
		    if(fmark.getGgp() == imark.getGgp()&&
		       fmark.getpAHnb() == imark.getpAHnb() &&
		       fmark.getpAHdist() == imark.getpAHdist() &&
		       fmark.getpAHangle() == imark.getpAHangle() &&
		       fmark.getpAHLength() == imark.getpAHLength() &&
		       fmark.getpAHlength() == imark.getpAHlength()) {
			//ok
			return i;
		    }
		    //not good
		} else {
		    //no imark
		    return i;
		}		    
	    }
	}
	    return -1;
    }

     /** Search if a repeated mark associated with the Node can
     * be directly include in the code of the node by checking their
     * common properties
     * @return the index of the node if one match, else -1
     */
   public int searchBestrMark() {
	int max;
	RMark rmark;
	max = vrMarks.size();
	for (int i = 0; i < max; i++) {
	    rmark = ((RMark)vrMarks.elementAt(i));
	    if (rmark.getGgp() == globalProp
		&& rmark.getplinegray() == plinegray
		&& rmark.getplinecolor()  == plinecolor
		&& rmark.getplinewidth() == getplinewidth()
		&& rmark.getpdash_length() == pdash_length
		&& rmark.getpdash_empty() == getpdash_empty()) {
		return i;
	    }
	}
	    return -1;
    }

    /** get the Gastex Code of each mark
     * excepted for the ones included in the node code
      * @param excludeImark the index of the initial mark which is include 
     * in the node code
     * @param excludeFmark the index of the final mark which is include 
     * in the node code
     * @param excludeRmark the index of the repeated mark which is include 
     * in the node code
     * @return a string containing the code for each mark except the ones
     * that are included in the node code
     */
    String getMarkCode(int excludeImark, int excludeFmark, int excludeRmark) {
	String markCode = "";
	int max;
	max = viMarks.size();
	for (int i = 0; i < max; i++) {
	    if (i != excludeImark) {
		IFMark ifm = (IFMark)viMarks.elementAt(i);
		if (ifm.getGgp() != lastggp) {
		    markCode += ifm.getGgp().getLatexCode(lastggp);
		    lastggp = ifm.getGgp();
		}
		markCode += ifm.getLatexCode();
	    }
	}
	max = vfMarks.size();
	for (int i = 0; i < max; i++) {
	    if (i != excludeFmark) {
		IFMark ifm = (IFMark)vfMarks.elementAt(i);
		if (ifm.getGgp() != lastggp) {
		    markCode += ifm.getGgp().getLatexCode(lastggp);
		    lastggp = ifm.getGgp();
		}
		markCode += ifm.getLatexCode();
	    }
	}
	max = vrMarks.size();
	for (int i = 0; i < max; i++) {
	    if (i != excludeRmark) {
		RMark rm = (RMark)vrMarks.elementAt(i);
		if (rm.getGgp() != lastggp) {
		    markCode += rm.getGgp().getLatexCode(lastggp);
		    lastggp = rm.getGgp();
		}
		markCode += rm.getLatexCode();
	    }
	}
	return markCode;
    }

    /** get the Gastex Properties Code of each label except the first one
     * @return a string containing the code of each label,
     * except the first one */
    public String getLabelCode() {
	String labelCode = "";
	int max;
	max = nodeLabels.size();
	for (int i = 0; i < max; i++) {
	    NodeLabel nl = (NodeLabel)nodeLabels.elementAt(i);
	    if (i != nodeLabelIndex || nl.getGgp() != globalProp) {
		if (nl.getGgp() != lastggp) {
		    labelCode += nl.getGgp().getLatexCode(lastggp);
		    lastggp = nl.getGgp();
		}
		labelCode += nl.getLatexCode();
	    }
	}
	return labelCode;
    }

    /** Get the actual height of the Object
     * Useful if the Object is modified by its content
     * @return the real height of the Object
     */
    public abstract double getActualHeight();
    /** Get the actual height of the Object
     * @param g The graphics on which the object must be draw
     * @return the real height of the Object
     */
    public abstract double getActualHeight(Graphics g);

    /** Get the actual width of the Object
     * Useful if the Object is modified by its content
     * @return the real width of the Object
     */
     public abstract double getActualWidth();
    /** Get the actual width of the Object
     * Useful if the Object is modified by its content
     * @param g The graphics on which the object must be draw
     * @return the real width of the Object
     */
     public abstract double getActualWidth(Graphics g);
    
    /** Get the height of the Object
     * not depending on the content
     * @return the height of the Object
     */
    public abstract double getHeight();
    /** Get the width of the Object
     * not depending on the content
     * @return the width of the Object
     */
   public abstract double getWidth();


    /** add an initial mark with default properties */
    public void addImark() {
	IFMark ifm;
	ifm= new IFMark(globalProp, this, true);
	viMarks.add(ifm);
	newAddedXMark.add(ifm);
    }			
    /** Add a specified initial mark
     * @param ifm the initial mark to add
     */
    public void addImark(IFMark ifm) {
	viMarks.add(ifm);
	newAddedXMark.add(ifm);
    }			
    
    /** add a final mark with default properties */
    public void addFmark() {
	IFMark ifm; 
	ifm = new IFMark(globalProp, this, false);
	vfMarks.add(ifm);
	newAddedXMark.add(ifm);
    }			

    /** Add a specified final mark
     * @param ifm the final mark to add
     */
    public void addFmark(IFMark ifm) {
	vfMarks.add(ifm);
	newAddedXMark.add(ifm);
    }			
    
    /** add a specified  initial or final mark
     * determine the type by itself
     * @param ifm the initial or final mar to add
     */
    public void addIFmark(IFMark ifm) {
	if (ifm.isInitial()) {
	    addImark(ifm);
	} else {
	    addFmark(ifm);
	}
    }
    
    /** add a repeated mark with default properties */
    public void addRmark() {
	RMark rm;
	rm = new RMark(globalProp, this);
	vrMarks.add(rm);
	//newAddedXMark.add(rm);
    }	
		
    /** Add a specified repeated mark
     * @param ifm the repeated mark to add
     */
    public void addRmark(RMark rm) {
	vrMarks.add(rm);
	//newAddedXMark.add(rm);
    }			


    /** remove a specified initial mark
     * @param mark the initial mark to remove
     */
    public void delIMark(IFMark mark) {
	viMarks.remove(mark);
    }
    /** remove a specified final mark
     * @param mark the final mark to remove
     */
    public void delFMark(IFMark mark) {
	vfMarks.remove(mark);
    }
    

    /** remove a specified repeated mark
     * @param mark the repeated mark to remove
     */
    public void delRMark(RMark mark) {
	vrMarks.remove(mark);
    }

	
    /** get thewider object containing the node
     * which is either the node itself, either a repeated mark
     * @return a JasObject wichi is a node or a repeated mark*/
    public JasObject getMaxObject() {
 	int vmax = vrMarks.size();
	JasObject maxObject = this;
	RMark crmark;

	double maxrdist = 0;
	double crdist;
	
	for (int i = 0; i < vmax; i++) {
	    crmark = ((RMark)vrMarks.elementAt(i));
	    crdist = crmark.getprdist();
	    if (crdist > maxrdist) {
		maxObject = crmark;
		maxrdist = crdist;
	    }
	}
	return maxObject;
    }

    /** add a label
     * @param l the label
     */
    public void addLabel(String l) {
	NodeLabel jl;
	nodeLabels.add(jl=new NodeLabel(globalProp, this, l));
	newAddedXMark.add(jl);
    }

    /** Add a label and adjust its properties
     * @param l the label
     * @param pExt to tell if the label is external or not
     * @param pAngle angle of the label from the center of the node
     * @param pDist distance of the label from the center of the node
     * @see NodeLabel
     */
    public void addLabel(String l, boolean pExt, 
			 double pAngle, double pDist) {
	NodeLabel jl;
	nodeLabels.add(jl=new NodeLabel(globalProp, this, l, pExt, pAngle, pDist));
	newAddedXMark.add(jl);
    }
 
    /* add a specified label 
     * @param jl a specified NodeLabel to add
     */ 
   public void addLabel(NodeLabel jl) {
	nodeLabels.add(jl);
	newAddedXMark.add(jl);
    }

    /** delete a label
     * @param jl a specified NodeLabel to remove
     */
    public void delLabel(NodeLabel jl) {
	nodeLabels.remove(jl);
    }

    /** add an edge
     * @param ae the edge to be added to the node
     */
    public void addEdge(AEdge ae) {
	vEdges.add(ae);
    }

    /** remove an Edge
     * @param ae the edge to be removed from the node
     */
    public void rmvEdge(AEdge ae) {
	vEdges.remove(ae);
    }

    /** append all elements of newAddedXMark to a specified vector
     * @param v the vector to which elements must be append
     */
    public void updateVector(Vector v){
// 	int sz=newAddedXMark.size();
// 	int j=0;
// 	while(j<sz){
// 	    v.addElement(newAddedXMark.elementAt(j));
// 	    j++;
// 	}
	v.addAll(newAddedXMark);
	newAddedXMark.clear();
    }

    /** Get the rectangle surrounding a node
     * plus the Rectangles surrounding the initial, final, repeated mark
     * of this node, plus the Rectangles of the edges attached to it
     * @return a rectangle which contains all the ellement attached to the node
     */
    public Rectangle getBoundsExtended() {
	Rectangle r;
	r=getBounds();
	Vector toInvalideIMark=getImarks();
	int sz=toInvalideIMark.size();
	int i;
	// IMark
	for (i=0;i<sz;i++)
	    r.add(((IFMark) toInvalideIMark.elementAt(i)).getBounds());
	
	// FMark
	toInvalideIMark=getFmarks();
	sz=toInvalideIMark.size();
	for (i=0;i<sz;i++)
	    r.add(((IFMark) toInvalideIMark.elementAt(i)).getBounds());
	// RMark
	toInvalideIMark=getRmarks();
	sz=toInvalideIMark.size();
	for (i=0;i<sz;i++)
	    r.add(((RMark) toInvalideIMark.elementAt(i)).getBounds());

	Vector toInvalideLabel=getNodeLabels();
	// label
	sz=toInvalideLabel.size();
	for (i=0;i<sz;i++)
	    r.add(((NodeLabel) toInvalideLabel.elementAt(i)).getBounds());
	
	Vector toInvalideEdge=getEdges();
	sz=toInvalideEdge.size();
	for (i=0;i<sz;i++)
	    r.add(((AEdge) toInvalideEdge.elementAt(i)).getBounds());
	
	//r.grow(2,2);
	return r;
    }

    public Vector getImarks() {	return viMarks; }
    public void   setImarks(Vector im) { viMarks = im; } 
    public void   rmvImarks(IFMark ifm) { viMarks.removeElement(ifm); }

    public Vector getRmarks() {	return vrMarks; }
    public void   setRmarks(Vector rm) { vrMarks = rm; } 
    public void   rmvRmarks(IFMark ifm) { vrMarks.removeElement(ifm); }

    public Vector getFmarks() {	return vfMarks; }
    public void   setFmarks(Vector fm) { vfMarks = fm; } 
    public void   rmvFmarks(IFMark ifm) { vfMarks.removeElement(ifm); }

    public Vector getnewAddedXMark() {return newAddedXMark; }
    public void   setnewAddedXMark(Vector naxm) {newAddedXMark = naxm;}
    public Vector getNodeLabels() { return nodeLabels; }
    public void   setNodeLabels(Vector nls) { nodeLabels = nls; }
    public void   rmvNodeLabels(IFMark ifm) { nodeLabels.removeElement(ifm); }

    public String getNodeName() { return nodeName; }
    public void   setNodeName(String nn) { nodeName = nn; }

    public Vector getEdges() { return vEdges; }
    public void   setEdges(Vector ed) { vEdges = ed; }	 
    public void   rmvEdges(AEdge ae)  { vEdges.removeElement(ae); }

    public void setpNadjust(int pNadjust){ this.pNadjust = pNadjust;}
    public int  getpNadjust() { return pNadjust;} 

    public void   setpNadjustdist(double pNadjustdist)
    { this.pNadjustdist = globalProp.unitLength2Internal(pNadjustdist);}
    public double getpNadjustdist() {
	return globalProp.internal2UnitLength(pNadjustdist);
    } 

    public void   setpNadjustdistTB(double pNadjustdist)
    { this.pNadjustdist = globalProp.unitLength2Internal(pNadjustdist) /
	  globalProp.unitlength ;}
    public double getpNadjustdistTB() {
	return globalProp.internal2UnitLength(pNadjustdist) *
	    globalProp.unitlength;
    } 
   
     public void setNodeLabelIndex(int index) { nodeLabelIndex = index; }
    public int  getNodeLabelIndex() { return nodeLabelIndex; }
		
    public GastexGlobalProperties getLastGgp() { return lastggp; }
    /** create a new instance of the same object
     * adjust the node internal name and all the vectors
     * @return  a new instance of the same object
    */
    public Object clone(){
	ANode node = null;
	node = (ANode) super.clone();
	if (node == null)
	    return null;

	node.setNodeName("n" + nodeNum);
	nodeNum++;

	//cloning vectors
	node.setnewAddedXMark(new Vector());

	node.setImarks(new Vector());
	for (Enumeration e = viMarks.elements() ; e.hasMoreElements() ;) {
	    IFMark cifm = (IFMark)(e.nextElement());
	    IFMark ifm = (IFMark) cifm.clone(node);
	    node.addImark(ifm);
	}
	
	node.setFmarks(new Vector());
	
	for (Enumeration e = vfMarks.elements() ; e.hasMoreElements() ;) {
	    IFMark cifm = (IFMark)(e.nextElement());
	    IFMark ifm = (IFMark) cifm.clone(node);
	    node.addFmark(ifm);
	}

	node.setRmarks(new Vector());
	
	for (Enumeration e = vrMarks.elements() ; e.hasMoreElements() ;) {
	    RMark crm = (RMark)(e.nextElement());
	    RMark rm = (RMark) crm.clone(node);
	    node.addRmark(rm);
	}
	node.setNodeLabels(new Vector());
	for (Enumeration e = nodeLabels.elements() ; e.hasMoreElements() ;) {
	    NodeLabel cnl = (NodeLabel)(e.nextElement());
	    NodeLabel nl = (NodeLabel) cnl.clone(node);
	    node.addLabel(nl);
	}
	
	//remove all edges
	node.setEdges(new Vector());
	return node;
    }

    public abstract Selection getObjectSelection(Rectangle slct);
}
 
