package jastex.old.obj;

import java.awt.geom.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

import jastex.*;
import jastex.parsing.OptionVal;



public class BezierCubicCurve extends AEdge {
    /** the internal representation of the curve */
    public CubicCurve2D.Double currentCurve;

    public Point2D pc1;

    /**
     * the 2 control point of the curve 
     */
    public Point2D pc2;

    public double pc1_angle;

    /**
     * the polar coordinates of the first control point 
     */
    public double pc1_radius;

    public double pc2_angle;

    /**
     * the polar coordinates of the second control point 
     */
    public double pc2_radius;
    /** a flag to determine if this curve is in polar or cartesian */
    public boolean isPolarCoords;

    public Point2D pc1adjusted;

    /**
     * the 2 points needed to set the adjusted by dichotomy curve 
     */
    public Point2D pc2adjusted;

    public double tparam1;

    /**
     * the 2 parameters needed to set the adjusted by dichotomy curve 
     */
    public double tparam2;
     /** the ArrowHeads of the curve */
    public Polygon[] arrowHeads;
    public double tlabelposition;


    // Hack
//     boolean setLoop=false;

    /** Constructor Needed by Edge.java */
    public BezierCubicCurve (ANode sn,ANode en) {
	super(sn, en);
 	isPolarCoords = true;
	sn.addEdge(this);
	en.addEdge(this);
	currentCurve = new CubicCurve2D.Double();
   }
    
    /** Constructor Needed by Edge.java */
    public BezierCubicCurve (GastexGlobalProperties _ggp,ANode sn,ANode en) {
	this(sn, en);
	setGgpValues(_ggp);
    }

    /** A simple constructor
     * @param sn the starting node
     * @param pc1 the first control point 
     * @param en the ending node
     * @param pc2 the second control point 
     */
 //    BezierCubicCurve (ANode sn, Point2D pc1, ANode en, Point2D pc2) {
// 	this(sn,en);
//     }

    /** A constructor which initialize the edge properties
     * using a given GastexGlobalProperties
     * @param _ggp the used GastexGlobalProperties
     * @param sn the starting node
     * @param pc1 the first control point 
     * @param en the ending node
     * @param pc2 the second control point 
     * @see GastexGlobalProperties
     */
    public BezierCubicCurve (GastexGlobalProperties _ggp, ANode sn, Point2D pc1,
		      ANode en, Point2D pc2) {
	this(sn, en);
	setGgpValues(_ggp);
	setCtrlPt1(pc1);
	setCtrlPt2(pc2);
	currentCurve.setCurve(sn.pos2Point2D(), pc1, pc2, en.pos2Point2D());
    }

    public BezierCubicCurve (GastexGlobalProperties _ggp,OptionVal param[],
		      ANode sn, double cp1x, double cp1y,
		      ANode en, double cp2x, double cp2y,
		      String edname,boolean isPolar){
	// constructor
	this(_ggp,sn,en);
	double coeff;
	coeff = globalProp.ratio*Configuration.scale;

	Point2D pct1,pct2;

	if(isPolar){
	    pct1=polar2Cartesian(sn.pos2Point2D(), cp1x*Math.PI/180, 
				 globalProp.unitLength2Internal(cp1y));
	    pct2=polar2Cartesian(en.pos2Point2D(), cp2x*Math.PI/180,
				 globalProp.unitLength2Internal(cp2y));
	} else {
	    pct1=new Point2D.Double(cp1x*coeff,cp1y*coeff);
	    pct2=new Point2D.Double(cp2x*coeff,cp2y*coeff);
	}

	currentCurve = new CubicCurve2D.Double();
	setCtrlPt1(pct1);
	setCtrlPt2(pct2);
	currentCurve.setCurve(sn.pos2Point2D(), pc1, pc2, en.pos2Point2D());
	sn.addEdge(this);
	en.addEdge(this);

	// parametre
	
	if(param!=null) {
	    // arrow param
	    if(!(param[20].optEmpty())){ setpAHnb(param[20].oi); }
	    if(!(param[21].optEmpty())){ setpAHdist(param[21].od); }
	    if(!(param[22].optEmpty())){ setpAHangle(param[22].od); }
	    if(!(param[23].optEmpty())){ setpAHLength(param[23].od); }
	    if(!(param[24].optEmpty())){ setpAHlength(param[24].od); }
	    // line param
	    if(!(param[28].optEmpty())){ setplinegray(param[28].od); }
	    if(!(param[30].optEmpty())){ setplinecolor(param[30].otxt); }
	    if(!(param[32].optEmpty())){ setplinewidth(param[32].od); }
	    if(!(param[33].optEmpty())){
		// dash option
		setpdash_length(param[33].oflist); 
		setpdash_empty(param[33].of); 
	    }
	    // *xo and *yo options
	    if(!(param[34].optEmpty())){ setpsxo(param[34].od); }
	    if(!(param[35].optEmpty())){ setpsyo(param[35].od); }
	    if(!(param[36].optEmpty())){ setpexo(param[36].od); }
	    if(!(param[37].optEmpty())){ setpeyo(param[37].od); }

	}
	// edge label
	if(!(edname.equals(""))){
	    // we have to add a label
	    addLabel(edname);
	    // param for label
	    if(param!=null) {
		if(!(param[25].optEmpty())){ getLabel().setpELside(param[25].oi); }
		if(!(param[26].optEmpty())){ getLabel().setpELpos(param[26].od); }
		if(!(param[27].optEmpty())){ getLabel().setpELdist(param[27].od); }
		if(!(param[38].optEmpty())){ getLabel().setpELdistC(param[38].ob); }
	    }
	}
    }

    /** Set the internal value to the one of the GastexGlobalProperties
     * @param _ggp the GastexGlobaProperties used to initialize values
     * @see GastexGlobalProperties
     */ 

//     public void setGgpValues(GastexGlobalProperties _ggp){
// 	super.setGgpValues(_ggp);
//     }

    /** get the angle of a line, depending on a given point
     * the angle is caculate by the center of the node end the given point
     * @param pt the point used to calculate the angle
     * @return an angle in radian
     */
   public double getAngle(Point2D pt) {
	Point2D center = startingpoint.pos2Point2D();
	Point2D temp = new Point2D.Double(pt.getX(),center.getY());
	double newangle;
	double hypotenuse=center.distance(pt);
	double adjacent  =center.distance(temp);
	if (hypotenuse==0) return 0;
	newangle=Math.acos(adjacent/hypotenuse)*180/Math.PI;
	newangle=(pt.getX()<center.getX())?180-newangle:newangle;
	newangle=(pt.getY()>center.getY())?360-newangle:newangle;
	return newangle;
    }

    /** get the angle made by two points and a horizontal line
     * @param pt1 the first point used to calculate the angle
     * @param pt2 the second point used to calculate the angle
     * @return an angle in radian
     */
    public double getAngle(Point2D pt1, Point2D pt2) {
	double x, y, pente;

	x = pt2.getX() - pt1.getX();
	y = pt2.getY() - pt1.getY();

	if (x==0) {
	    if (y>=0) pente = 1000;
	    else pente = -1000;
	}
	else pente = y/x;
	if (x < 0) return (Math.atan(pente) + Math.PI);
	else return Math.atan(pente);
    }


    /** create an array of polygon containing the arrow heads
     * and store the result in arrowHeads */
     public void getArrowHeads() {
	AffineTransform afr;
	arrowHeads = null;
	 Point2D[] ptSrc, ptDst;
	double          angle;
	double          delta,sx,sy;
	double          coeff;

	arrowHeads = null;
	coeff = Configuration.scale;
	PathIterator pit = currentCurve.getPathIterator(null);
	//0.5 is the precision
	FlatteningPathIterator fit = new FlatteningPathIterator(pit, 0.5);
	double[] coords = {0, 0, 0, 0, 0, 0};
	Point2D[] tab = new Point2D[1024];
	int itNb = 0;

	//get the coordinate of the potential spikes  of the arrows
	while(!fit.isDone() && itNb < 1024) {
	    fit.currentSegment(coords);
	    tab[itNb++] = new Point2D.Double((int)coords[0], (int)coords[1]);
	    fit.next();
	}
	arrowHeads = new Polygon[(int)pAHnb];
	ptSrc = new Point2D[4];
	ptDst = new Point2D[4];
	//the minus is due to the fact that Java turns in ClockWise
	delta = ( pAHLength * coeff * Math.sin(-pAHangle) );
	int index = 0;
 	Point2D pt = tab[itNb-1];

	for (int i=0; i < pAHnb; i++) {
	    double distRest, dist;
	    if (i != 0) {
		distRest = pAHdist * coeff;
		if (index>= (itNb-1)) {
		    return;
		}
		dist = pt.distance(tab[itNb-1-(index+1)]);
		while (dist < distRest) {
		    distRest -= dist;
		    index++;
		    if (index >= (itNb-1)) {
			return;
		    }
		    pt = tab[itNb-1-index];
		    dist = pt.distance(tab[itNb-1-(index+1)]);
		}
		if (distRest != 0) {
		    double alpha = getAngle(tab[itNb-1-index],
					    tab[itNb-1-(index+1)]);
		    pt = new Point2D.Double( pt.getX() + 
					     distRest * Math.cos(alpha),
					     pt.getY() + 
					     distRest * Math.sin(alpha));
		}
	    }
	    ptSrc[0] = pt;
	    sx = ptSrc[0].getX();
	    sy = ptSrc[0].getY();
	    
	    ptSrc[1] = new Point2D.Double( sx+pAHlength*coeff,
					   sy );
	    ptSrc[2] = new Point2D.Double( sx +
					   (pAHLength * coeff *
					    Math.cos(pAHangle)),
					   sy + delta);
	    ptSrc[3] = new Point2D.Double( ptSrc[2].getX(),
					   sy - delta );
	    
	    afr = new AffineTransform();

	    /** get the best angle to put the head and the tail on the curve */
	    int index2 = 1;
	    while (itNb-1-(index+index2) > 0 &&
		   tab[itNb-1-index].distance(tab[itNb-1-(index+index2)]) 
		   < pAHlength*coeff) {
		index2++; 
	    }
	    angle = getAngle(tab[itNb-1-index], tab[itNb-1-(index+index2)]);
	    afr.setToRotation(angle, sx, sy);

	    afr.transform(ptSrc, 0, ptDst, 0, 4);

	    arrowHeads[i] = new Polygon();
	    arrowHeads[i].addPoint(((int)ptDst[0].getX()),
				   ((int)ptDst[0].getY()));
	    arrowHeads[i].addPoint(((int)ptDst[2].getX()),
				   ((int)ptDst[2].getY()));
	    arrowHeads[i].addPoint(((int)ptDst[1].getX()),
				   ((int)ptDst[1].getY()));
	    arrowHeads[i].addPoint(((int)ptDst[3].getX()),
				   ((int)ptDst[3].getY()));

	    
	}
	return;
	
    }

    /** get the intersection point made by the curve and an object
     * by performing a dichotomy
     * use contains() methodes of the object
     * @param n the object
     * @param isStarting to telle if the node represent a starting or
     *  an ending one
     * @return the intersection point
     */
    public Point2D getIntersection(JasObject n, boolean isStarting) {
	double tolerance = 2;
	CubicCurve2D curve, left, right;
	Point2D.Double ptExt, ptInt;
	Point2D.Double ptCen, pt1;
	double paramt=0.5; // determine at the same time t value
	double pas=0.5;
	int tSign=1; // for t param

	ptExt = (Point2D.Double)currentCurve.getP2();
	pt1 = ptExt;
	ptInt = (Point2D.Double)currentCurve.getP1();
	curve = currentCurve;

	// hack here
// 	if (setLoop){
	if (!isStarting) {
	    ptExt = ptInt;
	    ptInt = pt1;
	    pt1 = ptExt;
	    curve = new CubicCurve2D.Double();
	    curve.setCurve(currentCurve.getP2(), currentCurve.getCtrlP2(),
			   currentCurve.getCtrlP1(), currentCurve.getP1());
	    tSign=-1;
	}
// 	}

	left = new CubicCurve2D.Double();
	right = new CubicCurve2D.Double();
	CubicCurve2D.subdivide(curve, left, right);
 	ptCen = null;

	do {
	    ptCen = (Point2D.Double)left.getP2();
	    if(n.contains(ptCen)) {
		curve = right;
		pas=pas/2;
		paramt=paramt+pas*tSign;
		ptInt = (Point2D.Double)curve.getP1();
	    } else {
		curve = left;
		pas=pas/2;
		paramt=paramt-pas*tSign;
		ptExt = (Point2D.Double)curve.getP2();
	    }
	    CubicCurve2D.subdivide(curve, left, right);
    } while ( ptExt.distance(ptInt) > tolerance );

	if (tSign==1) tparam1=paramt;
	else tparam2=paramt;

	return ptCen;
    }


    /** adjust the curve depending on the angle and the length
     * perform a dichotomy and trnaslate it to the intersection point
     */
    public void adjustCurve () {
	Point2D p0, p1, p2, p3;
	//Used to get the intersection point
	JasObject interObject ;

	// reinitiate the Curve
	p0 = startingpoint.pos2Point2D(psxo,psyo);
	p3 = endingpoint.pos2Point2D(pexo,peyo);
	if (isPolarCoords) {
	    pc1 = polar2Cartesian(p0, pc1_angle, pc1_radius);
	    pc2 = polar2Cartesian(p3, pc2_angle, pc2_radius);
	}
				     
	currentCurve.setCurve(p0, pc1, pc2, p3);

	// set the two equations x(t),y(t)
	double p0x=p0.getX(); double p0y=p0.getY();
	double p3x=p3.getX(); double p3y=p3.getY();
	double pc1x=pc1.getX(); double pc1y=pc1.getY();
	double pc2x=pc2.getX(); double pc2y=pc2.getY();
	// WARNING ax^3+bx^2+cx+d
	// { d,c,b,a }
	double[] eqnx={ p0x, -3*p0x+3*pc1x, 3*p0x-6*pc1x+3*pc2x, -p0x+3*pc1x-3*pc2x+p3x};
	double[] eqny={ p0y, -3*p0y+3*pc1y, 3*p0y-6*pc1y+3*pc2y, -p0y+3*pc1y-3*pc2y+p3y};
	// get the two intersection

	interObject = startingpoint.getMaxObject();
	p1 = getIntersection(interObject, true);
// 	setLoop=true;
	interObject = endingpoint.getMaxObject();
	p2 = getIntersection(interObject, false);
// 	setLoop=false;
        double t1=tparam1;
	double t2=tparam2;

	pc1adjusted=new Point2D.Double((t2-t1)*getDeriveeValue(eqnx,t1)/
				       3+p1.getX(),
				       (t2-t1)*getDeriveeValue(eqny,t1)/
				       3+p1.getY());
	pc2adjusted=new Point2D.Double((t1-t2)*getDeriveeValue(eqnx,t2)/
				       3+p2.getX(),
				       (t1-t2)*getDeriveeValue(eqny,t2)/
				       3+p2.getY());

	currentCurve.setCurve(p1, pc1adjusted, pc2adjusted, p2);
    }

    public double getDeriveeValue(double eqn[],double t){
	return (3*eqn[3]*t*t + 2*eqn[2]*t + eqn[1]); 
    }

    public double getEqnValue(double eqn[],double t){
	return (eqn[3]*t*t*t + eqn[2]*t*t + eqn[1]*t + eqn[0]);
    }

    /**Draw the object
     * @param g the Graphics instance on which to draw
     */
    public void draw(Graphics g) {
	Stroke s, olds;
	double coeff;
	coeff = Configuration.scale;
	adjustCurve();
	getArrowHeads();
	g.setColor(getLineColor());
	if(selectStatus()) g.setColor(slctColor);
	if (pdash_length == null) {
	    s = new BasicStroke((float)(plinewidth*coeff));
	} else {
	    int len = pdash_length.length;
	    float[] pdash_length2 = new float[len];
	    for (int i = 0; i < len; i++) {
		pdash_length2[i] = (float)(pdash_length[i]*coeff);
	    }
	    s =  new BasicStroke((float)(plinewidth*coeff),
				 BasicStroke.CAP_BUTT,
				 BasicStroke.JOIN_BEVEL,
				 0,   // Miter limit
				 pdash_length2,
				 pdash_empty);
	}
	olds = ((Graphics2D)g).getStroke();
	((Graphics2D)g).setStroke(s);
	((Graphics2D)g).draw(currentCurve);
	((Graphics2D)g).setStroke(olds);

	for(int i = 0; i < pAHnb; i++) {
	    if (arrowHeads[i] != null) {
		g.fillPolygon(arrowHeads[i]);
		g.drawPolygon(arrowHeads[i]);
	    }
	}
	

	//if (label != null) {
	//    label.draw(g);
	//}

    }


    /** Useful to select the object */
    public double getMinDistance(int x,int y) {
	Point2D p=new Point(x,y);
	double distanceToLine=1000;
	double temp;

	PathIterator pit = currentCurve.getPathIterator(null);
	//0.5 is the precision
	FlatteningPathIterator fit = new FlatteningPathIterator(pit, 0.5);
	double[] coords = {0, 0, 0, 0, 0, 0};
	double[] oldcoords = {0, 0, 0, 0, 0, 0};

	if (!fit.isDone()) {
	    fit.currentSegment(oldcoords);
	}
	Line2D.Double actualline;

	while(!fit.isDone()) {
	    fit.currentSegment(coords);
	    actualline = new Line2D.Double((int)oldcoords[0], (int)oldcoords[1], 
					   (int)coords[0], (int)coords[1]);
	    temp = actualline.ptSegDist(p)-4;
	    if (temp<distanceToLine) distanceToLine=temp;

	    oldcoords[0]=coords[0];	    
	    oldcoords[1]=coords[1];	    
	    fit.next();
	}
	return distanceToLine;



    }
    /** disabled */
    public void setPos(int x,int y) {}
 
     /** Useful to select the object
     * test if the Object is containing a Rectangle
     * @param r The rectangle which is used for the test
     * @return true if the rectangle is contained the Object
     */
   public boolean isInsideRectangle(Rectangle r) {

	PathIterator pit = currentCurve.getPathIterator(null);
	//0.5 is the precision
	FlatteningPathIterator fit = new FlatteningPathIterator(pit, 0.5);
	double[] coords = {0, 0, 0, 0, 0, 0};
	double[] oldcoords = {0, 0, 0, 0, 0, 0};

	if (!fit.isDone()) {
	    fit.currentSegment(oldcoords);
	}

	Line2D.Double actualline;

	while(!fit.isDone()) {
	    fit.currentSegment(coords);
	    actualline = new Line2D.Double((int)oldcoords[0], (int)oldcoords[1],
					   (int)coords[0], (int)coords[1]);
	    // test
	    if(r.intersectsLine(actualline)) return true;
	    oldcoords[0]=coords[0];	    
	    oldcoords[1]=coords[1];	    
	    fit.next();
	}
	return false;
   }

    /** Useful to draw the selection area
     * @return the Rectangle containing the Object
     */
    public Rectangle getBounds() { 
	Rectangle r=currentCurve.getBounds();
	Rectangle rr;
	if(getLabel()!=null && (rr=getLabel().getBounds())!=null ){
	    r.add(rr);
	    r.grow(10,10);
	} 
	int sz=arrowHeads.length;
	for(int i = 0; i <sz ; i++) {
	    if (arrowHeads[i] != null) {
		rr=arrowHeads[i].getBounds();
		rr.grow(5,5);
		r.add(rr);
	    }
	}
	return r; 
    }

    /** Generated the Gastex Properties Code 
     * associated with the current fields values, and which can include
     * and the code associated with the first label
     * @param isStandAlone a boolean to indicate how to format the return string
     * @return a String containing the properties modified compare to 
     * the current GastexGlobalProperties, surrounded by '[' and ']'
     * if isStandAlone is true, else a beginning by a common 
     * or the empty string ("")
     */
    public String getLatexCodeProperties(boolean isStandAlone) {
	String lProp = super.getLatexCodeProperties();

	if (lProp.equals("")) {
	    return "";
	} else {
	    if (isStandAlone) {
		//cut the first comma
		return ("[" + lProp.substring(1) + "]");
	    } else {
		return (lProp);
	    }
	}
    }

   /** Fonction which is used to get the Gastex Code
     * associated with the JasObject
     * @param oriX the X coordinate of the origin
     * @param oriY the Y coordinate of the origin
     * @return the associated latex code
     */     
    public String getLatexCode(double oriX, double oriY) {
	String bcCode = "";
	if (isPolarCoords) {
	    bcCode="\\drawbpedge"+
		getLatexCodeProperties(true)+
		"("+startingpoint.getNodeName()+","+
		(int)(getCtrlPt1_angle())+","+
		roundDouble(getCtrlPt1_radius())+","+
		endingpoint.getNodeName()+","
		+(int)(getCtrlPt2_angle())+","+
		roundDouble(getCtrlPt2_radius())+")"+
		"{"+(label==null?"":label.getLabel())+"}\n";;
	} else {
	    bcCode = "\\drawbcedge"+getLatexCodeProperties(true)+
		"("+startingpoint.getNodeName()+","+
		roundDouble(globalProp.internal2UnitLength(pc1.getX())-oriX)+","+
		roundDouble(globalProp.internal2UnitLength(-pc1.getY())-oriY)+","+
		endingpoint.getNodeName()+","+
		roundDouble(globalProp.internal2UnitLength(pc2.getX())-oriX)+","
		+roundDouble(globalProp.internal2UnitLength(-pc2.getY())-oriY)+")"+
		"{"+(label==null?"":label.getLabel())+"}\n";
	}
	return bcCode;
    }

    /** Convert polar coordinates to cartesian 
     * @param origin the origin
     * @param angle the angle
     * @param raduis the radius
     * @return a point */
    public Point2D polar2Cartesian(Point2D origin, double angle, double radius) {
	double coeff;
	coeff = Configuration.scale;
	radius *= coeff;
	return (new Point2D.Double(origin.getX()+Math.cos(angle)*radius,
				   origin.getY()+Math.sin(angle)*radius));
    }
	

    public Point2D getCtrlPt1(){return pc1;}
    public double getCtrlPt1_angle() {
	return Math.toDegrees(-pc1_angle);
    }
    public double getCtrlPt1_radius() {
	return  globalProp.internal2UnitLength(pc1_radius);
    }
 
   public Point2D getCtrlPt2(){return pc2;}
     public double getCtrlPt2_angle() {
	return Math.toDegrees(-pc2_angle);
    }
    public double getCtrlPt2_radius() {
	return  globalProp.internal2UnitLength(pc2_radius);
    }

   public void setCtrlPt1(Point2D pt){
	if (isPolarCoords) {
	    Point2D.Double nodePt;
	    double coeff;
	    coeff = Configuration.scale;
	    nodePt = startingpoint.pos2Point2D(psxo,psyo);
	    pc1_angle = getAngle(nodePt, pt);
	    pc1_radius = nodePt.distance(pt)/coeff;
	}
	pc1=pt;
    }
    public void setCtrlPt1_angle(double angle) {
	pc1_angle = Math.toRadians(angle);
	if (isPolarCoords) {
	    pc1 = polar2Cartesian(startingpoint.pos2Point2D(psxo,psyo),
				  pc1_angle, pc1_radius);
	}
    }
    public void setCtrlPt1_radius(double radius) {
	pc1_radius = globalProp.unitLength2Internal(radius);
	if (isPolarCoords) {
	    pc1 = polar2Cartesian(startingpoint.pos2Point2D(psxo,psyo),
				  pc1_angle, pc1_radius);
	}
    }

    public void setCtrlPt2(Point2D pt){
	if (isPolarCoords) {
	    Point2D.Double nodePt;
	    double coeff;
	    coeff = Configuration.scale;
	    nodePt = endingpoint.pos2Point2D(pexo,peyo);
	    pc2_angle = getAngle(nodePt, pt);
	    pc2_radius = nodePt.distance(pt)/coeff;
	}
	pc2=pt;
    }
    public void setCtrlPt2_angle(double angle) {
	pc2_angle = Math.toDegrees(angle);
	if (isPolarCoords) {
	    pc2 = polar2Cartesian(startingpoint.pos2Point2D(pexo,peyo),
				  pc2_angle, pc2_radius);
	}
    }
    public void setCtrlPt2_radius(double radius) {
	pc2_radius = globalProp.unitLength2Internal(radius);
	if (isPolarCoords) {
	    pc2 = polar2Cartesian(startingpoint.pos2Point2D(pexo,peyo),
				  pc2_angle, pc2_radius);
	}
    }

    public void setCurve(CubicCurve2D.Double cc) { currentCurve = cc; }

    public void setPolar(boolean polar) {
	isPolarCoords = polar;
    }
    public boolean isPolar() {
	return isPolarCoords;
    }
    public void setArrowHeads(Polygon[]ah) {
	arrowHeads = ah;
    }

    /** set the Position of the label
     * @param percent the position in percent of the label as in Gastex
     */
    public void setcurvePosLabel(double percent){
	Point2D p0, p1, p2, p3;

	p0 = startingpoint.pos2Point2D();
	p3 = endingpoint.pos2Point2D();
	p1=pc1;
	p2=pc2;
	if (isPolarCoords) {
	    p1 = polar2Cartesian(p0, pc1_angle, pc1_radius);
	    p2 = polar2Cartesian(p3, pc2_angle, pc2_radius);
	}
				     
	// set the two equations x(t),y(t)
	double p0x=p0.getX(); double p0y=p0.getY();
	double p3x=p3.getX(); double p3y=p3.getY();
	double pc1x=p1.getX(); double pc1y=p1.getY();
	double pc2x=p2.getX(); double pc2y=p2.getY();
	// WARNING ax^3+bx^2+cx+d
	// { d,c,b,a }
	double[] eqnx={ p0x, -3*p0x+3*pc1x, 3*p0x-6*pc1x+3*pc2x,
			-p0x+3*pc1x-3*pc2x+p3x};
	double[] eqny={ p0y, -3*p0y+3*pc1y, 3*p0y-6*pc1y+3*pc2y,
			-p0y+3*pc1y-3*pc2y+p3y};

        curvePosLabel=new Point2D.Double(getEqnValue(eqnx,percent/100),
					 getEqnValue(eqny,percent/100));

	//curvePosLabel2=curvePosLabel;
	setTLabelPosition(percent);
    }

    public Point2D getCurvePosLabel(){
	return curvePosLabel;
    }

    public void setcurvePosLabel(Point2D pt){
	curvePosLabel = pt;
    }

    /* @return true if the label must be translated */
   public boolean getCurveLabelSide() {
	//return true if the label muste be translated, else false
	double y1 = startingpoint.pos2Point2D().getY();
	double y2 = endingpoint.pos2Point2D().getY();
	return (((y2-y1)<0)?true:false);
    }


    public double getTLabelPosition(){ return tlabelposition; }
    public void setTLabelPosition(double tpos){ 
	tlabelposition=tpos;
	// globalProp.unitLength2Internal(dist);
    }

    public void setcurvePosLabel(double t,double dist){
	setcurvePosLabelInternal(t,globalProp.unitLength2Internal(dist));
    }

    public void setcurvePosLabelInternal(double t2,double dist){
	// the sign of dist determines the side of the label
	double t = t2/100;
	// set pos in internal unit
	Point2D p0, p1, p2, p3;

	p0 = startingpoint.pos2Point2D();
	p3 = endingpoint.pos2Point2D();
	p1=pc1;
	p2=pc2;
	if (isPolarCoords) {
	    p1 = polar2Cartesian(p0, pc1_angle, pc1_radius);
	    p2 = polar2Cartesian(p3, pc2_angle, pc2_radius);
	}
				     
	// set the two equations x(t),y(t)
	double p0x=p0.getX(); double p0y=p0.getY();
	double p3x=p3.getX(); double p3y=p3.getY();
	double pc1x=p1.getX(); double pc1y=p1.getY();
	double pc2x=p2.getX(); double pc2y=p2.getY();
	// WARNING ax^3+bx^2+cx+d
	// { d,c,b,a }
	double[] eqnx={ p0x, -3*p0x+3*pc1x, 3*p0x-6*pc1x+3*pc2x,
			-p0x+3*pc1x-3*pc2x+p3x};
	double[] eqny={ p0y, -3*p0y+3*pc1y, 3*p0y-6*pc1y+3*pc2y,
			-p0y+3*pc1y-3*pc2y+p3y};


	double tgvalx= getEqnValue(eqnx,t);
	double tgvaly= getEqnValue(eqny,t);
	// get the derivate values
	double derivx= getDeriveeValue(eqnx,t);
	double derivy= getDeriveeValue(eqny,t);
	double norm=Math.sqrt(derivx*derivx+derivy*derivy);
	norm=(norm==0)?1:norm;

	curvePosLabel=new Point2D.Double(tgvalx-dist*derivy/norm,
					 tgvaly+dist*derivx/norm);
	curvePosLabel2=new Point2D.Double(tgvalx,
					  tgvaly);
	setTLabelPosition(t2);
    }
    
    
    public Point2D getTangante(double t2) {
	// the sign of dist determines the side of the label
	double t = t2/100;
	// set pos in internal unit
	Point2D p0, p1, p2, p3;

	p0 = startingpoint.pos2Point2D();
	p3 = endingpoint.pos2Point2D();
	p1=pc1;
	p2=pc2;
	if (isPolarCoords) {
	    p1 = polar2Cartesian(p0, pc1_angle, pc1_radius);
	    p2 = polar2Cartesian(p3, pc2_angle, pc2_radius);
	}
				     
	// set the two equations x(t),y(t)
	double p0x=p0.getX(); double p0y=p0.getY();
	double p3x=p3.getX(); double p3y=p3.getY();
	double pc1x=p1.getX(); double pc1y=p1.getY();
	double pc2x=p2.getX(); double pc2y=p2.getY();
	// WARNING ax^3+bx^2+cx+d
	// { d,c,b,a }
	double[] eqnx={ p0x, -3*p0x+3*pc1x, 3*p0x-6*pc1x+3*pc2x,
			-p0x+3*pc1x-3*pc2x+p3x};
	double[] eqny={ p0y, -3*p0y+3*pc1y, 3*p0y-6*pc1y+3*pc2y,
			-p0y+3*pc1y-3*pc2y+p3y};


	double tgvalx= getEqnValue(eqnx,t);
	double tgvaly= getEqnValue(eqny,t);
	// get the derivate values
	double derivx= getDeriveeValue(eqnx,t);
	double derivy= getDeriveeValue(eqny,t);
	return (new Point2D.Double(derivx,derivy));
	
    }







    /** Create a new instance of the same object */
   public Object clone(ANode sn, ANode en) {
 	BezierCubicCurve bcc = null;
	bcc = (BezierCubicCurve) super.clone(sn, en);
	bcc.setCurve((CubicCurve2D.Double) currentCurve.clone());
	bcc.setCtrlPt1((Point2D) pc1.clone());
	bcc.setCtrlPt2((Point2D) pc2.clone());
// 	bcc.getArrowHeads();
	if (curvePosLabel != null) {
	    bcc.setcurvePosLabel((Point2D) curvePosLabel.clone());
	}
	return bcc;
    }
    
    public Selection getObjectSelection(Rectangle slct){
	return (new SelectBezierCubicCurve(this));
    }
}


