package jastex.old.obj;

import java.awt.geom.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

import jastex.*;
import jastex.parsing.OptionVal;



public class BezierQuadCurve extends BezierCubicCurve {

    public Point2D cpoint;
    public double cpoint_angle;  // refer to the starting point
    public double cpoint_radius; // refer to the starting point

    /** Constructor Needed by Edge.java */
    public BezierQuadCurve(ANode sn, ANode en) {
	super(sn, en);
    }

    /** Constructor Needed by Edge.java */
    public BezierQuadCurve (GastexGlobalProperties _ggp,ANode sn,ANode en) {
	this(sn, en);
	setGgpValues(_ggp);
    }
				     
    /** A simple constructor
     * @param sn the starting node
     * @param pc the control point 
     * @param en the ending node
     */
//     BezierQuadCurve (ANode sn, ANode en, Point2D pc) {
// 	super(sn,pc, en, pc);
//     }

   /** A constructor which initialize the edge properties
     * using a given GastexGlobalProperties
     * @param _ggp the used GastexGlobalProperties
     * @param sn the starting node
     * @param pc1 the first control point 
     * @param en the ending node
     * @see GastexGlobalProperties
     */
    public BezierQuadCurve (GastexGlobalProperties _ggp, ANode sn,
		     ANode en, Point2D pc) {
	super(_ggp,sn,pc, en,pc);
	setCtrlPtR(pc);
    }



    public BezierQuadCurve (GastexGlobalProperties _ggp,OptionVal param[],
		     ANode sn, ANode en, double cpx, double cpy,
		     String edname,boolean isAngle){
	// constructor
	super(_ggp,sn,en);
	double coeff;
	coeff = globalProp.ratio*Configuration.scale;
	Point2D pct=new Point2D.Double(cpx*coeff,cpy*coeff);
	isPolarCoords = true;
 	currentCurve = new CubicCurve2D.Double();
	if(!isAngle){
	    //setCtrlPt1(pct);
	    //setCtrlPt2(pct);
	    setCtrlPtR(pct);
	} else {
	    //cpx and cpy are angles
	    setCtrlPt(cpx,cpy);
	}
	//currentCurve.setCurve(sn.pos2Point2D(), pc1, pc2, en.pos2Point2D());
	currentCurve.setCurve(sn.pos2Point2D(), cpoint, cpoint, en.pos2Point2D());
	sn.addEdge(this);
	en.addEdge(this);

	// parametre
	
	if(param!=null) {
	    // arrow param
	    if(!(param[20].optEmpty())){ setpAHnb(param[20].oi); }
	    if(!(param[21].optEmpty())){ setpAHdist(param[21].od); }
	    if(!(param[22].optEmpty())){ setpAHangle(param[22].od); }
	    if(!(param[23].optEmpty())){ setpAHLength(param[23].od); }
	    if(!(param[24].optEmpty())){ setpAHlength(param[24].od); }
	    // line param
	    if(!(param[28].optEmpty())){ setplinegray(param[28].od); }
	    if(!(param[30].optEmpty())){ setplinecolor(param[30].otxt); }
	    if(!(param[32].optEmpty())){ setplinewidth(param[32].od); }
	    if(!(param[33].optEmpty())){
		// dash option
		setpdash_length(param[33].oflist); 
		setpdash_empty(param[33].of); 
	    }
	    // *xo and *yo options
	    if(!(param[34].optEmpty())){ setpsxo(param[34].od); }
	    if(!(param[35].optEmpty())){ setpsyo(param[35].od); }
	    if(!(param[36].optEmpty())){ setpexo(param[36].od); }
	    if(!(param[37].optEmpty())){ setpeyo(param[37].od); }
	}

	// edge label
	if(!(edname.equals(""))){
	    // we have to add a label
	    addLabel(edname);
	    // param for label
	    if(param!=null) {
		if(!(param[25].optEmpty())){ getLabel().setpELside(param[25].oi); }
		if(!(param[26].optEmpty())){ getLabel().setpELpos(param[26].od); }
		if(!(param[27].optEmpty())){ getLabel().setpELdist(param[27].od); }
		if(!(param[38].optEmpty())){ getLabel().setpELdistC(param[38].ob); }
	    }
	}

    }



    /** Generated the Gastex Properties Code 
    * include in the properties     
     * @return a String containing the properties modified compare to 
     * the current GastexGlobalProperties, surrounded by '[' and ']'
     * or the empty string ("")
     */
    public String getLatexCodeProperties() {
	String lProp = super.getLatexCodeProperties(false);

	if (lProp.equals("")) {
	    return "";
	} else {
	//cut the first comma
	return ("[" + lProp.substring(1) + "]");
	}
    }

   /** Fonction which is used to get the Gastex Code
     * associated with the JasObject
     * @param oriX the X coordinate of the origin
     * @param oriY the Y coordinate of the origin
     * @return the associated latex code
     */     
    public String getLatexCode(double oriX, double oriY) {
	String bqCode = "\\drawqbedge"+getLatexCodeProperties()+"("+
	    startingpoint.getNodeName()+","+
	    roundDouble(globalProp.internal2UnitLength(cpoint.getX())-oriX)+","+
	    roundDouble(globalProp.internal2UnitLength(-cpoint.getY())-oriY)+","+
	    endingpoint.getNodeName()+")"+
	    "{"+(label==null?"":label.getLabel())+"}\n";
	return bqCode;
    }


    public boolean setCtrlPt(double sangle, double eangle) {


	double srad =Math.toRadians(sangle);
	double erad =Math.toRadians(eangle);
	Point2D sp  = startingpoint.pos2Point2D(psxo,psyo);
	Point2D ep  = endingpoint.pos2Point2D(pexo,peyo);
	double xa = sp.getX();
	double ya = sp.getY();
	double xd = ep.getX();
	double yd = ep.getY();

	double scos = Math.cos(srad);
	double ecos = Math.cos(erad);
	double ssin = Math.sin(srad);
	double esin = Math.sin(erad);


	double pi = Math.PI;
	double alpha = srad;//pi-srad;
	double beta = erad;//erad-pi;
	double d = -Math.cos(beta) * Math.sin(alpha)
	           +Math.cos(alpha) * Math.sin(beta);
	    //-Math.cos(alpha) * Math.sin(beta)
	    //+Math.cos(beta)*Math.sin(alpha);
	double cx = Math.sin(alpha)*xa - Math.cos(alpha)*ya;
	double cy = Math.sin(beta)*xd - Math.cos(beta)*yd;
	
	
	Point2D.Double cp = 
	    new Point2D.Double((-cx * Math.cos(beta) + cy * Math.cos(alpha)) /d,
			       (cy * Math.sin(alpha) - cx * Math.sin(beta)) /d);

	    //new Point2D.Double((cx * Math.sin(beta) + cy * Math.cos(beta)) /d,
	    //	       (-cy * Math.cos(alpha) - cx * Math.sin(alpha))/d);
			       
	setCtrlPt(cp);
	return true;
    }

    public void setCtrlPt(Point2D ncp) {
	//super.setCtrlPt1(ncp);
	//super.setCtrlPt2(ncp);
	setCtrlPtR(ncp);
    }

    public Point2D getCtrlPt() {
	//return pc1;
	return cpoint;
    } 


    public double getCtrlPt_angle() {
	//return super.getCtrlPt1_angle();
	return Math.toDegrees(-cpoint_angle);
    }
   
    public double getCtrlPt_radius() {
	//return super.getCtrlPt1_radius();
	return globalProp.internal2UnitLength(cpoint_radius);
    }
    
    public void setCtrlPtR(Point2D pt){
	if (isPolarCoords) {
	    Point2D.Double nodePt;
	    double coeff;
	    coeff = Configuration.scale;
	    nodePt = startingpoint.pos2Point2D(psxo,psyo);
	    cpoint_angle = getAngle(nodePt, pt);
	    cpoint_radius = nodePt.distance(pt)/coeff;
	}
	cpoint=pt;
    }
    public void setCtrlPtR_angle(double angle) {
	cpoint_angle = Math.toRadians(angle);
	if (isPolarCoords) {
	    cpoint = polar2Cartesian(startingpoint.pos2Point2D(psxo,psyo),
				     cpoint_angle, cpoint_radius);
	}
    }
    public void setCtrlPtR_radius(double radius) {
	cpoint_radius = globalProp.unitLength2Internal(radius);
	if (isPolarCoords) {
	    cpoint = polar2Cartesian(startingpoint.pos2Point2D(psxo,psyo),
				     cpoint_angle, cpoint_radius);
	}
    }

 
    /** redifined to take in consideration only one control point */
    public void adjustCurve () {
	Point2D pccontrolunique;
	Point2D p0, p3;
	p0 = startingpoint.pos2Point2D(psxo,psyo);
	p3 = endingpoint.pos2Point2D(pexo,peyo);
	if (isPolarCoords) {
	    cpoint = polar2Cartesian(p0, cpoint_angle, cpoint_radius); //pc1
	}
	pccontrolunique=cpoint;//pc1;
	double temp;

	/* abscisse */
	temp = 2 * pccontrolunique.getX() ;
	double x_b = (p0.getX()+temp)/3;
	double x_c = (p3.getX()+temp)/3;
	
	/* ordonnee */
	temp = 2 * pccontrolunique.getY() ;
	double y_b = (p0.getY()+temp)/3;
	double y_c = (p3.getY()+temp)/3;
	super.setCtrlPt1(new Point2D.Double(x_b,y_b));
	super.setCtrlPt2(new Point2D.Double(x_c,y_c));
	super.adjustCurve();
	setCtrlPt(pccontrolunique);
    } 

    public Selection getObjectSelection(Rectangle slct){
	return (new SelectBezierQuadCurve(this));
    }
}


