package jastex.old.obj;

import java.awt.geom.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

import jastex.*;
import jastex.old.gui.JasObjectToolBox;
import jastex.parsing.OptionVal;


public class Circle extends JasObject implements SimpleObjectN {
    public double d;
    public Ellipse2D.Double geom;
    public Circle() {
	super();
	geom = new Ellipse2D.Double();
    }

    public Circle(GastexGlobalProperties _ggp) {
	this();
	setGgpValues(_ggp);
	d = globalProp.unitLength2Internal(8);
    }

    public Circle(GastexGlobalProperties _ggp,OptionVal param[],
	   double dx,double dy,double radius) {
	this(_ggp);
	if(param!=null) {
	    if(!(param[5].optEmpty())){ pNframe=param[5].ob; }
	    if(!(param[6].optEmpty())){ pNfill=param[6].ob; }

	    if(!(param[28].optEmpty())){ setplinegray(param[28].od); }
	    if(!(param[29].optEmpty())){ 
		setpfillgray(param[29].od); 
		pNfill=true;
	    }
	    if(!(param[30].optEmpty())){ setplinecolor(param[30].otxt); }
	    if(!(param[31].optEmpty())){ 
		setpfillcolor(param[31].otxt); 
		pNfill=true;
	    }
	    if(!(param[32].optEmpty())){ setplinewidth(param[32].od); }
	    if(!(param[33].optEmpty())){
		// dash option
		setpdash_length(param[33].oflist); 
		setpdash_empty(param[33].of); 
	    }
	}
	setPos((int)(globalProp.unitLength2Internal(dx)),
	       (int)(globalProp.unitLength2Internal(dy)));
	deprecated_setULX(dx);
	deprecated_setULY(-dy);
	d = globalProp.unitLength2Internal(radius);
    }

    public void draw(Graphics g) {
	int ulx, uly;
	Stroke s, olds;
	calcGeom();
	int ds = (int)(d*Configuration.scale);
	ulx = (int)(deprecated_getX() - d*Configuration.scale/2);
	uly = (int)(deprecated_getY() - d*Configuration.scale/2);
	Color fillColor = getFillColor();
	if (fillColor != null) {
	    g.setColor(fillColor);
	    g.fillOval(ulx,uly,ds,ds);
	}
	olds = ((Graphics2D)g).getStroke();
	if (pNframe) {
	    g.setColor(getLineColor());
	    if(selectStatus()) g.setColor(slctColor);
	    if (pdash_length == null) {
		s = new BasicStroke((float)(plinewidth*Configuration.scale));
	    } else {
		int len = pdash_length.length;
		float[] pdash_length2 = new float[len];
		for (int i = 0; i < len; i++) {
		    pdash_length2[i] = (float)(pdash_length[i]*
					       Configuration.scale);
		}
		s =  new BasicStroke((float)(plinewidth*
					 Configuration.scale),
				     BasicStroke.CAP_BUTT,
				     BasicStroke.JOIN_BEVEL,
				     0,   // Miter limit
				     pdash_length2,
				     (float)(pdash_empty*
					     Configuration.scale));
	    }
	    
	    ((Graphics2D)g).setStroke(s);
	} else {
	    g.setColor(Color.lightGray);
	    if(selectStatus()) g.setColor(slctColor);
	}
	g.drawOval(ulx,uly,ds,ds);
	((Graphics2D)g).setStroke(olds);
    }
    
    /** calculate the internal geometry */
    public void calcGeom() {
  	double coeff;
	coeff = Configuration.scale;
	double ds = d*coeff;
	geom.setFrame(deprecated_getX()-(ds+plinewidth*coeff)/2,
		      deprecated_getY()-(ds+plinewidth*coeff)/2,
		      ds+plinewidth*coeff,
		      ds+plinewidth*coeff);
	
    }

    public void setGeom(Ellipse2D.Double g) {
	geom = g;
    }
    /** Useful to draw the selection area
     * @return the Rectangle containing the Object
     */
    public Rectangle getBounds() {
	Rectangle rect= geom.getBounds();
	rect.grow(2,2);
	return rect;
    }

    /** Useful to select the object
     * test if the Object is containing a Rectangle
     * @param r The rectangle which is used for the test
     * @return true if the rectangle is contained the Object
    */
    public boolean isInsideRectangle(Rectangle r) {
	return geom.intersects(r.getX(),
				   r.getY(),
				   r.getWidth(),
				   r.getHeight());
    }
  /** Fonction which is used to get the Gastex Code
     * associated with the JasObject
     * @param oriX the X coordinate of the origin
     * @param oriY the Y coordinate of the origin
     * @return the associated latex code
     */     
    public String getLatexCode(double oriX, double oriY) {
	String circleCode = null;
	circleCode = "\\drawcircle"+getLatexCodeProperties(true)+"("+
	    roundDouble(deprecated_getULX()-oriX)+","+
 	    roundDouble(deprecated_getULY()-oriY)+","+
	    roundDouble(getDiameter())+")";
	return circleCode;
    }

     /** Useful to select the object */
    public double getMinDistance(int gx,int gy) {
	double coeff;
	coeff = Configuration.scale;
	// needed to be recalculate because it it slightly wider
	double ds = d*coeff;
	Ellipse2D.Double ell =
	    new Ellipse2D.Double(deprecated_getX()-(ds+plinewidth*coeff)/2-5,
				 deprecated_getY()-(ds+plinewidth*coeff)/2-5,
				 ds+plinewidth*coeff+10,
				 ds+plinewidth*coeff+10);
	if (ell.contains(gx,gy)) return 0;
	return 1000;
    }

    /** Useful to Select the object
     * Tell if the Object contains the point P(gx,gy)
     * used by Node and RMark
     * need to be override
     * @param gx the x coordinates of the point to test
     * @param yx the y coordinates of the point to test
     * @return true if the Object contains the points
     */
    public boolean contains(double gx, double gy) {	    
	return geom.contains(gx, gy);
    }

   /** Move the Object
     * @param x the new x position of the object
     * @param y the new y position of the object
     */
    public void setPos(int posx, int posy) {
	deprecated_x=posx;
	deprecated_y=posy;
 	calcGeom();
    }

    public void moveIt(Point pt) {
	super.moveIt(pt);
    }


    public double getDiameter() {
	return (globalProp.internal2UnitLength(d));
    }
    public void setDiameter(double diam) {
	d = globalProp.unitLength2Internal(diam);
    }

    public void setDiameterInternal(double d){ this.d=d; }
    public double getDiameterInternal(){ return d; }


   /** Callback to update the ToolBox when the object move
     *  Called by SurfaceDessin
     *  @param tooBox the ToolBox to Warn
     *  @param manager the ObjectManager
     */
    public void updatingWhenMove(JasObjectToolBox tooBox,ObjetManager manager){
	tooBox.updateNodeX(deprecated_getXTB());
	tooBox.updateNodeY(deprecated_getYTB());
	tooBox.updateCircleDiameter(getDiameter());
    }    

    /** create a new instance of the same object
     * adjust the node internal name and all the vectors
     * @return  a new instance of the same object
    */
    public Object clone(){
	Circle c = null;
	c = (Circle) super.clone();
	if (c == null) return null;
	c.setGeom((Ellipse2D.Double) geom.clone());
	return c;
    }
    public Selection getObjectSelection(Rectangle slct){
	return (new SelectCircle(this));
    }
}
