package jastex.old.obj;

import java.awt.geom.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

import jastex.*;
import jastex.parsing.OptionVal;



public class Edge extends BezierCubicCurve {

    /** Curved Edge */
    public double pcurvedepth;  // depth in unitlength of curvededges between two nodes
                      //   The absolute value of curvedepth defines 
                      //   the distance between the middle of the curved edge 
                      //   and the center of the line joining the two nodes 
                      //   With a positive/negative value the curved edge
                      // is on the left/right of the line joining the two nodes.

    /** Simple constructor which create a straight line
     * by initializing curvedepth to 0
     * @param sn the starting node
     * @param en the ending node
     */
    public Edge(ANode sn, ANode en) {
	super(sn,en);
    }

    /** Constructor
     * @param sn the starting node
     * @param en the ending node
     * @param _ggp the GastexGlobalProperties used to initialise the fields
     * @see GastexGlobalProperties
     * @see ANode
     */
    public Edge(GastexGlobalProperties _ggp,ANode sn, ANode en) {
	this(sn,en);
	setGgpValues(_ggp);
     }
 
     /** Constructor
     * @param sn the starting node
     * @param en the ending node
     * @param _ggp the GastexGlobalProperties used to initialise the fields
     * @param curvedepth the new curve depth of the Edge
     * @see GastexGlobalProperties
     * @see ANode
     */
    public Edge(GastexGlobalProperties _ggp,ANode sn, ANode en, double curvedepth) {
	this(_ggp,sn,en);
	pcurvedepth = globalProp.unitLength2Internal(curvedepth);
    }
    
    // constructeur needed by parser
    public Edge(GastexGlobalProperties _ggp,OptionVal param[],ANode sn, ANode en,String edname){
	this(_ggp,sn,en);
	
	if(param!=null) {
	    // curving option
	    if(!(param[19].optEmpty())){
		pcurvedepth = globalProp.unitLength2Internal(param[19].od);
	    }
	    // arrow param
	    if(!(param[20].optEmpty())){ setpAHnb(param[20].oi); }
	    if(!(param[21].optEmpty())){ setpAHdist(param[21].od); }
	    if(!(param[22].optEmpty())){ setpAHangle(param[22].od); }
	    if(!(param[23].optEmpty())){ setpAHLength(param[23].od); }
	    if(!(param[24].optEmpty())){ setpAHlength(param[24].od); }
	    // line param
	    if(!(param[28].optEmpty())){ setplinegray(param[28].od); }
	    if(!(param[30].optEmpty())){ setplinecolor(param[30].otxt); }
	    if(!(param[32].optEmpty())){ setplinewidth(param[32].od); }
	    if(!(param[33].optEmpty())){
		// dash option
		setpdash_length(param[33].oflist); 
		setpdash_empty(param[33].of); 
	    }
	    // *xo and *yo options
	    if(!(param[34].optEmpty())){ setpsxo(param[34].od); }
	    if(!(param[35].optEmpty())){ setpsyo(param[35].od); }
	    if(!(param[36].optEmpty())){ setpexo(param[36].od); }
	    if(!(param[37].optEmpty())){ setpeyo(param[37].od); }
	}

	// TODO Label
	// edge label
	if(!(edname.equals(""))){
	    // we have to add a label
	    addLabel(edname);
	    // param for label
	    if(param!=null) {
		if(!(param[25].optEmpty())){ getLabel().setpELside(param[25].oi); }
		if(!(param[26].optEmpty())){ getLabel().setpELpos(param[26].od); }
		if(!(param[27].optEmpty())){ getLabel().setpELdist(param[27].od); }
		if(!(param[38].optEmpty())){ getLabel().setpELdistC(param[38].ob); }
	    }
	}
    }

      /** Constructor
     * @param sn the starting node
     * @param en the ending node
     * @param curvedepth the new curve depth of the Edge
     * @see ANode
     */
//     Edge(ANode sn, ANode en, double curvedepth) {
// 	super(sn,en);
// 	sn.addEdge(this);
// 	en.addEdge(this);
// 	pcurvedepth = curvedepth;
// 	currentCurve = new CubicCurve2D.Double();
//     }

     /** Set the internal value to the one of the GastexGlobalProperties
     * @param _ggp the GastexGlobaProperties used to initialize values
     * @see GastexGlobalProperties
     */ 
    public void setGgpValues(GastexGlobalProperties _ggp){
	super.setGgpValues(_ggp);
	pcurvedepth = _ggp.pcurvedepth;
    }

   /** This method adjust the control point of the edge curve
     * to make it just between the two node and at 2*pcurvedepth above
     */
    public void adjustCtrlPt() {
	Point2D p1,p2;
	double angle;
	double diffX, diffY;
	double coeff;
	coeff = Configuration.scale;
	p1 = startingpoint.pos2Point2D(psxo,psyo);
	p2 = endingpoint.pos2Point2D(pexo,peyo);
	diffX = (p1.getX()+p2.getX())/2;
	diffY = (p1.getY()+p2.getY())/2;

 	pc1 = pc2  = new Point2D.Double(diffX,
 				   diffY);
	if (pcurvedepth == 0) {
	    return;
	}
	angle = getAngle(p1,p2)-Math.PI/2;
	pc1 = pc2 = new Point2D.Double(diffX + (2*pcurvedepth 
						* coeff 
						* Math.cos(angle)),
				       diffY + (2*pcurvedepth 
						* coeff 
						* Math.sin(angle)));
    }


    /** This methode adjust the curve by cutting it on the edge off the node
     * and adjust the control point
     */
    public void adjustCurve () {
    Point2D pccontrolunique;
	adjustCtrlPt();
	//FIXME: needed to accelerate things
	//better to draw Line but not sure
// 	if (pcurvedepth == 0) {
// 	    Point2D p1, p2;
	    
// 	    p1 = startingpoint.pos2Point2D();
// 	    p2 = endingpoint.pos2Point2D();
// 	} else {
// 	}

	pccontrolunique=pc1;
	double temp,temp1;

	Point2D p0, p3;
	p0 = startingpoint.pos2Point2D(psxo,psyo);
	p3 = endingpoint.pos2Point2D(pexo,peyo);

	double xx=p3.getX()-p0.getX();
	double yy=p3.getY()-p0.getY();

	temp1=Math.sqrt(Math.abs(xx*xx+yy*yy));
	if (temp1 == 0) {
	    //PANIC : The starting and ending nodes 
	    //should not have the same coordinates.
	    return; 
	    }
	/* abscisse */
	
	temp = p0.getX() + p3.getX() - 4 *
	    pcurvedepth * yy / temp1 *-(Configuration.scale);
	double x_b = (p0.getX()+temp)/3;
	double x_c = (p3.getX()+temp)/3;
	
	/* ordonnee */
	
	temp = p0.getY() + p3.getY() + 4 *
	    pcurvedepth * xx / temp1 *-(Configuration.scale);
	double y_b = (p0.getY()+temp)/3;
	double y_c = (p3.getY()+temp)/3;

	
	setCtrlPt1(new Point2D.Double(x_b,y_b));
	setCtrlPt2(new Point2D.Double(x_c,y_c));

	super.adjustCurve();
	pc1=pc2=pccontrolunique;
	
     }

    /** Return the Gastex Code associated with the current fields values
     * @return a String containing the properties modified compare to 
     * the current GastexGlobalProperties, beginning with a comma or 
     * the empty string ("")
     */
    public String getLatexCodeProperties() {
	String lProp = super.getLatexCodeProperties(false);
	if (pcurvedepth != globalProp.pcurvedepth) {
	    lProp += ",curvedepth="+roundDouble(getpcurvedepth());
	}
	if (lProp.equals("")) {
	    return "";
	} else {
	//cut the first comma
	return ("[" + lProp.substring(1) + "]");
	}
    }

   /** Fonction which is used to get the Gastex Code
     * associated with the JasObject
     * @param oriX the X coordinate of the origin
     * @param oriY the Y coordinate of the origin
     * @return the associated latex code
     */     
    public String getLatexCode(double oriX, double oriY) {
	String lpCode = getLatexCodeProperties();
	String edgeCode = "\\drawedge"+lpCode
	    +"("+startingpoint.getNodeName()+","
	    +endingpoint.getNodeName()+")"+
	    "{"+(label==null?"":label.getLabel())+"}\n";
	return edgeCode;
    }
    
    public double getpcurvedepth() {
	return globalProp.internal2UnitLength(pcurvedepth);
    }
    public void setpcurvedepth(double pcurvedepth) { 
	this.pcurvedepth = globalProp.unitLength2Internal(pcurvedepth);
    }


    public void setCtrlPt(Point2D ncp) {
	pc1 = pc2 = ncp;
    }
    
    public Point2D getCtrlPt() {
	return pc1;
    } 
   

    public Selection getObjectSelection(Rectangle slct){
	return (new SelectEdge(this));
    }
}
    
    

