package jastex.old.obj;

import java.awt.*;
import javax.swing.*;
import java.util.Vector;
import java.io.*;
import java.awt.geom.*;

import jastex.*;
import jastex.old.gui.JasObjectToolBox;
import jastex.parsing.OptionVal;


public class IFMark extends AEdge {
    /** true if Imark, false if Fmark */
    public boolean imark;
    /** length in unitlength of arrows marking initial nodes */
    public double pilength;
    /** direction in degree of arrows marking initial nodes */
    public double piangle;
    /** length in unitlength of arrows marking final nodes */
    public double pflength;
    /** direction in degree of arrows marking final nodes */
    public double pfangle;

    /** the interna representation of the IFMark */
    public Line2D.Double ligne;
    /** the ArrowHeads of the IFMark */
    public Polygon[] arrowHeads;

    /** Basic Constructor */
    public IFMark (ANode n, boolean isImark) {
 	super(n, null);
	imark = isImark;
	ligne = new Line2D.Double();
   }

    /** A constructor which initialize the mark properties
     * using a given GastexGlobalProperties
     * @param _ggp the used GastexGlobalProperties
     * @param n the node to which this IFMark belongs
     * @param isImark the type (initial or final) of the mark
     * @see GastexGlobalProperties
     */
     public IFMark (GastexGlobalProperties _ggp, ANode n, boolean isImark) {
	this(n, isImark);
	setGgpValues(_ggp);
	piangle = _ggp.piangle;
	pilength =_ggp.pilength ;
	pfangle = _ggp.pfangle;
	pflength = _ggp.pflength;

     }
    
    /** A constructor which initialize the mark properties
     * using a given GastexGlobalProperties and provided values
     * @param _ggp the used GastexGlobalProperties
     * @param pia the initial mark angle properties
     * @param pil the initial mark length properties
     * @param pia the final mark angle properties
     * @param pil the final mark length properties
     * @param n the node to which this IFMark belongs
     * @param isImark the type (initial or final) of the mark
     * @see GastexGlobalProperties
     */
    public IFMark (GastexGlobalProperties _ggp, double pia, double pil, double pfa, double pfl, ANode n, boolean isImark) {
	this(_ggp, n, isImark);
	piangle = Math.toRadians(pia);
	pilength = globalProp.unitLength2Internal(pil);
	pfangle = Math.toRadians(pfa);
	pflength = globalProp.unitLength2Internal(pfl);
    }

    // Constructor Needed by parser
    public IFMark (GastexGlobalProperties _ggp,OptionVal param[],ANode n, boolean isImark){
	this(_ggp, n, isImark);
	if(param!=null) {
	    if(!(param[11].optEmpty())){ setplength(param[11].od); }
	    if(!(param[12].optEmpty())){ setpangle(param[12].od); }

	    if(!(param[13].optEmpty())){ setplength(param[13].od); }
	    if(!(param[14].optEmpty())){ setpangle(param[14].od); }

	    if(!(param[20].optEmpty())){ setpAHnb(param[20].oi); }
	    if(!(param[21].optEmpty())){ setpAHdist(param[21].od); }
	    if(!(param[22].optEmpty())){ setpAHangle(param[22].od); }
	    if(!(param[23].optEmpty())){ setpAHLength(param[23].od); }
	    if(!(param[24].optEmpty())){ setpAHlength(param[24].od); }

	    if(!(param[28].optEmpty())){ setplinegray(param[28].od); }
	    //if(!(param[29].optEmpty())){ setpfillgray(param[29].od); }
	    if(!(param[30].optEmpty())){ setplinecolor(param[30].otxt); }
	    //if(!(param[31].optEmpty())){ setpfillcolor(param[31].otxt); }
	    if(!(param[32].optEmpty())){ setplinewidth(param[32].od); }
	    if(!(param[33].optEmpty())){
		// dash option
		setpdash_length(param[33].oflist); 
		setpdash_empty(param[33].of); 
	    }
	}
    }


     /** Set the internal value to the one of the GastexGlobalProperties
     * @param _ggp the GastexGlobaProperties used to initialize values
     * @see GastexGlobalProperties
     */ 
    public void setGgpValues(GastexGlobalProperties _ggp){
	super.setGgpValues(_ggp);
	piangle = _ggp.piangle;
	pilength = _ggp.pflength;
	pfangle = _ggp.piangle;
	pflength = _ggp.pflength;
    }

   /** set the angle of the mark, depending on a given point
     * the angle is caculate by the center of the node end the given point
     * @param pt the point used to calculate the angle
     * @return an angle in radian
     */
    public void updateAngle(Point2D pt) {
	Point2D center = startingpoint.pos2Point2D();
	Point2D temp = new Point2D.Double(pt.getX(),center.getY());
	double newangle;
	double hypotenuse=center.distance(pt);
	double adjacent  =center.distance(temp);
	if (hypotenuse==0) return;
	newangle=Math.acos(adjacent/hypotenuse)*180/Math.PI;
	newangle=(pt.getX()<center.getX())?180-newangle:newangle;
	newangle=(pt.getY()>center.getY())?360-newangle:newangle;
	setpangle(newangle);
    }

    /** get the intersection point made by a line from the center of a node 
     * and a given point, and the node itself
     * @param pt the point used to make the line
     * @return the intersection point
     */
    public Point2D getIntersection(Point2D pt) {
	JasObject interObject;
	Line2D.Double l=new Line2D.Double(startingpoint.pos2Point2D(),pt);
	interObject = startingpoint.getMaxObject();
	return getIntersection(interObject,l);
    }

    /** get the intersection point made by a line and an object
     * by performing a dichotomy
     * use contains() methodes of the object
     * @param n the object
     * @param l the line  
     */
    public Point2D getIntersection(JasObject n,  Line2D.Double l) {
	//a tolerance to stop the dichotomy
	double tolerance = 2;
	Point2D.Double ptExt, ptInt;
	Point2D.Double ptCen;
	double diffX, diffY;

	ptExt = (Point2D.Double)l.getP2();
	ptInt = n.pos2Point2D();

	if (ptExt.equals(ptInt)) {
	    //little trick
	    ptExt.setLocation(ptExt.getX()+
			      Math.cos(-getInternalpangle())*10,
			      ptExt.getY()+
			      Math.sin(-getInternalpangle())*10);
	}
	//extends the line if it is too small
	while(n.contains(ptExt)) {
	    //Make the line longer
	    ptExt.setLocation(ptExt.getX()+(ptExt.getX()-ptInt.getX()),
			      ptExt.getY()+(ptExt.getY()-ptInt.getY()));
	    
	}
	diffX = (ptExt.getX()+ptInt.getX())/2;
	diffY = (ptExt.getY()+ptInt.getY())/2;

	    ptCen = new Point2D.Double(diffX,
				       diffY);
	//starting dichotomy
	//ptExt is the last point seen outside
	//ptInt is the last point seen inside
	//ptCen is the center of ptExt and ptInt
	while (ptExt.distance(ptInt) > tolerance) {
	    ptCen = new Point2D.Double(diffX,
				       diffY);
	    if(n.contains(ptCen)) {
		ptInt = ptCen;
	    } else {
		ptExt = ptCen;
	    }
	diffX = (ptExt.getX()+ptInt.getX())/2;
	diffY = (ptExt.getY()+ptInt.getY())/2;
	}
	return ptCen;
    }

    /** create an array of polygon containing the arrow heads
     * and store the result in arrowHeads */
    public void getArrowHeads() {
	AffineTransform af,aff, afr;
	Line2D.Double   l;
	Point2D[]       ptStd, ptSrc, ptDst;
	double          transX, transY, angle;
	double          delta,sx,sy;
	Point2D         ptIntersect;
	Point2D         p1, p2;
	double          coeff;
	coeff = Configuration.scale;

	if (pAHnb == 0) return; //no arrow head
	
	arrowHeads = new Polygon[(int)pAHnb];

 	angle = -(getInternalpangle());

	l = ligne;
	p1 = l.getP1();
	p2 = l.getP2();
	if (p1 == p2) {
	    if (imark) {
		p2 = new Point2D.Double(p1.getX()+
					Math.cos(angle)*
					Math.max(startingpoint.getHeight(),
						 startingpoint.getWidth()),
					p1.getY()+
					Math.sin(angle)*
					Math.max(startingpoint.getHeight(), 
						 startingpoint.getWidth()));

		l = new Line2D.Double(p1,p2);
	    } else {
		p1 = new Point2D.Double(p2.getX()+
					Math.cos(angle)*
					Math.max(startingpoint.getHeight(),
						 startingpoint.getWidth()),
					p2.getY()+
					Math.sin(angle)*
					Math.max(startingpoint.getHeight(), 
						 startingpoint.getWidth()));
		l = new Line2D.Double(p1,p2);
	    }
	}
		
	//the minus is due to the fact that Java turns in ClockWise
	delta = ( pAHLength * coeff * Math.sin(-pAHangle) );

	ptSrc = new Point2D[4];
	ptDst = new Point2D[4];
	ptStd = new Point2D[4];
	//transform the edge:
	// 0 is the spike
	// 1 is the intersection between the edge and the arrow
	// 2 is the left corner
	// 3 is the right corner
	if (imark) {
	    ptStd[0] = l.getP1();
	} else {
	    ptStd[0] = l.getP2();
	}
	sx = ptStd[0].getX();
	sy = ptStd[0].getY();

	ptStd[1] = new Point2D.Double( sx+pAHlength*coeff,
				       sy );
	ptStd[2] = new Point2D.Double( sx +
				       ( pAHLength * coeff *
					 Math.cos(pAHangle)),
				       sy + delta);
 	ptStd[3] = new Point2D.Double( ptStd[2].getX(),
				       sy - delta );
	
	afr = new AffineTransform();
	afr.setToRotation(angle, sx, sy);
	aff = new AffineTransform();
	aff.setToRotation(Math.toRadians(-180),sx, sy);
	
	for (int i=0; i < pAHnb; i++) {
	    //get the standard coords of an arrow
	    ptSrc[0] = new Point2D.Double(ptStd[0].getX(),ptStd[0].getY());
	    ptSrc[1] = new Point2D.Double(ptStd[1].getX(),ptStd[1].getY());
	    ptSrc[2] = new Point2D.Double(ptStd[2].getX(),ptStd[2].getY());
	    ptSrc[3] = new Point2D.Double(ptStd[3].getX(),ptStd[3].getY());

	    //rotate it
	    afr.transform(ptSrc, 0, ptDst, 0, 4);
	    
 	    if ((!imark && getInternalplength()>0) || (imark && getInternalplength()<0)) {
  		//return it to make it look in the right way
		aff.transform(ptDst, 0, ptSrc, 0, 4);
		ptDst[0] = ptSrc[0];
		ptDst[1] = ptSrc[1];
		ptDst[2] = ptSrc[2];
		ptDst[3] = ptSrc[3];
  	    }
	    
	    //translate it (in case of multiple Arrow Head)
	    af = new AffineTransform();
	    //i*pAHdist*coeff
	    if (!imark) {
		af.setToTranslation(-Math.cos(angle)*(i*pAHdist*coeff),
				    -Math.sin(angle)*(i*pAHdist*coeff));
	    } else {
		af.setToTranslation(Math.cos(angle)*(i*pAHdist*coeff),
				    Math.sin(angle)*(i*pAHdist*coeff));
	    }
	    af.transform(ptDst, 0, ptSrc, 0, 4);
	    
	    //fill the polygon
	    arrowHeads[i] = new Polygon();
	    arrowHeads[i].addPoint(((int)ptSrc[0].getX()),
				   ((int)ptSrc[0].getY()));
	    arrowHeads[i].addPoint(((int)ptSrc[2].getX()),
				   ((int)ptSrc[2].getY()));
	    arrowHeads[i].addPoint(((int)ptSrc[1].getX()),
				   ((int)ptSrc[1].getY()));
	    arrowHeads[i].addPoint(((int)ptSrc[3].getX()),
				   ((int)ptSrc[3].getY()));
	}
    }

    /** adjust the line depending on the angle and the length
     * perform a dichotomy and trnaslate it to the intersection point
     */
    public void adjustLine() {
	AffineTransform af;
	Line2D.Double   l;
	Point2D[]       ptSrc, ptDst;
	double          transX, transY, angle;
	Point2D         ptIntersect;
// 	double coeff;
// 	coeff = Configuration.scale;

	l = nodePos_plength2Ligne2D();
	ptSrc = new Point2D[2];
	ptDst = new Point2D[2];

	//the minus is due to the fact that Java turns in ClockWise
	angle = -getInternalpangle();

	ptSrc[0] = l.getP1();
	ptSrc[1] = l.getP2();

	af = new AffineTransform();
	//first rotate

 	af.setToRotation(angle, l.getX1(), l.getY1()); 
 	af.transform(ptSrc, 0, ptDst, 0, 2);

	//then translate
	l = new Line2D.Double(ptDst[0], ptDst[1]);

	//get the intersection point
	JasObject interObject = startingpoint.getMaxObject();
	ptIntersect = getIntersection(interObject, l);

	transX = ptIntersect.getX() - l.getX1();
	transY = ptIntersect.getY() - l.getY1();
	if (getplength() < 0) {
	    transX = -transX;
	    transY = -transY;
	}
	af.setToTranslation(transX,  transY);	
	af.transform(ptDst, 0, ptSrc, 0, 2);
	ligne.setLine(ptSrc[0].getX(), ptSrc[0].getY(),
			      ptSrc[1].getX(), ptSrc[1].getY());
    }


    /** Return a horizontal line from the center of the node
     * the line is pilength or pflength long depending on the type of the mark
     * @return the line
     */
    public Line2D.Double nodePos_plength2Ligne2D() {
	Point2D.Double nodePos, p2;
	double coeff;

	coeff = Configuration.scale;
	nodePos = startingpoint.pos2Point2D();
	p2 = new Point2D.Double(nodePos.getX()+(getInternalplength()*coeff),
				nodePos.getY());
	return (new Line2D.Double(nodePos, p2));
    }

     /** Generated the Gastex Properties Code 
     * associated with the current fields values
     * @param isStandAlone a boolean to indicate how to format the return string
     * @return a String containing the properties modified compare to 
     * the current GastexGlobalProperties, surrounded by '[' and ']'
     * if isStandAlone is true, else a beginning by a common 
     * or the empty string ("")
     */     
    public String getLatexCodeProperties(boolean isStandAlone) {
	String lProp = super.getLatexCodeProperties();
	if (imark) {
	    if (pilength != globalProp.pilength) {
		lProp += ",ilength="+roundDouble(getpilength());
	    }
	    if (piangle != globalProp.piangle) {
		lProp += ",iangle="+roundDouble(getpiangle());
	    }
	} else {
	    if (pflength != globalProp.pflength) {
		lProp += ",flength="+roundDouble(getpflength());
	    }
	    if (pfangle != globalProp.pfangle) {
		lProp += ",fangle="+roundDouble(getpfangle());
	    }
	}

	if (lProp.equals("")) {
	    return "";
	} else {
	    if (isStandAlone) {
		//cut the first comma
		return ("[" + lProp.substring(1) + "]");
	    } else {
		return (lProp);
	    }
	}
    }


   /** Fonction which is used to get the Gastex Code
     * associated with the JasObject when the imark code is node included
     * in the associate node code
     * @return the associated latex code
     */     
    public String getLatexCode() {
	String lpCode = getLatexCodeProperties(true);
	String markCode;
	if (imark) {
	    markCode = "\\imark"+
		lpCode +
		"("+startingpoint.getNodeName()+")"+"\n";
	} else {
	    markCode = "\\fmark"+
		lpCode +
		"("+startingpoint.getNodeName()+")"+"\n";
	}
	return markCode;
    }

    /**Draw the object
     * @param g the Graphics instance on which to draw
     */
    public void draw(Graphics g) {
	Point2D p1, p2;
	Stroke s, olds;
	double coeff;
	coeff = Configuration.scale;

	adjustLine();
	getArrowHeads();
	g.setColor(getLineColor());
	if(selectStatus()) g.setColor(slctColor);
	p1 = ligne.getP1();
	p2 = ligne.getP2();
	if (pdash_length == null) {
	    s = new BasicStroke((float)(plinewidth*coeff));
	} else {
	    int len = pdash_length.length;
	    float[] pdash_length2 = new float[len];
	    for (int i = 0; i < len; i++) {
		pdash_length2[i] = (float)(pdash_length[i]*coeff);
	    }
	    s =  new BasicStroke((float)(plinewidth*coeff),
				 BasicStroke.CAP_BUTT,
				 BasicStroke.JOIN_BEVEL,
				 0,   // Miter limit
				 pdash_length2,
				 (float)(pdash_empty*coeff));
	}
	olds = ((Graphics2D)g).getStroke();
	((Graphics2D)g).setStroke(s);
	g.drawLine((int)(p1.getX()), (int)(p1.getY()),
		   (int)(p2.getX()), (int)(p2.getY()));

	((Graphics2D)g).setStroke(olds);

	for(int i = 0; i < pAHnb; i++) {
	    g.fillPolygon(arrowHeads[i]);
	    g.drawPolygon(arrowHeads[i]);
	}
	
    }
 
    /** Useful to draw the selection area
     * @return the Rectangle containing the Object
     */
    public Rectangle getBounds() {
	Rectangle r, lr;
	r = ligne.getBounds();
	getArrowHeads();
	for(int i = 0; i < pAHnb; i++)
	    r = r.union(arrowHeads[i].getBounds());
	return r;
    }

    /** Useful to select the object */
    public double getMinDistance(int x,int y) {
	Point2D p=new Point(x,y);
	double distanceToLine=ligne.ptSegDist(p)-4; // 4 is for tolerance
	getArrowHeads();
	for(int i = 0; i < pAHnb; i++)
	    if(arrowHeads[i].contains(p)) return 0;
	return distanceToLine;
    }
    
   /** Move the Object
     * @param x the new x position of the object
     * @param y the new y position of the object
     */
    public void setPos(int x,int y) {
	// in fact very useful for dragging the object in free mode
	Point2D pt=new Point2D.Double(x,y);
	updateAngle(pt);
	return;
    } 
    
    /** Useful to select the object
     * test if the Object is containing a Rectangle
     * @param r The rectangle which is used for the test
     * @return true if the rectangle is contained the Object
     */
    public boolean isInsideRectangle(Rectangle r) { 
	return r.intersectsLine(ligne); 
    }

	
    /** used to tell to the node to removed this imark */
    public void removeFromNode(){
	if (imark) {
	    startingpoint.rmvImarks(this);
	} else {
	    startingpoint.rmvFmarks(this);
	}
    }

    public void setpilength(double pilength){
	this.pilength = globalProp.unitLength2Internal(pilength);
    }
    public double getpilength() {
	return globalProp.internal2UnitLength(pilength);
    }

    public void setpiangle(double piangle){
	this.piangle = Math.toRadians(piangle);
    }
    public double getpiangle() {
	return Math.toDegrees(piangle);
    }

    public void setpflength(double pflength){
	this.pflength = globalProp.unitLength2Internal(pflength);
    }
    public double getpflength() {
	return globalProp.internal2UnitLength(pflength);
    }

    public void setpfangle(double pfangle){ 
	this.pfangle = Math.toRadians(pfangle);
    }
    public double getpfangle() {
	return Math.toDegrees(pfangle);
    }
    public void setplength(double plength){
	if (imark) {
	    setpilength(plength);
	} else {
	    setpflength(plength);
	}
    }
    public double getplength() {
	if (imark) {
	    return getpilength();
	} else {
	    return getpflength();
	}
    }
    public double getInternalplength() {
	if (imark) {
	    return pilength;
	} else {
	    return pflength;
	}
    }


    public double getplengthTB(){
	if (imark) {
	    return (globalProp.internal2UnitLength(pilength) *
		    globalProp.unitlength);
	} else {
	    return (globalProp.internal2UnitLength(pflength) * 
		    globalProp.unitlength);
	}
    }
    public void setplengthTB(double plength) {
	if (imark) {
	    this.pilength = globalProp.unitLength2Internal(plength) /
		globalProp.unitlength;
	} else {
	    this.pflength = globalProp.unitLength2Internal(plength) /
		globalProp.unitlength;
	}
    }

    public void setpangle(double pangle){ 
	if (imark) {
	    setpiangle(pangle); 
	} else {
	    setpfangle(pangle); 
	}
    }
    public double getpangle() { 
	if (imark) {
	    return getpiangle();
	} else {
	    return getpfangle();
	}	    
    }
    public double getInternalpangle() { 
	if (imark) {
	    return piangle;
	} else {
	    return pfangle;
	}	    
    }

    public boolean isInitial() { return imark; }
    public ANode getNode() { return startingpoint; }
    public void setNode(ANode n) { startingpoint = n;}
    public Line2D getLine() { return ligne; }
    public void   setLine(Line2D.Double l) { ligne = l; } 

    public Point2D getcurvePosLabel(){ return null; }
    public void setcurvePosLabel(double percent) {}
    public boolean getCurveLabelSide() {return false;}

    /** Callback to update the ToolBox when the object move
     *  Called by SurfaceDessin
     *  @param tooBox the ToolBox to Warn
     *  @param manager the ObjectManager
     */
    public void updatingWhenMove(JasObjectToolBox tooBox,ObjetManager manager){
	tooBox.updateIFMarkAngle(getpangle());
	tooBox.updateIFMarkLength(getplength());
    }

    /** Create a new instance of the same object */
    public Object clone(){
       IFMark ifm = null;
       ifm = (IFMark) super.clone();
       ifm.setLine((Line2D.Double)ligne.clone());
        return ifm;
  
   }

    /** Create a new instance of the same object and associate it
     * to the given node
     * @param n the node to be associated with the mark 
     */
    public Object clone(ANode n){
	IFMark ifm = null;
	ifm = (IFMark) this.clone();
	ifm.setNode(n);
	ifm.adjustLine();
	return ifm; 
    }
    
    public Selection getObjectSelection(Rectangle slct){
	return (new SelectIMark(this));
    }

    public void setcurvePosLabelInternal(double t,double dist){}
}
