package jastex.old.obj;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.Vector;
import java.io.*;
import java.awt.geom.*;
import jastex.old.gui.JasObjectToolBox;
import jastex.JasColorName;
import jastex.Configuration;
import jastex.ObjetManager;
import jastex.GastexGlobalProperties;


/** This Class is an abstract class which represents all the item that can
 * be put on the Drawing Surface and define all the necessary methods plus
 * some of the properties common to all Gastex object
 */
public abstract class JasObject implements Cloneable {

    /* Properties common to node and edge */
    
    /** gray level used to draw lines. 0=black, 1=white. */
    public double plinegray;  // decimal number between 0 and 1
                      //   gray level used to draw lines. 0=black, 1=white.
    
    /** Color used to draw line, if null, use plinegray */
    public String plinecolor;// color used to draw lines
                      //   The color name should be defined in dvipsnam.def
                      //   and one should include usepackage[usenames]{color}.
                      //   This is to avoid the trouble of defining our own
                      //   colors.
                      //   The drawback is that it is not possible to define and
                      //   use other colors.
                      //   It should not be very restrictive since plenty of
                      //   colours are defined in dvipsnam.def.
                      //   It should not be difficult to add the possibility
                      //   of using new colors if needed. 
    
    /**  width in unitlength of lines */
    public double plinewidth;   // width in unitlength of lines
    /** Set the dash pattern used for drawing postscript paths */
    public float[] pdash_length;
    /** offset allows to start the pattern at some distance 
     * from its beginning */
    public float   pdash_empty;
    //   Set the dash pattern used for drawing postscript paths.
    //   The numbers in the list indicate alternatively lengths
    //   in unitlength of dashes and lengths in unitlength of spaces. 
    //   The list of lenghts is used circularly.
    //   offset allows to start the pattern at some distance from its beginning.
    //   Here are some examples:
    //      dash={}{0} % continuous path
    //      dash={1.5}0 % dashs of length 1.5 and empty spaces of length 1.5
    //      dash={0.2 0.5}0 % looks like a sequence of dots
    //      dash={4 1 1 1}0 % alternation of long and short dashs
    //      dash={1.5}{1.5} % we start with the empty space and not the dash
    //      dash={4}{2} % we start in the middle of the first dash
 
    public boolean pNfill;   //   Automatically set to true by the parameters
                      //   fillgray and fillcolor
   /** gray level used to fill nodes. 0=black, 1=white. */
    public double pfillgray;  // decimal number between 0 and 1

    /** color used to fill nodes */
    public String pfillcolor; // See remarks on plineoclor in JasObject.java.

    /** boolean flag defining whether nodes/others object are framed or not */
    public boolean pNframe;

    int deprecated_x;

    /**
     * Actual position of the Current JasObject 
     */
    int deprecated_y;

    /** Cursor used when the mouse is over the JasObject */
    public Cursor cursor;

    int deprecated_oldx;

    /**
     * Old Position 
     */
    int deprecated_oldy;


    /** Dash index in combolist (dirty hack)**/
    public int dashIndex=-1;
    
    /** Gastex Global Properties
     * @see GastexGlobalProperties
     */
    public GastexGlobalProperties globalProp;

    /** coordinates in unitlength, with the origin in the upperleft corner */
    private double deprecated_ulX;
    private double deprecated_ulY;
       

    /** tell if this object is selected **/
    public boolean isObjectSelected=false; // default is false
    public static Color slctColor=Color.blue;


    /** SImple constructor, maybe not useful */
    public JasObject(){}

    /** Constructor initializing the field with 
     * GastexGlobalProperties default value
     * @param _ggp The GastexGlobalProperties used to init
     * @see GastexGlobalProperties
     */
    public JasObject(GastexGlobalProperties _ggp){
	setGgpValues(_ggp);
    }
    
    /** Set the internal value to the one of the GastexGlobalProperties
     * @param _ggp the GastexGlobaProperties used to initialize values
     * @see GastexGlobalProperties
     */ 
    public void setGgpValues(GastexGlobalProperties _ggp){
	plinegray = _ggp.plinegray;
	plinewidth = _ggp.plinewidth;

	pdash_length = _ggp.pdash_length;
	pdash_empty = _ggp.pdash_empty;

	plinecolor = _ggp.plinecolor;

	globalProp = _ggp;
			
	pfillcolor = _ggp.pfillcolor;
	pfillgray = _ggp.pfillgray;

	pNframe = _ggp.pNframe;
	
	pNfill = _ggp.pNfill;
    }

    public double getplinegray() {
	return plinegray;
    }   
    public void setplinegray(double plinegray) { 
	this.plinegray = plinegray;
    }
    
    public String getplinecolor() { return plinecolor;}   
    public void setplinecolor(String plinecolor) { 
	this.plinecolor = plinecolor;
    }
    public void    setpNfill(boolean pNfill) {this.pNfill = pNfill;}
    public boolean getpNfill() {return pNfill;}
    
    public void   setpfillgray(double pfillgray){ this.pfillgray = pfillgray; }
    public double getpfillgray() { return pfillgray;}

    public void   setpfillcolor(String pfillcolor)
    { this.pfillcolor = pfillcolor; }
    public String getpfillcolor() { return pfillcolor;}

   public void    setpNframe(boolean pNframe) {this.pNframe = pNframe;}
    public boolean getpNframe() {return pNframe;}
    

    
    public double getplinewidth() {
	return globalProp.internal2UnitLength(plinewidth);
    }   
    public void setplinewidth(double plinewidth) { 
	this.plinewidth = globalProp.unitLength2Internal(plinewidth);
    }

   public double getplinewidthTB() {
       return globalProp.internal2UnitLength(plinewidth) *
	   globalProp.unitlength ;
   }   
    public void setplinewidthTB(double plinewidth) { 
	this.plinewidth = globalProp.unitLength2Internal(plinewidth) /
	    globalProp.unitlength ;
    }

    public float[] getpdash_length() {
	float[] pdash_length2;
	if (pdash_length != null) {
	    int len = pdash_length.length;
	    pdash_length2 = new float[len];
	    for (int i = 0; i < len; i++) {
		pdash_length2[i] = 
		    (float)(globalProp.internal2UnitLength(pdash_length[i]));
	    }
	    return pdash_length2;
	} else {
	    return null;
	}
    }   
    public void setpdash_length(float[] pdash_length) {
	float[] pdash_length2;
	if (pdash_length != null) {
	    int len = pdash_length.length;
	    pdash_length2 = new float[len];
	    for (int i = 0; i < len; i++) {
		pdash_length2[i] = 
		    (float)(globalProp.unitLength2Internal(pdash_length[i]));
	    }
	    this.pdash_length = pdash_length2;
	} else {
	    this.pdash_length = null;
	}
    }

    public float getpdash_empty() {
	return (float)globalProp.internal2UnitLength(pdash_empty);
    }   
    public void setpdash_empty(float pdash_empty) { 
	this.pdash_empty = (float)globalProp.unitLength2Internal(pdash_empty);
    }
    
    public double deprecated_getULX() {
	//return the X modified
// 	    return roundDouble(globalProp.internal2UnitLength(x));
	return deprecated_ulX;
    }
    public void deprecated_setULX(double _ulX) {
	deprecated_ulX = roundDouble(_ulX);
    }

    public int deprecated_getX() {
	return (int)Math.round(deprecated_x*Configuration.scale);
    }

    public double deprecated_getXTB() {
// 	return globalProp.internal2UnitLength(x) * globalProp.unitlength;
	return deprecated_ulX * globalProp.unitlength;
    }
    public void deprecated_setXTB(double x) {
	this.deprecated_x = (int)Math.round(globalProp.unitLength2Internal(x) / globalProp.unitlength);
	deprecated_ulX = roundDouble(x/ globalProp.unitlength);	
    }



    public double deprecated_getULY() {
// 	    return roundDouble(-globalProp.internal2UnitLength(y));
	return deprecated_ulY;
    }

    public void deprecated_setULY(double _ulY) {
	deprecated_ulY = roundDouble(_ulY);
    }

    public int deprecated_getY() {
	return (int)Math.round(deprecated_y*Configuration.scale);
    }

    public double deprecated_getYTB() {
// 	return -globalProp.internal2UnitLength(y)  * globalProp.unitlength;
	return (deprecated_ulY* globalProp.unitlength);
    }
    public void deprecated_setYTB(double y) {
	this.deprecated_y = (int)Math.round(globalProp.unitLength2Internal(-y) 
		       / globalProp.unitlength);
	deprecated_ulY = roundDouble(y/ globalProp.unitlength);	
    }


    public GastexGlobalProperties getGgp(){
	return globalProp;
    }

    public void setGgp(GastexGlobalProperties ggp){
	globalProp = ggp;
    }
	
    /** Function used to round a Double using Configuration.roundDecal
     * @param value the number to round up
     * @see Configuration#roundDecal
    */
    public double roundDouble(double value) {
	return ((Math.round(value*Configuration.roundDecal))/
		Configuration.roundDecal);
    }


    /** Return the Gastex Code associated with the current fields values
     * @return a String containing the properties modified compare to 
     * the current GastexGlobalProperties, beginning with a comma or 
     * the empty string ("")
     */
   public String getLatexCodeProperties(boolean isStandAlone) {
	String lProp = "";
	if (plinecolor == null) {
	    if (plinegray != globalProp.plinegray) {
		lProp += ",linegray="+roundDouble(plinegray);
	    }
	} else if (!plinecolor.equals(globalProp.plinecolor)) {
	    lProp += ",linecolor="+plinecolor;
	}
	
	if (plinewidth != globalProp.plinewidth) {
	    lProp += ",linewidth="+
		roundDouble(getplinewidth());
	}

	if (pNframe != globalProp.pNframe) {
		lProp += ",Nframe="+(pNframe?"y":"n");
	}
	
	if (pNfill != globalProp.pNfill) {
		lProp += ",Nfill="+(pNfill?"y":"n");
	}
	
	if (pfillcolor == null) {
	    if (pNfill && (pfillgray != globalProp.pfillgray)) {
		lProp += ",fillgray="+roundDouble(pfillgray);
	    }
	} else {
	    if (pNfill && !pfillcolor.equals(globalProp.pfillcolor)) {
		lProp += ",fillcolor="+pfillcolor;
	    }
	}

	//here, get dash property
	if (!GastexGlobalProperties.isDashEquals(pdash_length,
					       pdash_empty,
					       globalProp.pdash_length,
					       globalProp.pdash_empty)) {
	    lProp += ",dash={";
	    if (pdash_length != null) {
		int len = pdash_length.length;
		for(int i = 0; i < len; i++) {
		    lProp += 
			roundDouble(globalProp.internal2UnitLength(pdash_length[i]));
		    if (i < len-1) {
			lProp += " ";
		    }
		}
	    }		
	    lProp +="}{"+ roundDouble((getpdash_empty())) +"}";
	}
	if (lProp.equals("")) {
	    return "";
	} else {
	    if (isStandAlone) {
		//cut the first comma
		return ("[" + lProp.substring(1) + "]");
	    } else {
		return (lProp);
	    }
	}
    }

     /** get the fill color
     * @return a color, depending on pNfill and (pfillcolor or pfillgray)
     */
    public Color getFillColor() {
	if (pNfill) {
	    if (pfillcolor != null) {
		JasColorName x = new JasColorName(pfillcolor);
		return (x.getColor());
	    }
	    else {
		//gray level
		return (new Color((float)pfillgray,
				  (float)pfillgray,
				  (float)pfillgray));
	    }
	} else {
	    //Not filled
	    return null;
	}
    }

  /** Fonction which is used to get the Gastex Code
     * associated with the JasObject
     * @param oriX the X coordinate of the origin
     * @param oriY the Y coordinate of the origin
     * @return the associated latex code
     */     
     public String getLatexCode(double oriX, double oriY) { return null; }
    
    /**Draw the object
     * @param g the Graphics instance on which to draw
     */
    public abstract void draw(Graphics g);
    
    /** Useful to draw the selection area
     * @return the Rectangle containing the Object
     */
    public abstract Rectangle getBounds();
    
    /** Useful to select the object
     * test if the Object is containing a Rectangle
     * @param r The rectangle which is used for the test
     * @return true if the rectangle is contained the Object
     */
    public abstract boolean isInsideRectangle(Rectangle r);

    /** Useful to select the object */
     public abstract double getMinDistance(int x,int y);

   /** Move the Object
     * @param x the new x position of the object
     * @param y the new y position of the object
     */
    public void setPos(int x,int y) {
	//System.out.println("Test");
	this.deprecated_x = (int) (x/Configuration.scale); // Configuration Scale added
	this.deprecated_y = (int) (y/Configuration.scale);
    }
    


    /** Useful to Select the object
     * Tell if the Object contains the point P(gx,gy)
     * used by Node and RMark
     * need to be override
     * @param gx the x coordinates of the point to test
     * @param yx the y coordinates of the point to test
     * @return true if the Object contains the points
     */
    public boolean contains(double gx, double gy) {
	return false;
    }
    /** Used to perform a dichotomy
     * Tell if the Object contains a specified point
     * used by Node and RMark
     * need to be override
     * @param pt the point to be tested
     * @return true if the Object contains the points
     **/
   public boolean contains(Point2D pt) {
	return false;
    }

    /** Return the Cursor associated with the object
     * @return a cursor
     */
    public Cursor getCursor(){
	return cursor;
    }

    /** Save the actual position of the JasObject */
    public void savePos() {
	deprecated_oldx=deprecated_x;
	deprecated_oldy=deprecated_y;
    }

    /** Translate the JasObject
     * @param pasx the unit of translation on x-axis
     * @param pasy the unit of translation on y-axis
     */
    public void moveIt(int pasx,int pasy){
	deprecated_x+=pasx;
	deprecated_y+=pasy;
	deprecated_setULX(deprecated_getULX()+getGgp().internal2UnitLength(pasx));
	deprecated_setULY(deprecated_getULY()-getGgp().internal2UnitLength(pasy));
    }

    /** Translate the JasObject
     * @param pt the point used to determine the translation
     */ 
    public void moveIt(Point pt){
	deprecated_x+=pt.getX();
	deprecated_y+=pt.getY();
	deprecated_setULX(deprecated_getULX()+getGgp().internal2UnitLength(pt.getX()));
	deprecated_setULY(deprecated_getULY()-getGgp().internal2UnitLength(pt.getY()));
    }
    
    /** get the color used to draw a line
     * @return a color, depending on plinecolor or plinegray
     */
    public Color getLineColor() {
	if (plinecolor != null) {
	    JasColorName x = new JasColorName(plinecolor);
	    return (x.getColor());
	}
	else {
	    //graylevel line
	    return (new Color((float)plinegray,
			      (float)plinegray,
			      (float)plinegray));
	}
    }

    /** return the point associated with the JasObject position
     * @return a Point2D
     */
    public Point2D.Double pos2Point2D() {
	return (new Point2D.Double(deprecated_getX(), deprecated_getY()));
    }

    /** return the point associated with the JasObject position
     * @param xoff x offset
     * @param yoff y offset
     * @return a Point2D
     */
    public Point2D.Double pos2Point2D(double xoff, double yoff) {
	return (new Point2D.Double(deprecated_getX()+xoff, deprecated_getY()-yoff));
    }
    /** Callback to update the ToolBox when the object move
     *  Called by SurfaceDessin
     *  @param tooBox the ToolBox to Warn
     *  @param manager the ObjectManager
     */
    public abstract void updatingWhenMove(JasObjectToolBox tooBox,ObjetManager manager);

    /** Create a new instance of the same object */
    public Object clone(){
	JasObject jo = null;
	try{
	    jo = (JasObject) super.clone();
	} catch (CloneNotSupportedException e) {
	    System.err.println("Internal Error : cloning object failed");
	    return jo;
	}
	//FIXME: maybe need to clone the dashes
	return jo;

	
    }

    public abstract Selection getObjectSelection(Rectangle slct);

    public void setDashIndex(int i)  {	dashIndex = i;    }
    public int getDashIndex() {return dashIndex;} 

    public void selectObject(){ isObjectSelected=true; }
    public void deselectObject(){ isObjectSelected=false; }
    public boolean selectStatus() { return isObjectSelected; }
    
}
