package jastex.old.obj;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import java.util.Vector;
import java.io.*;
import java.awt.geom.*;
import jastex.*;
import jastex.ObjetManager;
import jastex.old.gui.JasObjectToolBox;
import jastex.parsing.OptionVal;

/** This class implements the node as in Gastex, using their properties like 
 * Nw, Nh, Nmr
 */
public class Node extends ANode {
    /** width of node */
    public double pNw;          // width in unitlength of oval nodes
    /** height of node */
    public double pNh;          // height in unitlength of oval nodes
    /** maximal radius of oval nodes */
    public double pNmr;         // maximal radius in unitlength of oval nodes
                      //   The actual radius will be min(Nw/2,Nh/2,Nmr)
                      //   Nmr=0 yields a rectangle node

    /** the internal representation of the node */
     public RoundRectangle2D.Double nodeGeom;
    /** the real width of the node */
    public double actualWidth;
    /** the real height of the node */
    public double actualHeight;

    /** Simple Constructor */
    public Node() {
	super();
	double coeff;
 	nodeGeom = new RoundRectangle2D.Double();
    }

    /* A constructor which initialize the node properties
     * with provided values
     * @param nw the node width
     * @param nh the node height
     * @param nrm the node maximal radius
     */
//     Node (double nw, double nh,double nmr) {
// 	this();
// 	double coeff;
// 	coeff = globalProp.ratio*Configuration.scale;
// 	pNmr = nmr;
// 	pNw  = nw;
// 	pNh = nh;
// 	actualWidth = pNw*coeff;
// 	actualHeight = pNh*coeff;
//     }

    /** A constructor which initialize the node properties
     * using a given GastexGlobalProperties
     * @param _ggp the used GastexGlobalProperties
     * @see GastexGlobalProperties
     */
    public Node(GastexGlobalProperties _ggp) {
	this();
	setGgpValues(_ggp);
	actualWidth = pNw*Configuration.scale;
	actualHeight = pNh*Configuration.scale;
    }

    /** A constructor which initialize the node properties
     * using a given GastexGlobalProperties and provided values
     * @param _ggp the used GastexGlobalProperties
     * @param nw the node width
     * @param nh the node height
     * @param nrm the node maximal radius
     * @see GastexGlobalProperties
     */
    public Node (GastexGlobalProperties _ggp, double nw, double nh, double nmr) {
	this();
	setGgpValues(_ggp);
	pNmr = globalProp.unitLength2Internal(nmr);
	pNw  = globalProp.unitLength2Internal(nw);
	pNh = globalProp.unitLength2Internal(nh);
	actualWidth = pNw*Configuration.scale;
	actualHeight = pNh*Configuration.scale;
    }

    /** A constructor which initialize the node properties
     * using a given GastexGlobalProperties and provided values
     * @param _ggp the used GastexGlobalProperties
     * @param nrm the node maximal radius
     * @see GastexGlobalProperties
     */
    public Node (GastexGlobalProperties _ggp,  double nmr) {
	this();
	setGgpValues(_ggp);
	pNmr = globalProp.unitLength2Internal(nmr);
	actualWidth = pNw*Configuration.scale;
	actualHeight = pNh*Configuration.scale;
    }

    /** Constructor used by the parser */
    public Node(GastexGlobalProperties _ggp,OptionVal param[],String ndname,double dx,double dy,String ndlabel) {
	this(_ggp);

	// add here all only param that are considered
	if(param!=null) {
	    if(!(param[0].optEmpty())){ 
		pNw=globalProp.unitLength2Internal(param[0].od);
		actualWidth = pNw*Configuration.scale;
	    }
	    if(!(param[1].optEmpty())){ 
		pNh=globalProp.unitLength2Internal(param[1].od); 
		actualHeight = pNh*Configuration.scale;
	    }
	    if(!(param[2].optEmpty())){
		pNmr=globalProp.unitLength2Internal(param[2].od);
	    }
	    if(!(param[3].optEmpty())){
		pNadjust= param[3].oi;}
	    if(!(param[4].optEmpty())){ 
		pNadjustdist=globalProp.unitLength2Internal(param[4].od);
	    }
	    if(!(param[5].optEmpty())){ pNframe=param[5].ob; }
	    if(!(param[6].optEmpty())){ pNfill=param[6].ob; }

	    if(!(param[28].optEmpty())){ setplinegray(param[28].od); }
	    if(!(param[29].optEmpty())){ 
		setpfillgray(param[29].od); 
		pNfill=true;
	    }
	    if(!(param[30].optEmpty())){ setplinecolor(param[30].otxt); }
	    if(!(param[31].optEmpty())){ 
		setpfillcolor(param[31].otxt); 
		pNfill=true;
	    }
	    if(!(param[32].optEmpty())){ setplinewidth(param[32].od); }
	    if(!(param[33].optEmpty())){
		// dash option
		setpdash_length(param[33].oflist); 
		setpdash_empty(param[33].of); 
	    }
	    // TODO DASH

	    if(!(param[10].optEmpty())){ 
		pNmarks=param[10].oi;
		//System.out.println(pNmarks);
		if( (pNmarks&1)==1 ) {
		    //imark
		    //System.out.println("youhou");
		    addImark();
		    IFMark ifm=(IFMark) (getImarks().lastElement());
		    if(param!=null) {
			if(!(param[11].optEmpty())){ ifm.setplength(param[11].od); }
			if(!(param[12].optEmpty())){ ifm.setpangle(param[12].od); }
			if(!(param[20].optEmpty())){ ifm.setpAHnb(param[20].oi); }
			if(!(param[21].optEmpty())){ ifm.setpAHdist(param[21].od); }
			if(!(param[22].optEmpty())){ ifm.setpAHangle(param[22].od); }
			if(!(param[23].optEmpty())){ ifm.setpAHLength(param[23].od); }
			if(!(param[24].optEmpty())){ ifm.setpAHlength(param[24].od); }
		    }
		}
		if( (pNmarks&2)==2 ) {
		    //fmark 
		    addFmark();
		    IFMark ifm=(IFMark) (getFmarks().lastElement());
		    if(param!=null) {
			if(!(param[13].optEmpty())){ ifm.setplength(param[13].od); }
			if(!(param[14].optEmpty())){ ifm.setpangle(param[14].od); }
			if(!(param[20].optEmpty())){ ifm.setpAHnb(param[20].oi); }
			if(!(param[21].optEmpty())){ ifm.setpAHdist(param[21].od); }
			if(!(param[22].optEmpty())){ ifm.setpAHangle(param[22].od); }
			if(!(param[23].optEmpty())){ ifm.setpAHLength(param[23].od); }
			if(!(param[24].optEmpty())){ ifm.setpAHlength(param[24].od); }
		    }
		}
		
		if( (pNmarks&4)==4 ) {
		    //rmark 
		    addRmark();
		    RMark rm=(RMark) (getRmarks().lastElement());
		    if(param!=null) {
			if(!(param[15].optEmpty())){ rm.setprdist(param[15].od); }
		    }
		}
	    }
	    
	}
	// ------------

	if(!(ndlabel.equals(""))){
	    // we have to add a label
	    addLabel(ndlabel);
	    NodeLabel nl=(NodeLabel) (getNodeLabels().elementAt(0));
	    if(param!=null) {
		if(!(param[7].optEmpty())){ nl.setpExtNL(param[7].ob); }
		if(!(param[8].optEmpty())){ nl.setpNLangle(param[8].od); }
		if(!(param[9].optEmpty())){ nl.setpNLdist(param[9].od); }
	    }
	}
	nodeName=ndname;
// 	double coeff = globalProp.ratio*Configuration.scale;
	setPos((int)(globalProp.unitLength2Internal(dx)),
	       (int)(globalProp.unitLength2Internal(dy)));
	deprecated_setULX(dx);
	deprecated_setULY(-dy);
    }
    // --

    /** Set the internal value to the one of the GastexGlobalProperties
     * @param _ggp the GastexGlobaProperties used to initialize values
     * @see GastexGlobalProperties
     */ 
    public void setGgpValues(GastexGlobalProperties _ggp){
	super.setGgpValues(_ggp);
	pNw = _ggp.pNw;
	pNh = _ggp.pNh;
	pNmr = _ggp.pNmr;
    }
    /** Get the actual width of the Object
     * Useful if the Object is modified by its content
     * @param g The graphics on which the object must be draw
     * @return the real width of the Object
     */
    public double getActualWidth(Graphics g) {
 	Rectangle2D labelRect;
 	double labelWidth;
	if ((pNadjust&1) != 0 && nodeLabels.size() > nodeLabelIndex) {
	    labelRect = ((NodeLabel)
			 nodeLabels.elementAt(nodeLabelIndex)).getStringBounds2D(g);
	    labelWidth = labelRect.getWidth()+
		pNadjustdist*Configuration.scale;
// 	    return ((maxWidth<pNw*coeff)?pNw*coeff:maxWidth);
	    return (labelWidth);
	} else {
	    return (pNw*Configuration.scale);
	}

	
    }
	
    /** Get the actual width of the Object
     * Useful if the Object is modified by its content
     * @return the real width of the Object
     */
    public double getActualWidth() {
	return actualWidth;
    }
 
    /** Get the actual height of the Object
     * @param g The graphics on which the object must be draw
     * @return the real height of the Object
     */
   public double getActualHeight(Graphics g) {
 	Rectangle2D labelRect;
 	double labelHeight;
	if ((pNadjust&2) != 0 && nodeLabels.size() > nodeLabelIndex) {
	    labelRect = ((NodeLabel)
			 nodeLabels.elementAt(nodeLabelIndex)).getStringBounds2D(g);
	    labelHeight = labelRect.getHeight()+
		pNadjustdist*Configuration.scale;
// 	    return ((maxHeight<pNh*coeff)?pNh*coeff:maxHeight);
	    return (labelHeight);
	} else {
	    return (pNh*Configuration.scale);
	}
    }

    /** Get the actual height of the Object
     * Useful if the Object is modified by its content
     * @return the real height of the Object
     */
   public double getActualHeight() {
	return actualHeight;
    }
    
    /** Get the height of the Object
     * not depending on the content
     * @return the height of the Object
     */
    public double getHeight() {
	return pNh;
    }

   

    /** Get the width of the Object
     * not depending on the content
     * @return the width of the Object
     */
    public double getWidth() {
	return pNw;
    }

   /**Draw the object
     * @param g the Graphics instance on which to draw
     */
    public void draw(Graphics g) {
	int ulx, uly, width, height,arc;
	Stroke s, olds;

	actualWidth =  getActualWidth(g);
	actualHeight=  getActualHeight(g);
	calcNodeGeom();
	width = (int)(actualWidth);
	height = (int)(actualHeight);
	arc = (int)(pNmr * Configuration.scale);
	ulx = (int)((deprecated_getX()-(actualWidth)/2));
	uly = (int)((deprecated_getY()-(actualHeight)/2));
	//fill the node
	Color fillColor = getFillColor();
	if (fillColor != null) {
	    g.setColor(fillColor);
	    if (arc != 0) {
		g.fillRoundRect(ulx, uly, width, height, arc*2, arc*2);
	    } else {
		g.fillRect(ulx, uly, width, height);
	    }
	}
	olds = ((Graphics2D)g).getStroke();
	//draw its frame with the chosen color or graylevel if needed,
	if (pNframe) {
	    g.setColor(getLineColor());
	    if(selectStatus()) g.setColor(slctColor);
	    if (pdash_length == null) {
		s = new BasicStroke((float)(plinewidth*Configuration.scale));
	    } else {
		int len = pdash_length.length;
		float[] pdash_length2 = new float[len];
		for (int i = 0; i < len; i++) {
		    pdash_length2[i] = (float)(pdash_length[i]*
					       Configuration.scale);
		}
		s =  new BasicStroke((float)(plinewidth*
					     Configuration.scale),
				     BasicStroke.CAP_BUTT,
				     BasicStroke.JOIN_BEVEL,
				     0,   // Miter limit
				     pdash_length2,
				     (float)(pdash_empty*
					     Configuration.scale));
	    }
				     
	    ((Graphics2D)g).setStroke(s);
	} else { //else draw with a lightgray
	    g.setColor(Color.lightGray);
	    if(selectStatus()) g.setColor(slctColor);
	}
	if (arc != 0) {
	    g.drawRoundRect(ulx, uly, width, height, arc*2, arc*2);
	} else {
	    g.drawRect(ulx, uly, width, height);
	}
	((Graphics2D)g).setStroke(olds);
	int sz=vrMarks.size();
	for(int ri=0;ri<sz;ri++){
	    ((RMark) vrMarks.elementAt(ri)).draw(g);
	}
	
    }


    /** calculate the node interna geometry */
     public void calcNodeGeom() {
	double coeff;
	coeff = Configuration.scale;
	nodeGeom.setRoundRect(deprecated_getX()-(actualWidth+plinewidth*coeff)/2,
			      deprecated_getY()-(actualHeight+plinewidth*coeff)/2,
			      actualWidth+plinewidth*coeff,
			      actualHeight+plinewidth*coeff,
			      pNmr*coeff*2,
			      pNmr*coeff*2);
    }
   
    /** Useful to draw the selection area
     * @return the Rectangle containing the Object
     */
    public Rectangle getBounds() {
	Rectangle rect= nodeGeom.getBounds();
	int sz=vrMarks.size();
	for(int ri=0;ri<sz;ri++){
	    rect.add(((RMark) vrMarks.elementAt(ri)).getBounds());
	}
	//rect.grow(2,2);
	return rect;
    }


    /** Useful to select the object
     * test if the Object is containing a Rectangle
     * @param r The rectangle which is used for the test
     * @return true if the rectangle is contained the Object
    */
    public boolean isInsideRectangle(Rectangle r) {
	return nodeGeom.intersects(r.getX(),
				   r.getY(),
				   r.getWidth(),
				   r.getHeight());
    }

    /** Useful to select the object */
    public double getMinDistance(int gx,int gy) {
	double coeff;
	coeff = Configuration.scale;
	// needed to be recalculate because it it slightly wider
	RoundRectangle2D.Double rr =
	    new RoundRectangle2D.Double(deprecated_getX()-(actualWidth+plinewidth*coeff)/2-5,
					deprecated_getY()-(actualHeight+plinewidth*coeff)/2-5,
					(actualWidth+plinewidth*coeff)+10,
					(actualHeight+plinewidth*coeff)+10,
					pNmr*coeff*2,
					pNmr*coeff*2);
	if (rr.contains(gx, gy)) return 0;
	return 1000;
    }

    /** Useful to Select the object
     * Tell if the Object contains the point P(gx,gy)
     * used by Node and RMark
     * need to be override
     * @param gx the x coordinates of the point to test
     * @param yx the y coordinates of the point to test
     * @return true if the Object contains the points
     */
    public boolean contains(double gx, double gy) {	    
	return nodeGeom.contains(gx, gy);
    }

    /** Used to perform a dichotomy
     * Tell if the Object contains a specified point
     * used by Node and RMark
     * need to be override
     * @param pt the point to be tested
     * @return true if the Object contains the points
     **/
    public boolean contains (Point2D pt) {
	return contains(pt.getX(), pt.getY());
    }

   /** Move the Object
     * @param x the new x position of the object
     * @param y the new y position of the object
     */
    public void setPos(int posx, int posy) {
	deprecated_x=(int)(posx/Configuration.scale); // Configuration Scale added;
	deprecated_y=(int)(posy/Configuration.scale);
 	calcNodeGeom();
    }
    public void   setpNw(double pNw){
	this.pNw = globalProp.unitLength2Internal(pNw);
    }
    public double getpNw() { return globalProp.internal2UnitLength(pNw);}

    public void   setpNwTB(double pNw){
	this.pNw = globalProp.unitLength2Internal(pNw) /
	    globalProp.unitlength;
    }
    public double getpNwTB() {
	return globalProp.internal2UnitLength(pNw) * globalProp.unitlength;
    }


    public void   setpNh(double pNh){
	this.pNh = globalProp.unitLength2Internal(pNh);
    }
    public double getpNh() { return globalProp.internal2UnitLength(pNh);}
    
    public void   setpNhTB(double pNh){
	this.pNh = globalProp.unitLength2Internal(pNh)/
	    globalProp.unitlength;
    }
    public double getpNhTB() {
	return globalProp.internal2UnitLength(pNh) * globalProp.unitlength;
    }


    public void   setpNmr(double pNmr){
	this.pNmr = globalProp.unitLength2Internal(pNmr);
    }
    public double getpNmr() { return globalProp.internal2UnitLength(pNmr);} 
    public void   setpNmrTB(double pNmr){
	this.pNmr = globalProp.unitLength2Internal(pNmr)/
	    globalProp.unitlength;
    }
    public double getpNmrTB() { return globalProp.internal2UnitLength(pNmr)
				    * globalProp.unitlength;} 
    public double getInternalpNmr() { return pNmr; }
    public void   setNodeGeom(RoundRectangle2D.Double rr) {
	nodeGeom = rr;
    }

    /** Callback to update the ToolBox when the object move
     *  Called by SurfaceDessin
     *  @param tooBox the ToolBox to Warn
     *  @param manager the ObjectManager
     */
    public void updatingWhenMove(JasObjectToolBox tooBox,ObjetManager manager){
	tooBox.updateNodeX(deprecated_getXTB());
	tooBox.updateNodeY(deprecated_getYTB());
	tooBox.updateNodeW(getpNwTB());
	tooBox.updateNodeH(getpNhTB());
	tooBox.updateNodeNmr(getpNmrTB());
    }

    /** Generated the Gastex Properties Code 
     * associated with the current fields values, and which can include
     * also the code associated with one initial and/or final mark,
     * and the code associated with the first label
     * @param bestIMark the index in the initial mark vector which can be
     * include in the properties
     * @param bestFMark the index in the final mark vector which can be
     * include in the properties
     * @param bestRMark the index in the repeated mark vector which can be
     * include in the properties     
     * @return a String containing the properties modified compare to 
     * the current GastexGlobalProperties, surrounded by '[' and ']'
     * or the empty string ("")
     */
    public String getLatexCodeProperties(int bestIMark, int bestFMark, int bestRMark) {
	String lProp = super.getLatexCodeProperties(bestIMark,bestFMark,
						    bestRMark,false);
	if (pNw != globalProp.pNw) {
		lProp += ",Nw="+roundDouble(globalProp.internal2UnitLength(pNw));
	}
	if (pNh != globalProp.pNh) {
		lProp += ",Nh="+roundDouble(globalProp.internal2UnitLength(pNh));
	}
	if (pNmr != globalProp.pNmr) {
		lProp += ",Nmr="+roundDouble(globalProp.internal2UnitLength(pNmr));
	}
	
	if (lProp.equals("")) {
	    return "";
	} else {
	//cut the first comma
	return ("[" + lProp.substring(1) + "]");
	}
    }

   /** Fonction which is used to get the Gastex Code
     * associated with the JasObject
     * @param oriX the X coordinate of the origin
     * @param oriY the Y coordinate of the origin
     * @return the associated latex code
     */     
    public String getLatexCode(double oriX, double oriY) {
	String lCode;
	String nodeCode;
	String markCode;
	String labelCode;
	int bestIMark, bestFMark, bestRmark;
	lastggp = globalProp;

	bestIMark = searchBestiMark();
	bestFMark = searchBestfMark(bestIMark);
	bestRmark = searchBestrMark();
	String nodeProp = getLatexCodeProperties(bestIMark, bestFMark, bestRmark);
	nodeCode = "\\node" +  nodeProp + 
	    "("+nodeName+")"+"("+
	    roundDouble(deprecated_getULX()-oriX)+","+
	    roundDouble(deprecated_getULY()-oriY)+")"+"{";
	if (nodeLabels.size() > nodeLabelIndex && nodeLabelIndex >= 0) {
	    NodeLabel nl = (NodeLabel)nodeLabels.elementAt(nodeLabelIndex);
	    if (nl.getGgp() == globalProp) {
		nodeCode += nl.getLabel();
	    }
	}
	nodeCode += "}"+"\n";
	markCode = getMarkCode(bestIMark, bestFMark, bestRmark);
	labelCode = getLabelCode();
	lCode = nodeCode+markCode + labelCode;
	return lCode;
    }

    /** create a new instance of the same object
     * adjust the node internal name and all the vectors
     * @return  a new instance of the same object
    */
    public Object clone(){
	Node node = null;
	node = (Node) super.clone();
	if (node == null)
	    return null;
	node.setNodeGeom((RoundRectangle2D.Double) nodeGeom.clone());
	return node;
    }

    public Selection getObjectSelection(Rectangle slct){
	//slct.grow(10,10);
	SelectBox sb=new SelectBox(slct);
	sb.setSelectBoxNode(this);
	sb.setResizable(true); 
	return sb;
    }

    public void selectObject(){ 
	isObjectSelected=true;
	int sz=vrMarks.size();
	for(int ri=0;ri<sz;ri++){
	    ((RMark) vrMarks.elementAt(ri)).selectObject();
	}
    }
    public void deselectObject(){ 
	isObjectSelected=false; 
	int sz=vrMarks.size();
	for(int ri=0;ri<sz;ri++){
	    ((RMark) vrMarks.elementAt(ri)).deselectObject();
	}
    }

    public double getNmrPercent(){
	double maxV=Math.min(pNh/2,pNw/2);
	return (pNmr/maxV);
    }
    
    public void setNmrPercent(double percent){
	double maxV=Math.min(pNh/2,pNw/2);
	pNmr=percent*maxV;
    }

}


/*
int sz=vrMarks.size();
	for(int ri=0;ri<sz;ri++){
	    ((RMark) vrMarks.elementAt(ri)).draw(g);
	}


 */
