package jastex.old.obj;

import java.awt.geom.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

import jastex.*;
import jastex.old.gui.JasObjectToolBox;
import jastex.parsing.OptionVal;


public class Oval extends JasObject implements SimpleObjectN {
    public double w;
    public double h;
    public double mr;
    public RoundRectangle2D.Double geom;
    public Oval() {
	super();
	geom = new RoundRectangle2D.Double();
    }

    public Oval(GastexGlobalProperties _ggp) {
	this();
	setGgpValues(_ggp);
	w = globalProp.unitLength2Internal(16);
	h = globalProp.unitLength2Internal(8);
	mr = globalProp.unitLength2Internal(4);
    }

    public Oval(GastexGlobalProperties _ggp,OptionVal param[],
	 double dx,double dy,double dw,double dh,double mr) {
	this(_ggp);
	if(param!=null) {
	    if(!(param[5].optEmpty())){ pNframe=param[5].ob; }
	    if(!(param[6].optEmpty())){ pNfill=param[6].ob; }

	    if(!(param[28].optEmpty())){ setplinegray(param[28].od); }
	    if(!(param[29].optEmpty())){ 
		setpfillgray(param[29].od); 
		pNfill=true;
	    }
	    if(!(param[30].optEmpty())){ setplinecolor(param[30].otxt); }
	    if(!(param[31].optEmpty())){ 
		setpfillcolor(param[31].otxt); 
		pNfill=true;
	    }
	    if(!(param[32].optEmpty())){ setplinewidth(param[32].od); }
	    if(!(param[33].optEmpty())){
		// dash option
		setpdash_length(param[33].oflist); 
		setpdash_empty(param[33].of); 
	    }
	}
	setPos((int)(globalProp.unitLength2Internal(dx)),
	       (int)(globalProp.unitLength2Internal(dy)));
	deprecated_setULX(dx);
	deprecated_setULY(-dy);
	setWidth(dw);
	setHeight(dh);
	setMr(mr);
    }


    public void draw(Graphics g) {
	int ulx, uly, ws, hs, mrs;
	Stroke s, olds;
	calcGeom();
	ulx = (int)(deprecated_getX() - w* Configuration.scale/2);
	uly = (int)(deprecated_getY() - h* Configuration.scale/2);
	Color fillColor = getFillColor();
	ws = (int)(w* Configuration.scale);
	hs = (int)(h* Configuration.scale);
	mrs = (int)(mr* Configuration.scale*2);
	if (fillColor != null) {
	    g.setColor(fillColor);
	    g.fillRoundRect(ulx,uly,ws,hs,mrs,mrs);
	}
	olds = ((Graphics2D)g).getStroke();
	if (pNframe) {
	    g.setColor(getLineColor());
	    if(selectStatus()) g.setColor(slctColor);
	    if (pdash_length == null) {
		s = new BasicStroke((float)(plinewidth*Configuration.scale));
	    } else {
		int len = pdash_length.length;
		float[] pdash_length2 = new float[len];
		for (int i = 0; i < len; i++) {
		    pdash_length2[i] = (float)(pdash_length[i]*
					       Configuration.scale);
		}
		s =  new BasicStroke((float)(plinewidth*
					     Configuration.scale),
				     BasicStroke.CAP_BUTT,
				     BasicStroke.JOIN_BEVEL,
				     0,   // Miter limit
				     pdash_length2,
				     (float)(pdash_empty*
					     Configuration.scale));
	    }
	((Graphics2D)g).setStroke(s);
	} else {
	    g.setColor(Color.lightGray);
	    if(selectStatus()) g.setColor(slctColor);
	}
	if (mrs != 0) {
	    g.drawRoundRect(ulx,uly,ws,hs,mrs,mrs);
	} else {
	    g.drawRect(ulx, uly, ws, hs);
	}	    
	((Graphics2D)g).setStroke(olds);
    }
    
    /** calculate the internal geometry */
    public void calcGeom() {
  	double coeff;
	coeff = Configuration.scale;
	geom.setRoundRect(deprecated_getX()-(w*coeff+plinewidth*coeff)/2,
			  deprecated_getY()-(h*coeff+plinewidth*coeff)/2,
			  w*coeff+plinewidth*coeff,
			  h*coeff+plinewidth*coeff,
			  mr*coeff*2,
			  mr*coeff*2);
	
    }

    public void setGeom(RoundRectangle2D.Double g) {
	geom = g;
    }
    /** Useful to draw the selection area
     * @return the Rectangle containing the Object
     */
    public Rectangle getBounds() {
	Rectangle rect= geom.getBounds();
	rect.grow(2,2);
	return rect;
    }

    /** Useful to select the object
     * test if the Object is containing a Rectangle
     * @param r The rectangle which is used for the test
     * @return true if the rectangle is contained the Object
    */
    public boolean isInsideRectangle(Rectangle r) {
	return geom.intersects(r.getX(),
				   r.getY(),
				   r.getWidth(),
				   r.getHeight());
    }

/** Fonction which is used to get the Gastex Code
     * associated with the JasObject
     * @param oriX the X coordinate of the origin
     * @param oriY the Y coordinate of the origin
     * @return the associated latex code
     */     
    public String getLatexCode(double oriX, double oriY) {
	String ovalCode = null;
	ovalCode = "\\drawoval"+getLatexCodeProperties(true)+"("+
	    roundDouble(deprecated_getULX()-oriX)+","+
 	    roundDouble(deprecated_getULY()-oriY)+","+
	    roundDouble(getWidth())+","+
	    roundDouble(getHeight())+","+
	    roundDouble(getMr())+")";
	return ovalCode;
    }

     /** Useful to select the object */
    public double getMinDistance(int gx,int gy) {
	double coeff;
	coeff = Configuration.scale;
	// needed to be recalculate because it it slightly wider
	RoundRectangle2D.Double rr =
	    new RoundRectangle2D.Double(deprecated_getX()-(w*coeff+plinewidth*coeff)/2,
			  deprecated_getY()-(h*coeff+plinewidth*coeff)/2,
			  w*coeff+plinewidth*coeff,
			  h*coeff+plinewidth*coeff,
			  mr*coeff*2,
			  mr*coeff*2);
	if (rr.contains(gx,gy)) return 0;
	return 1000;
    }

    /** Useful to Select the object
     * Tell if the Object contains the point P(gx,gy)
     * used by Node and RMark
     * need to be override
     * @param gx the x coordinates of the point to test
     * @param yx the y coordinates of the point to test
     * @return true if the Object contains the points
     */
    public boolean contains(double gx, double gy) {	    
	return geom.contains(gx, gy);
    }

   /** Move the Object
     * @param x the new x position of the object
     * @param y the new y position of the object
     */
    public void setPos(int posx, int posy) {
	deprecated_x=posx;
	deprecated_y=posy;
 	calcGeom();
    }

    public void moveIt(Point pt) {
	super.moveIt(pt);
    }


    public double getWidth() {
	return (globalProp.internal2UnitLength(w));
    }
    public void setWidth(double wi) {
	w = globalProp.unitLength2Internal(wi);
    }
    public double getHeight() {
	return (globalProp.internal2UnitLength(h));
    }
    public void setHeight(double he) {
	h = globalProp.unitLength2Internal(he);
    }
    public double getMr() {
	return (globalProp.internal2UnitLength(mr));
    }
    public void setMr(double r) {
	mr = globalProp.unitLength2Internal(r);
    }


    public double getInternalWidth() { return w; }
    public void setInternalWidth(double wi) { w=wi; }
    public double getInternalHeight() { return h; }
    public void setInternalHeight(double he) { h=he; }
    public double getInternalMr() { return mr; }
    public void setInternalMr(double r) { mr=r; }
    public double getRatioMr() {
	double maxMr=Math.min(w,h);
	maxMr=maxMr/2;
	return (mr/maxMr);
    }
    public void setRatioMr(double ratio) {
	double maxMr=Math.min(w,h);
	maxMr=maxMr/2;
	setInternalMr(ratio*maxMr);
    }





   /** Callback to update the ToolBox when the object move
     *  Called by SurfaceDessin
     *  @param tooBox the ToolBox to Warn
     *  @param manager the ObjectManager
     */
    public void updatingWhenMove(JasObjectToolBox tooBox,ObjetManager manager){
	tooBox.updateNodeX(deprecated_getXTB());
	tooBox.updateNodeY(deprecated_getYTB());
	tooBox.updateNodeW(getWidth());
	tooBox.updateNodeH(getHeight());
	tooBox.updateNodeNmr(getMr());
    }    

    /** create a new instance of the same object
     * adjust the node internal name and all the vectors
     * @return  a new instance of the same object
    */
    public Object clone(){
	Oval o = null;
	o = (Oval) super.clone();
	if (o == null) return null;
	o.setGeom((RoundRectangle2D.Double) geom.clone());
	return o;
    }
    public Selection getObjectSelection(Rectangle slct){
	return (new SelectOval(this));
    }
}
