package jastex.parsing;
import java_cup.runtime.Symbol;

%%

%{
    private boolean ignore_bloc = false;
    private int bracket_level = 0;
    private int bloc_brace_level =0;
    private int brace_level=0;
    private int parent_level=0;
    private String chaine;
    private String command;
    private boolean var_isCommandSet=false;
    public static boolean _VERBOSE_ = false;
    String jas_err="";
    int markedindex=0;

    private void setCommand(String s){
	command=s;
	var_isCommandSet=true;
    }

    private boolean isCommandSet(){
	return var_isCommandSet;
    } 

    public int getline(){ return yyline; }
    void adderr(String s){ jas_err+=s+"\n"; }
    String geterr(){ return jas_err; }

%} 

%eofval{
    return(new Symbol(sym.EOF));
%eofval}

%cup
%line
%char
%state COMMENT BLOC GASTEXP
%type java_cup.runtime.Symbol


DIGIT          = [0-9]
LETTER         = [A-Za-z]
NUMBER         = (-?{DIGIT}+([\.]{DIGIT}*)?|-?\.{DIGIT}+)
SEP            = [\t\b\r \012]
COMMENT_TEXT   = [^\n\r\n]+
N_L            = \r?\n
LATEXBLOC      = [^\}\{]+
WORD           = (\_|{LETTER})+(\_|{LETTER}|{DIGIT})*
STARTCMD       = \\
LATEXBLOC2     = [^\(\)]+
LATEXBLOC3     = [^\[\]]+

%%

<YYINITIAL> "%"        { yybegin(COMMENT); }
<YYINITIAL> {STARTCMD} { markedindex=yy_buffer_index; brace_level=0; bracket_level=0; parent_level=0; var_isCommandSet=false; yybegin(GASTEXP); }

<YYINITIAL> "{"        { if(_VERBOSE_) System.out.println("lexer "+yyline+" : GPLBRACE"); return(new Symbol(sym.GPLBRACE)); }
<YYINITIAL> "}"        { if(_VERBOSE_) System.out.println("lexer "+yyline+" : GPRBRACE"); return(new Symbol(sym.GPRBRACE)); }
<YYINITIAL> {N_L}      { }
<YYINITIAL> {SEP}+     { }
<YYINITIAL> .          { /* ignoring */ }


<COMMENT>   {COMMENT_TEXT} { if(_VERBOSE_) System.out.println("lexer "+yyline+" : Commentaire corps");}
<COMMENT>   {N_L}          { 
  if(_VERBOSE_) System.out.println("lexer "+yyline+" : Commentaire fin");
  yybegin(YYINITIAL);
}


<BLOC> {LATEXBLOC} {
    chaine+=yytext();
    //System.out.println("lexer "+yyline+" : BLOC : "+chaine); 
    //return new Symbol(sym.BLOC, new String(yytext()));
}
<BLOC> "{" { bloc_brace_level++; chaine+=yytext(); /*return(new Symbol(sym.LBRACEBIS));*/}
<BLOC> "}" { 
    if(bloc_brace_level!=0){
	bloc_brace_level--; chaine+=yytext();
	//System.out.println("lexer "+yyline+" : RBRACEBIS");
	//return(new Symbol(sym.RBRACEBIS));
    } else {
	bloc_brace_level=0;
	ignore_bloc=false;
	yybegin(YYINITIAL); //GASTEXP
	if(_VERBOSE_)  System.out.println("lexer "+yyline+" : BLOC : "+chaine);
	return new Symbol(sym.BLOC, new String(chaine));	
    } 
}



<COMMENT,BLOC>   . { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : ERROR: Illegal character: <" + yytext() + "> in line: "+yyline );
 adderr("Warning line "+yyline+" : Illegal character <"+ yytext()+ ">"); 
}

<GASTEXP> "nullfont" { }


<GASTEXP> "gasset" { setCommand(yytext()); if(_VERBOSE_)  System.out.println("lexer "+yyline+" : GASSET"); return(new Symbol(sym.GASSET));}
<GASTEXP> "imark" { setCommand(yytext()); if(_VERBOSE_)  System.out.println("lexer "+yyline+" : IMARK"); return(new Symbol(sym.IMARK));}
<GASTEXP> "fmark" { setCommand(yytext()); if(_VERBOSE_)  System.out.println("lexer "+yyline+" : FMARK"); return(new Symbol(sym.FMARK));}
<GASTEXP> "rmark" { setCommand(yytext()); if(_VERBOSE_)  System.out.println("lexer "+yyline+" : RMARK"); return(new Symbol(sym.RMARK));}
<GASTEXP> "nodelabel" { setCommand(yytext()); ignore_bloc=true; if(_VERBOSE_)  System.out.println("lexer "+yyline+" : NODELABEL"); return(new Symbol(sym.NODELABEL));}
<GASTEXP> "drawedge" { setCommand(yytext()); ignore_bloc=true; if(_VERBOSE_)  System.out.println("lexer "+yyline+" : DRAWEDGE"); return(new Symbol(sym.DRAWEDGE));}
<GASTEXP> "drawqbedge" { setCommand(yytext()); ignore_bloc=true; if(_VERBOSE_)  System.out.println("lexer "+yyline+" : DRAWQBEDGE"); return(new Symbol(sym.DRAWQBEDGE));}
<GASTEXP> "drawqbpedge" { setCommand(yytext()); ignore_bloc=true; if(_VERBOSE_)  System.out.println("lexer "+yyline+" : DRAWQBPEDGE"); return(new Symbol(sym.DRAWQBPEDGE));}
<GASTEXP> "drawbpedge" { setCommand(yytext()); ignore_bloc=true; if(_VERBOSE_)  System.out.println("lexer "+yyline+" : DRAWBPEDGE"); return(new Symbol(sym.DRAWBPEDGE));}
<GASTEXP> "drawbcedge" { setCommand(yytext()); ignore_bloc=true; if(_VERBOSE_)  System.out.println("lexer "+yyline+" : DRAWBCEDGE"); return(new Symbol(sym.DRAWBCEDGE));}
<GASTEXP> "drawloop" { setCommand(yytext()); ignore_bloc=true; if(_VERBOSE_)  System.out.println("lexer "+yyline+" : DRAWLOOP"); return(new Symbol(sym.DRAWLOOP));}
<GASTEXP> "drawcircle" { setCommand(yytext()); if(_VERBOSE_)  System.out.println("lexer "+yyline+" : DRAWCIRCLE"); return(new Symbol(sym.DRAWCIRCLE));}
<GASTEXP> "drawrect" { setCommand(yytext()); if(_VERBOSE_)  System.out.println("lexer "+yyline+" : DRAWRECT"); return(new Symbol(sym.DRAWRECT));}
<GASTEXP> "drawoval" { setCommand(yytext()); if(_VERBOSE_)  System.out.println("lexer "+yyline+" : DRAWOVAL"); return(new Symbol(sym.DRAWOVAL));}
<GASTEXP> "drawline" { setCommand(yytext()); if(_VERBOSE_)  System.out.println("lexer "+yyline+" : DRAWLINE"); return(new Symbol(sym.DRAWLINE));}
<GASTEXP> "drawqbezier" { setCommand(yytext()); if(_VERBOSE_)  System.out.println("lexer "+yyline+" : DRAWQBEZIER"); return(new Symbol(sym.DRAWQBEZIER));}
<GASTEXP> "drawcbezier" { setCommand(yytext()); if(_VERBOSE_)  System.out.println("lexer "+yyline+" : DRAWCBEZIER"); return(new Symbol(sym.DRAWCBEZIER));}
<GASTEXP> "end{picture}" { setCommand(yytext()); if(_VERBOSE_)  System.out.println("lexer "+yyline+" : ENDPIC"); return(new Symbol(sym.ENDPIC));}
<GASTEXP> "begin{picture}" { setCommand(yytext()); if(_VERBOSE_)  System.out.println("lexer "+yyline+" : BEGINPIC"); return(new Symbol(sym.BEGINPIC));}
<GASTEXP> "node" { setCommand(yytext()); ignore_bloc=true; if(_VERBOSE_)  System.out.println("lexer "+yyline+" : NODE"); return(new Symbol(sym.NODE));}

<GASTEXP> "end"  { /*if(_VERBOSE_)  System.out.println("lexer "+yyline+" : END");*/ yybegin(YYINITIAL); }
<GASTEXP> "begin"  { /*if(_VERBOSE_)  System.out.println("lexer "+yyline+" : BEGIN");*/ yybegin(YYINITIAL); }
<GASTEXP> "put" { /*if(_VERBOSE_)  System.out.println("lexer "+yyline+" : PUT");*/ yybegin(YYINITIAL); }
<GASTEXP> "framebox" {/* if(_VERBOSE_)  System.out.println("lexer "+yyline+" : FRAMEBOX");*/ yybegin(YYINITIAL); }
<GASTEXP> "documentclass" { /*if(_VERBOSE_)  System.out.println("lexer "+yyline+" : DOCUMENTCLASS");*/ yybegin(YYINITIAL); }
<GASTEXP> "usepackage" { setCommand(yytext()); if(_VERBOSE_)  System.out.println("lexer "+yyline+" : USEPACKAGE"); return(new Symbol(sym.USEPACKAGE)); }

<GASTEXP> "Nw" {  if(_VERBOSE_)  System.out.println("lexer "+yyline+" : NW"); return(new Symbol(sym.NW)); }
<GASTEXP> "Nh" {  if(_VERBOSE_)  System.out.println("lexer "+yyline+" : NH"); return(new Symbol(sym.NH)); }
<GASTEXP> "Nmr" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : NMR"); return(new Symbol(sym.NMR)); }
<GASTEXP> "Nadjust" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : NADJUST"); return(new Symbol(sym.NADJUST)); }
<GASTEXP> "Nadjustdist" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : NADJUSTDIST"); return(new Symbol(sym.NADJUSTDIST)); }
<GASTEXP> "Nframe" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : NFRAME"); return(new Symbol(sym.NFRAME)); }
<GASTEXP> "Nfill" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : NFILL"); return(new Symbol(sym.NFILL)); }
<GASTEXP> "ExtNL" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : EXTNL"); return(new Symbol(sym.EXTNL)); }
<GASTEXP> "NLangle" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : NLANGLE"); return(new Symbol(sym.NLANGLE)); }
<GASTEXP> "NLdist" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : NLDIST"); return(new Symbol(sym.NLDIST)); }
<GASTEXP> "Nmarks" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : NMARKS"); return(new Symbol(sym.NMARKS)); }
<GASTEXP> "ilength" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : ILENGTH"); return(new Symbol(sym.ILENGTH)); }
<GASTEXP> "iangle" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : IANGLE"); return(new Symbol(sym.IANGLE)); }
<GASTEXP> "flength" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : FLENGTH"); return(new Symbol(sym.FLENGTH)); }
<GASTEXP> "fangle" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : FANGLE"); return(new Symbol(sym.FANGLE)); }
<GASTEXP> "rdist" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : RDIST"); return(new Symbol(sym.RDIST)); }
<GASTEXP> "loopdiam" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : LOOPDIAM"); return(new Symbol(sym.LOOPDIAM)); }
<GASTEXP> "loopangle" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : LOOPANGLE"); return(new Symbol(sym.LOOPANGLE)); }
<GASTEXP> "loopCW" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : LOOPCW"); return(new Symbol(sym.LOOPCW)); }
<GASTEXP> "curvedepth" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : CURVEDEPTH"); return(new Symbol(sym.CURVEDEPTH)); }
<GASTEXP> "AHnb" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : AHNB"); return(new Symbol(sym.AHNB)); }
<GASTEXP> "AHdist" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : AHDIST"); return(new Symbol(sym.AHDIST)); }
<GASTEXP> "AHangle" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : AHANGLE"); return(new Symbol(sym.AHANGLE)); }
<GASTEXP> "AHLength" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : AHLLENGTH"); return(new Symbol(sym.AHLLENGTH)); }
<GASTEXP> "AHlength" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : AHLENGTH"); return(new Symbol(sym.AHLENGTH)); }
<GASTEXP> "ELside" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : ELSIDE"); return(new Symbol(sym.ELSIDE)); }
<GASTEXP> "ELpos" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : ELPOS"); return(new Symbol(sym.ELPOS)); }
<GASTEXP> "ELdist" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : ELDIST"); return(new Symbol(sym.ELDIST)); }
<GASTEXP> "ELdistC" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : ELDISTC"); return(new Symbol(sym.ELDISTC)); }
<GASTEXP> "linegray" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : LINEGRAY"); return(new Symbol(sym.LINEGRAY)); }
<GASTEXP> "fillgray" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : FILLGRAY"); return(new Symbol(sym.FILLGRAY)); }
<GASTEXP> "linecolor" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : LINECOLOR"); return(new Symbol(sym.LINECOLOR)); }
<GASTEXP> "fillcolor" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : FILLCOLOR"); return(new Symbol(sym.FILLCOLOR)); }
<GASTEXP> "linewidth" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : LINEWIDTH"); return(new Symbol(sym.LINEWIDTH)); }
<GASTEXP> "dash" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : DASH"); return(new Symbol(sym.DASH)); }
<GASTEXP> "sxo" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : SXO"); return(new Symbol(sym.SXO)); }
<GASTEXP> "syo" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : SYO"); return(new Symbol(sym.SYO)); }
<GASTEXP> "exo" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : EXO"); return(new Symbol(sym.EXO)); }
<GASTEXP> "eyo" { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : EYO"); return(new Symbol(sym.EYO)); }


<GASTEXP> {N_L}      { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : N_L"); if(bracket_level==0 && brace_level==0 && parent_level==0) yybegin(YYINITIAL);}
<GASTEXP> {SEP}+     { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : SEP"); if(bracket_level==0 && brace_level==0 && parent_level==0) yybegin(YYINITIAL);}
<GASTEXP> "%"        { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : SEP"); if(bracket_level==0 && brace_level==0 && parent_level==0) yybegin(COMMENT);}

<GASTEXP> {NUMBER} { 
    if(_VERBOSE_)  System.out.println("lexer "+yyline+" : NUMBER");
    return new Symbol(sym.NUMBER, new Double(yytext())); 
}
<GASTEXP> {WORD}   { 
    if(!isCommandSet()){ yybegin(YYINITIAL); }
    chaine=yytext();
    if(_VERBOSE_)  System.out.println("lexer "+yyline+" : WORD "+chaine);
    return new Symbol(sym.WORD, new String(yytext())); 
}

<GASTEXP> ","      { if(_VERBOSE_)  System.out.println("lexer "+yyline+" : COMMA"); return(new Symbol(sym.COMMA)); }
<GASTEXP> "("      { parent_level++; if(_VERBOSE_)  System.out.println("lexer "+yyline+" : LPAR"); return(new Symbol(sym.LPAR)); }
<GASTEXP> ")"      { parent_level--; if(_VERBOSE_)  System.out.println("lexer "+yyline+" : RPAR"); return(new Symbol(sym.RPAR)); }
<GASTEXP> "["      { bracket_level++; if(_VERBOSE_)  System.out.println("lexer "+yyline+" : LBRACKET"); return(new Symbol(sym.LBRACKET)); }
<GASTEXP> "]"      { bracket_level--; if(_VERBOSE_)  System.out.println("lexer "+yyline+" : RBRACKET"); return(new Symbol(sym.RBRACKET)); }
<GASTEXP> "{"      { 
    if(!(ignore_bloc && bracket_level==0)) {
	brace_level++;
	if(_VERBOSE_)  System.out.println("lexer "+yyline+" : LBRACE"); return(new Symbol(sym.LBRACE)); 	
    } else { chaine=""; yybegin(BLOC); }
}

<GASTEXP> "}"      { 
    brace_level--;
    if(_VERBOSE_)  System.out.println("lexer "+yyline+" : RBRACE"); 
    return(new Symbol(sym.RBRACE)); 
}
<GASTEXP> "="      { if(_VERBOSE_) System.out.println("lexer "+yyline+" : EGAL"); return(new Symbol(sym.EGAL)); }
<GASTEXP> {STARTCMD} { brace_level=0; bracket_level=0; parent_level=0; var_isCommandSet=false; yybegin(GASTEXP); }

<GASTEXP> . { System.out.println("lexer "+yyline+" : ERROR: Illegal character: <" + yytext() + "> in line: "+yyline ); 
 adderr("Warning line "+yyline+" : Illegal character <"+ yytext()+ ">");
}
